@@ -1,7 +1,7 @@
 /* buffer.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_buffer[] = "$Id: buffer.c,v 2.68 1996/09/26 01:05:21 steve Exp $";
+char id_buffer[] = "$Id: buffer.c,v 2.56 1996/06/03 17:23:24 steve Exp $";
 
 #include "elvis.h"
 
@@ -26,14 +26,8 @@ BUFFER buffers;
  */
 BUFFER bufdefault;
 
-/* This stores the message type that will be used for reporting the number
- * of lines read or written.  It is normally MSG_STATUS so that other messages
- * will be allowed to overwrite it; however, when quitting it is set to
- * MSG_INFO so messages will be queued and eventually displayed somewhere
- * else after the window is closed.  It is also set to MSG_INFO during
- * initialization so the "read..." message appears in the new window.
- */
-MSGIMP bufmsgtype = MSG_INFO;
+/* This counter is used for generating unique names for temp buffers */
+static int untitledctr = 1;
 
 /* This buffer's contents are irrelevent.  The values of its options, though,
  * are significant because the values of its options are used as the default 
@@ -47,8 +41,8 @@ static OPTDESC bdesc[] =
 {
 	{"filename", "file",	optsstring,	optisstring	},
 	{"bufname", "buffer",	optsstring,	optisstring	},
-	{"buflines", "bl",	optnstring,	optisnumber	},
-	{"bufchars", "bc",	optnstring,	optisnumber	},
+	{"buflines", "bufl",	optnstring,	optisnumber	},
+	{"bufchars", "bufc",	optnstring,	optisnumber	},
 	{"retain", "ret",	NULL,		NULL		},
 	{"modified", "mod",	NULL,		NULL,		},
 	{"edited", "samename",	NULL,		NULL,		},
@@ -58,17 +52,17 @@ static OPTDESC bdesc[] =
 	{"inputtab", "itab",	opt1string,	optisoneof,	"tab spaces filename"},
 	{"autotab", "at",	NULL,		NULL,		},
 	{"tabstop", "ts",	optnstring,	optisnumber,	"1:100"},
-	{"ccprg", "cp",		optsstring,	optisstring	},
-	{"equalprg", "ep",	optsstring,	optisstring	},
-	{"keywordprg", "kp",	optsstring,	optisstring	},
-	{"makeprg", "mp",	optsstring,	optisstring	},
+	{"ccprg", "cc",		optsstring,	optisstring	},
+	{"equalprg", "eq",	optsstring,	optisstring	},
+	{"keywordprg", "kw",	optsstring,	optisstring	},
+	{"makeprg", "make",	optsstring,	optisstring	},
 	{"paragraphs", "para",	optsstring,	optisstring	},
 	{"sections", "sect",	optsstring,	optisstring	},
 	{"shiftwidth", "sw",	optnstring,	optisnumber	},
-	{"undolevels", "ul",	optnstring,	optisnumber	},
-	{"textwidth", "tw",	optnstring,	optisnumber	},
+	{"undolevels", "undo",	optnstring,	optisnumber	},
+	{"linelength", "ll",	optnstring,	optisnumber	},
 	{"internal", "internal",NULL,		NULL		},
-	{"bufdisplay", "bd",    optsstring,	optisstring	},
+	{"bufdisplay", "bufdis",optsstring,	optisstring	},
 	{"errlines", "errlines",optnstring,	optisnumber	},
 	{"binary", "bin",	NULL,		NULL		}
 };
@@ -171,14 +165,12 @@ static void proc(bufinfo, nchars, nlines, changes, prevloc, name)
 			CHARncmp(name, toCHAR("Elvis untitled"), 14))
 		{
 			/* probably an internal buffer */
-			optpreset(o_internal(buf), True, OPT_HIDE);
-			optpreset(o_modified(buf), False, OPT_HIDE);
+			o_internal(buf) = True;
 		}
 		else
 		{
 			/* the filename is probably the same as the buffer name */
-			optpreset(o_filename(buf), CHARdup(name), OPT_FREE|OPT_HIDE);
-			optpreset(o_internal(buf), False, OPT_HIDE);
+			optpreset(o_filename(buf), CHARdup(name), OPT_FREE);
 
 			/* Mark it as readonly so the user will have to think
 			 * before clobbering an existing file.
@@ -188,7 +180,7 @@ static void proc(bufinfo, nchars, nlines, changes, prevloc, name)
 			/* Mark it as modified, so the user has to think
 			 * before exitting and losing this session file.
 			 */
-			optpreset(o_modified(buf), True, OPT_HIDE);
+			o_modified(buf) = True;
 		}
 		return;
 	}
@@ -272,14 +264,13 @@ BUFFER bufalloc(name, bufinfo)
 	BUFFER	buffer;
 	BUFFER	scan, lag;	/* used for inserting new buffer */
 	char	unique[255];	/* name of untitled buffer */
-	int	i = 1;		/* for generating a name for untitled buffer */
 
 	/* if no name was specified, generate a unique untitled name */
 	if (!name)
 	{
 		do
 		{
-			sprintf(unique, UNTITLED_BUF, i++);
+			sprintf(unique, UNTITLED_BUF, untitledctr++);
 		} while (buffind(toCHAR(unique)));
 		name = toCHAR(unique);
 	}
@@ -305,19 +296,19 @@ BUFFER bufalloc(name, bufinfo)
 	}
 
 	/* initialize the buffer options */
-	optpreset(o_readonly(buffer), o_defaultreadonly, OPT_HIDE);
+	o_readonly(buffer) = o_defaultreadonly;
 	if (bufdefopts)
 	{
 		/* copy all options except the following: filename bufname
 		 * buflines bufchars modified edited newfile internal autotab
 		 */
-		optpreset(buffer->retain, bufdefopts->retain, OPT_HIDE);
+		buffer->retain = bufdefopts->retain;
 		buffer->autoindent = bufdefopts->autoindent;
 		buffer->inputtab = bufdefopts->inputtab;
 		buffer->tabstop = bufdefopts->tabstop;
 		buffer->shiftwidth = bufdefopts->shiftwidth;
 		buffer->undolevels = bufdefopts->undolevels;
-		buffer->textwidth = bufdefopts->textwidth;
+		buffer->linelength = bufdefopts->linelength;
 		buffer->autotab = bufdefopts->autotab;
 
 		/* Strings are tricky, because we may need to allocate a
@@ -326,7 +317,6 @@ BUFFER bufalloc(name, bufinfo)
 		buffer->cc = bufdefopts->cc;
 		if (buffer->cc.flags & OPT_FREE)
 			o_cc(buffer) = CHARdup(o_cc(bufdefopts));
-		buffer->cc.flags |= OPT_HIDE;
 		buffer->equalprg = bufdefopts->equalprg;
 		if (buffer->equalprg.flags & OPT_FREE)
 			o_equalprg(buffer) = CHARdup(o_equalprg(bufdefopts));
@@ -336,7 +326,6 @@ BUFFER bufalloc(name, bufinfo)
 		buffer->make = bufdefopts->make;
 		if (buffer->make.flags & OPT_FREE)
 			o_make(buffer) = CHARdup(o_make(bufdefopts));
-		buffer->make.flags |= OPT_HIDE;
 		buffer->paragraphs = bufdefopts->paragraphs;
 		if (buffer->paragraphs.flags & OPT_FREE)
 			o_paragraphs(buffer) = CHARdup(o_paragraphs(bufdefopts));
@@ -364,12 +353,12 @@ BUFFER bufalloc(name, bufinfo)
 #else
 		o_make(buffer) = toCHAR("make $1");
 #endif
-		o_paragraphs(buffer) = toCHAR("PPppIPLPQPP");
+		o_paragraphs(buffer) = toCHAR("PPppIPLPQP");
 		o_sections(buffer) = toCHAR("NHSHSSSEse");
 		o_shiftwidth(buffer) = 8;
 		o_undolevels(buffer) = 0;
 		o_bufdisplay(buffer) = toCHAR("normal");
-		optflags(o_textwidth(buffer)) = OPT_REDRAW;
+		optflags(o_linelength(buffer)) = OPT_REDRAW;
 	}
 
 	/* set the name of this buffer, and limit access to some options */
@@ -453,10 +442,10 @@ BOOLEAN bufread(mark, rfile)
 	}
 	ioclose();
 	if (newbuf)
-		msg(bufmsgtype, "[sdd]read $1, $2 lines, $3 chars", rfile,
+		msg(MSG_INFO, "[sdd]read $1, $2 lines, $3 chars", rfile,
 					o_buflines(buf), o_bufchars(buf));
 	else if (!o_internal(buf))
-		msg(bufmsgtype, "[d]read $1 lines", o_buflines(buf) - origlines);
+		msg(MSG_INFO, "[d]read $1 lines", o_buflines(buf) - origlines);
 
 	return True;
 }
@@ -494,19 +483,15 @@ BUFFER bufload(bufname, filename, reload)
 			return buf;
 
 		/* Save the text as an "undo" version, and then delete it */
-		if (windefault && markbuffer(windefault->cursor) == buf)
-			bufwilldo(windefault->cursor);
-		else
-			bufwilldo(marktmp(top, buf, 0));
-		bufreplace(marktmp(top, buf, 0), marktmp(end, buf, o_bufchars(buf)), (CHAR *)0, 0);
+		bufwilldo(marktmp(top, buf, 0));
+		bufreplace(&top, marktmp(end, buf, o_bufchars(buf)), (CHAR *)0, 0);
 	}
 
 	/* Set the buffer's options */
 	optpreset(o_filename(buf), CHARkdup(filename), OPT_HIDE|OPT_LOCK|OPT_FREE);
-	optpreset(o_internal(buf), (bufname ? True : False), OPT_HIDE);
-	optpreset(o_edited(buf), True, OPT_HIDE);
-	o_readonly(buf) = False;
-	optpreset(o_newfile(buf), False, OPT_HIDE);
+	o_internal(buf) = (bufname ? True : False);
+	o_edited(buf) = True;
+	o_readonly(buf) = o_newfile(buf) = False;
 	switch (dirperm(filename))
 	{
 	  case DIR_INVALID:
@@ -549,7 +534,7 @@ BUFFER bufload(bufname, filename, reload)
 	/* read the file's contents into the buffer */
 	if (o_newfile(buf))
 	{
-		msg(bufmsgtype, "[sdd]$1 [NEW FILE]", filename);
+		msg(MSG_INFO, "[sdd]$1 [NEW FILE]", filename);
 	}
 	else if (!bufread(marktmp(top, buf, 0), filename))
 	{
@@ -559,7 +544,7 @@ BUFFER bufload(bufname, filename, reload)
 
 	/* set other options to describe the file */
 	o_modified(buf) = False;
-	optpreset(o_errlines(buf), o_buflines(buf), OPT_HIDE);
+	o_errlines(buf) = o_buflines(buf);
 
 	/* Restore the marks to their previous offsets.  Otherwise any marks
 	 * which refer to this buffer will be set to the end of the file.
@@ -852,7 +837,7 @@ BOOLEAN bufunload(buf, force, save)
 }
 
 
-/* Return True if "buf" can be deleted without loosing data, or False if it
+/* Return True if "buf" can be deleted without loosing data, of False if it
  * can't -- in which case it also emits a message describing why.  NOTE THAT
  * YOU ALSO NEED TO MAKE SURE THE BUFFER ISN'T BEING EDITED IN A WINDOW.
  *
@@ -987,7 +972,7 @@ BOOLEAN bufwrite(from, to, wfile, force)
 	/* Try to write the file */
 	if (ioopen(wfile, append ? 'a' : 'w', False, True, o_binary(markbuffer(from))))
 	{
-		if (wholebuf && !filter)
+		if (wholebuf)
 		{
 			msg(MSG_STATUS, "[s]writing $1", wfile);
 		}
@@ -1023,16 +1008,16 @@ BOOLEAN bufwrite(from, to, wfile, force)
 		ioclose();
 		scanfree(&cp);
 
-		if (!filter)
-		{
-			if (append)
-				msg(bufmsgtype, "[ds]appended $1 lines to $2",
+		if (filter)
+			msg(MSG_INFO, "[d]wrote $1 lines to filter",
+					markline(to) - markline(from));
+		else if (append)
+			msg(MSG_INFO, "[ds]appended $1 lines to $2",
 					markline(to) - markline(from), wfile);
-			else
-				msg(bufmsgtype, "[sdd]wrote $1, $2 lines, $3 chars",
+		else
+			msg(MSG_INFO, "[sdd]wrote $1, $2 lines, $3 chars",
 					wfile, markline(to) - markline(from),
 					markoffset(to) - markoffset(from));
-		}
 	}
 	else
 	{
@@ -1163,10 +1148,6 @@ static void bufdo(buf, wipe)
 
 	checkundo("before bufdo");
 
-	/* never save an undo version of an internal buffer */
-	if (o_internal(buf))
-		return;
-
 	/* allocate an undo structure */
 	undo = (struct undo_s *)safealloc(1, sizeof *undo);
 #ifdef DEBUG_ALLOC
@@ -1271,7 +1252,7 @@ static void bufdo(buf, wipe)
 	freeundo(buf);
 
 	/* make sure this is written to disk */
-	if (!o_internal(buf) && eventcounter > 2)
+	if (!o_internal(buf))
 		sessync();
 }
 
@@ -1396,7 +1377,6 @@ long bufundo(cursor, back)
 	o_buflines(buffer) = undo->buflines;
 	o_bufchars(buffer) = undo->bufchars;
 	buffer->changes++;
-	o_modified(buffer) = True;
 
 	/* Adjust the values of any marks.  Most marks can be fixed just by
 	 * calling markadjust(), but the named marks' old values are stored in
@@ -1454,10 +1434,10 @@ long bufundo(cursor, back)
  * "willdo" flag is set, then it will automatically create an "undo" version
  * of the buffer before making the change.
  */
-void bufreplace(from, to, newp, newlen)
+void bufreplace(from, to, new, newlen)
 	MARK	from;	/* starting position of old text */
 	MARK	to;	/* ending position of old text */
-	CHAR	*newp;	/* pointer to new text (in RAM) */
+	CHAR	*new;	/* pointer to new text (in RAM) */
 	long	newlen;	/* length of new text */
 {
 	long	chglines;
@@ -1485,7 +1465,7 @@ void bufreplace(from, to, newp, newlen)
 		}
 
 		/* we're inserting */
-		chglines = lowinsert(markbuffer(from)->bufinfo, markoffset(from), newp, newlen);
+		chglines = lowinsert(markbuffer(from)->bufinfo, markoffset(from), new, newlen);
 	}
 	else if (newlen == 0)
 	{
@@ -1495,7 +1475,7 @@ void bufreplace(from, to, newp, newlen)
 	else
 	{
 		/* we're replacing */
-		chglines = lowreplace(markbuffer(from)->bufinfo, markoffset(from), markoffset(to), newp, newlen);
+		chglines = lowreplace(markbuffer(from)->bufinfo, markoffset(from), markoffset(to), new, newlen);
 	}
 
 	/* adjust the buffer totals */
