@@ -1,7 +1,7 @@
 /* calc.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_calc[] = "$Id: calc.c,v 2.34 1996/10/01 19:45:25 steve Exp $";
+char id_calc[] = "$Id: calc.c,v 2.30 1996/05/15 06:53:46 steve Exp $";
 
 #include "elvis.h"
 #ifdef TRY
@@ -238,10 +238,7 @@ static BOOLEAN func(name, arg)
 	CHAR	*arg;	/* the argument to the function */
 {
 	CHAR	*tmp;
-	char	*c;
 	int	i;
-	MARK	begin;
-	MARKBUF	end;
 
 	if (!CHARcmp(name, toCHAR("strlen")))
 	{
@@ -407,104 +404,6 @@ static BOOLEAN func(name, arg)
 		CHARcpy(name, dmsknown(tochar8(arg)) ? "true" : "false");
 		return True;
 	}
-	else if (!CHARcmp(name, toCHAR("current")))
-	{
-		/* The default return value is an empty string */
-		*name = '\0';
-
-		/* Other possible values depend on the arg */
-		switch (*arg)
-		{
-		  case 'l':	/* line number */
-			if (windefault)
-				sprintf(tochar8(name), "%ld", markline(windefault->cursor));
-			break;
-
-		  case 'c':	/* column number */
-			if (windefault)
-				sprintf(tochar8(name), "%ld", (*windefault->md->mark2col)(windefault, windefault->cursor, viiscmd(windefault)) + 1);
-			break;
-
-		  case 'w':	/* word at cursor */
-		  	if (windefault)
-		  	{
-				end = *windefault->cursor;
-				begin = wordatcursor(&end);
-				if (begin)
-				{
-					scanalloc(&tmp, begin);
-					for (i = (int)(markoffset(&end) - markoffset(begin)); i > 0; i--)
-					{
-						*name++ = *tmp;
-						scannext(&tmp);
-					}
-					scanfree(&tmp);
-					*name = '\0';
-				}
-			}
-			break;
-
-		  case 'm':	/* mode */
-		  	if (windefault && !windefault->state->acton)
-		  	{
-				for (c = windefault->state->modename; *c; c++)
-				{
-					if (*c != ' ')
-						*name++ = tolower(*c);
-				}
-			}
-			break;
-
-		  case 's':	/* visible selection */
-		  	if (windefault && !windefault->state->acton && windefault->seltop)
-		  	{
-		  		switch (windefault->seltype)
-		  		{
-		  		  case 'c':
-		  			CHARcpy(name, toCHAR("character"));
-		  			break;
-
-		  		  case 'r':
-		  			CHARcpy(name, toCHAR("rectangle"));
-		  			break;
-
-		  		  default:
-		  		  	CHARcpy(name, toCHAR("line"));
-		  		}
-		  	}
-		  	break;
-
-		  case 'n':	/* next arg */
-			if (arglist[argnext])
-			{
-				for (c = arglist[argnext]; *c; )
-				{
-					*name++ = *c++;
-				}
-				*name = '\0';
-			}
-			break;
-
-		  case 'p':	/* previous arg */
-			if (argnext >= 2)
-			{
-				for (c = arglist[argnext - 2]; *c; )
-				{
-					*name++ = *c++;
-				}
-				*name = '\0';
-			}
-			break;
-
-		  case 't':	/* tagstack */
-			if (windefault && windefault->tagstack->origin)
-			{
-				CHARcpy(name, o_bufname(markbuffer(windefault->tagstack->origin)));
-			}
-			break;
-		}
-		return True;
-	}
 #endif
 
 #ifdef TRY
@@ -636,15 +535,6 @@ static BOOLEAN apply()
 				}
 				msg(MSG_WARNING, "<< and >> only partially implemented");
 			}
-			else if (subcode == '/')
-			{
-				/* When the / operator is passed strings as
-				 * arguments, it contatenates them as a
-				 * directory name and a file name.
-				 */
-				CHARcpy(first, toCHAR(dirpath(tochar8(first), tochar8(second))));
-				break;
-			}
 			second[-1] = subcode;
 		}
 		else
@@ -834,7 +724,7 @@ CHAR *calculate(expr, arg, asmsg)
 
 		  case '\\':
 			/* any single character following a \ is copied */
-			if (!*++expr || isalnum(*expr) || *expr == '*' || *expr == '?' || *expr == '.')
+			if (!*++expr || isalnum(*expr) || *expr == '*')
 			{
 				*build++ = '\\';
 			}
