@@ -1,6 +1,6 @@
 /* ctags.c */
 
-char id_ctags[] = "$Id: ctags.c,v 2.8 1996/09/21 02:12:31 steve Exp $";
+char id_ctags[] = "$Id: ctags.c,v 2.5 1996/05/07 17:56:07 steve Exp $";
 
 /* This is a reimplementation of the ctags(1) program.  It supports ANSI C,
  * and has heaps o' flags.  It is meant to be distributed with elvis.
@@ -63,15 +63,6 @@ extern void	usage P_((void));
 extern void	main P_((int, char **));
 
 
-#if defined (GUI_WIN32)
-extern void set_current_file (char *file_name);
-extern void set_current_tags (int num_tags);
-extern void set_total_tags (int num_tags);
-
-static int  num_tags = 0;
-static int  total_tags = 0;
-#endif
-
 /* -------------------------------------------------------------------------- */
 /* Some global variables */
 
@@ -700,11 +691,6 @@ void maketag(scope, name, lnum, seek, number)
 
 	if (make_tags)
 	{
-#if defined (GUI_WIN32)
-		set_current_tags (++num_tags);
-		set_total_tags (++total_tags);
-#endif
-
 		fprintf(tags, "%s\t%s\t", name, file_name);
 
 		if (number)
@@ -743,12 +729,6 @@ void ctags(name)
 	int	gotname;/* boolean: does lex_name contain a tag candidate? */
 	long	tagseek;/* start of line that contains lex_name */
 
-#if defined (GUI_WIN32)
-    set_current_file (name);
-    set_current_tags (0);
-    num_tags = 0;
-#endif
-
 	/* open the file */
 	cpp_open(name);
 
@@ -879,7 +859,6 @@ void usage()
 	fprintf(stderr, "\t-x      Write cross-reference table to stdout; skip \"tags\"\n");
 	fprintf(stderr, "\t-r      Write a \"refs\" file, in addition to \"tags\"\n");
 	fprintf(stderr, "\t-a      Append to \"tags\", instead of overwriting\n");
-	fprintf(stderr, "Report bugs to kirkenda@cs.pdx.edu\n");
 	exit(2);
 }
 
@@ -895,41 +874,6 @@ void main(argc, argv)
 #endif
 
 
-#if defined (GUI_WIN32)
-    set_current_file ("");
-    set_current_tags (0);
-    set_total_tags (0);
-#endif
-
-	/* detect special GNU flags */
-	if (argc >= 2)
-	{
-		if (!strcmp(argv[1], "-V") /* oops, lowercase -v already used */
-		 || !strcmp(argv[1], "-version")
-		 || !strcmp(argv[1], "--version"))
-		{
-			printf("ctags (elvis) %s\n", VERSION);
-#ifdef COPY1
-			puts(COPY1);
-#endif
-#ifdef COPY2
-			puts(COPY2);
-#endif
-#ifdef COPY3
-			puts(COPY3);
-#endif
-#ifdef COPY4
-			puts(COPY4);
-#endif
-#ifdef GUI_WIN32
-# ifdef PORTEDBY
-			puts(PORTEDBY);
-# endif
-#endif
-			exit(0);
-		}
-	}
-
 	/* parse the option flags */
 	for (i = 1; i < argc && argv[i][0] == '-'; i++)
 	{
