@@ -1,7 +1,7 @@
 /* dmmarkup.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_dmmarkup[] = "$Id: dmmarkup.c,v 2.43 1996/09/18 16:42:14 steve Exp $";
+char id_dmmarkup[] = "$Id: dmmarkup.c,v 2.35 1996/06/03 17:25:16 steve Exp $";
 
 /* This file contains some fairly generic text formatting code -- generic
  * in the sense that it can be easily tweaked to format a variety of types
@@ -46,15 +46,15 @@ typedef struct
 	long	 offset;	/* offset of start of line */
 	struct
 	{
-	    unsigned indent : 8;  /* indentation amount, in spaces */
-	    unsigned listcnt : 8; /* Counter for nest#1 numbered list; 0=not numbered */
-	    unsigned nest : 6;	  /* nesting level of list/menu; 0=not in list*/
 	    unsigned prefmt : 1;  /* 1=literal whitespace, 0=fill */
 	    unsigned graphic : 1; /* 1=replace |-^. with graphic chars */
 	    unsigned midline: 1;  /* 1=after a newline, 0=after other char */
 	    unsigned reduce : 1;  /* 1=fewer newlines, 0=normal qty newlines */
 	    unsigned deffont : 3; /* index into "nbiufe" of default font char */
 	    unsigned curfont : 3; /* index into "nbiufe" of current font char */
+	    unsigned nest : 6;	  /* nesting level of list/menu; 0=not in list*/
+	    unsigned indent : 8;  /* indentation amount, in spaces */
+	    unsigned listcnt : 8; /* Counter for nest#1 numbered list; 0=not numbered */
 	}	state;
 } LINEINFO;
 
@@ -73,7 +73,7 @@ static BOOLEAN	first;	/* is this the first token on this line? */
 static BOOLEAN	anyspc;	/* has whitespace been encountered? */
 static BOOLEAN	title;	/* collecting characters of the title */
 static BOOLEAN	list;	/* o_list */
-static int	textwidth;/* o_columns */
+static int	linelength;/* o_columns */
 static int	tabstop;/* o_tabstop */
 static int	listind;/* o_shiftwidth/2, or 2 if shiftwidth<=4 */
 static int	col;	/* logical column number */
@@ -164,7 +164,7 @@ static CHAR	bullet = '*';
 /*----------------------------------------------------------------------------*/
 /* HTML-specific functions and variables                                      */
 
-/* Replace entities such as &lt; with their single-character equivelent. */
+/* Replace escapes such as &lt; with their single-character equivelent. */
 static void htmlescape(token)
 	TOKEN	*token;	/* a token whose text is to be expanded */
 {
@@ -215,6 +215,8 @@ static void htmlescape(token)
 			*dst++ = '"';
 		else if (len == 6 && (!strncmp(&src[1], "nbsp", 4) || !strncmp(&src[1], "NBSP", 4)))
 			*dst++ = ' ';
+		else if (len == 6 && (!strncmp(&src[1], "copy", 4) || !strncmp(&src[1], "COPY", 4)))
+			*dst++ = digraph('O', 'C');
 		else if (len == 7 && !strncmp(&src[1], "AElig", 5))
 			*dst++ = digraph('E', 'A');
 		else if (len == 7 && !strncmp(&src[1], "aelig", 5))
@@ -247,17 +249,6 @@ static void htmlescape(token)
 			*dst++ = digraph('/', (_CHAR_)src[1]);
 		else if (src[1] == '#')
 			*dst++ = atoi(&src[2]);
-		else if (len == 6 && (!strncmp(&src[1], "copy", 4) || !strncmp(&src[1], "COPY", 4)))
-			*dst++ = digraph('O', 'c');
-		else if (len == 5 && (!strncmp(&src[1], "reg", 3) || !strncmp(&src[1], "REG", 3)))
-			*dst++ = digraph('O', 'r');
-		else if (len == 7 && (!strncmp(&src[1], "ldquo", 5) || !strncmp(&src[1], "LDQUO", 5)
-				|| !strncmp(&src[1], "rdquo", 5) || !strncmp(&src[1], "RDQUO", 5)))
-			*dst++ = '"';
-		else if (len == 7 && (!strncmp(&src[1], "lsquo", 5) || !strncmp(&src[1], "LSQUO", 5)))
-			*dst++ = '`';
-		else if (len == 7 && (!strncmp(&src[1], "rsquo", 5) || !strncmp(&src[1], "RSQUO", 5)))
-			*dst++ = '\'';
 		else
 		{
 			/* Tweak the value of "width" so that after all of
@@ -298,24 +289,24 @@ static BOOLEAN htmlhr(token)
 	int	len;
 	long	offset;
 
-	assert(col == 0 && textwidth > 0);
+	assert(col == 0 && linelength > 0);
 
 	/* Normally the hrule is as wide as the line, but if the cursor is on
 	 * the "<hr>" token then we want to shrink it slightly so that "<hr>"
 	 * will fit on the same row of the window.  This allows the window to
 	 * be updated more efficiently.
 	 */
-	len = textwidth;
+	len = linelength;
 	if (list || (o_showmarkups && token->offset[0] <= cursoff && cursoff <= token->offset[token->nchars - 1]))
 	{
 		/* leave room for showing the <hr> tag */
-		len -= textwidth - token->nchars;
+		len -= linelength - token->nchars;
 		offset = -1;
 	}
 	else
 	{
 		/* make it as wide as the screen; the <hr> won't show */
-		len = textwidth;
+		len = linelength;
 		offset = token->offset[0];
 	}
 	if (len < 3)
@@ -455,7 +446,7 @@ static BOOLEAN htmlinput(token)
 
 	/* parse the arguments */
 	height = (token->text[1] == 't') ? 3 : 1;
-	width = vallen = validx = 0;
+	width = vallen = 0;
 	button = radio = False;
 	font = 'u';
 	for (i = 4; i < token->nchars; i++)
@@ -491,21 +482,8 @@ static BOOLEAN htmlinput(token)
 			i += 5;
 			if (token->text[i] == '"')
 				i++;
-			if (!CHARncmp(&token->text[i], toCHAR("checkbox"), 8)
-			 || !CHARncmp(&token->text[i], toCHAR("CHECKBOX"), 8))
-			{
-				/* CHECKBOX button */
-				button = radio = True;
-				i += 8;
-			}
-			else if (!CHARncmp(&token->text[i], toCHAR("hidden"), 6)
-			      || !CHARncmp(&token->text[i], toCHAR("HIDDEN"), 6))
-			{
-				/* HIDDEN field -- do nothing with it */
-				return False;
-			}
-			else if (!CHARncmp(&token->text[i], toCHAR("radio"), 5)
-			      || !CHARncmp(&token->text[i], toCHAR("RADIO"), 5))
+			if (!CHARncmp(&token->text[i], toCHAR("radio"), 5)
+			 || !CHARncmp(&token->text[i], toCHAR("RADIO"), 5))
 			{
 				/* RADIO button */
 				button = radio = True;
@@ -525,13 +503,7 @@ static BOOLEAN htmlinput(token)
 		}
 	}
 
-	/* Most buttons are always as wide as their value, but radio & checkbox
-	 * buttons only need to show a single character.
-	 */
-	if (radio)
-	{
-		vallen = 1;
-	}
+	/* buttons are always as wide as their value */
 	if (button)
 	{
 		width = vallen;
@@ -548,7 +520,7 @@ static BOOLEAN htmlinput(token)
 	}
 
 	/* will it fit on this line? */
-	if (!first && mycol + width > textwidth)
+	if (!first && mycol + width > linelength)
 	{
 		/* no it won't */
 		(*drawchar)(&newline, 1, 'n', -1);
@@ -680,8 +652,7 @@ static void htmlmarkup(token)
 	/* look it up in the table */
 	for (scan = tbl;
 	     scan->name &&
-		(strlen(scan->name) != (unsigned)len ||
-		    strncmp(scan->name, (char *)token->text + 1, (size_t)len));
+		(strlen(scan->name) != len || strncmp(scan->name, (char *)token->text + 1, (size_t)len));
 	     scan++)
 	{
 	}
@@ -713,12 +684,10 @@ static TOKEN *htmlget(refp)
 	if (rettok.text[0] == '<')
 	{
 		/* This is a markup.  Collect characters up to next '>' */
-		for (lower = True; *refp && **refp != '>'; offset++, scannext(refp))
+		for (lower = True;
+		     *refp && rettok.nchars < QTY(rettok.text) - 2 && **refp != '>';
+		     offset++, scannext(refp))
 		{
-			/* if token text is full, then skip this char */
-			if (rettok.nchars >= QTY(rettok.text) - 2)
-				continue;
-
 			/* Store the character.  This is a little complex
 			 * because we want to convert uppercase tags and
 			 * parameter names to lowercase, but leave parameter
@@ -948,7 +917,7 @@ static MARK htmltagload(tagname, from)
 	CHAR	*anchorname;	/* name of tag's anchor */
 	CHAR	*p;
 	TOKEN	*token;
-	int	i, j;
+	int	i;
 
 	/* separate the tagname into filename and anchorname */
 	filename = safealloc((int)CHARlen(tagname) + 1, sizeof(char));
@@ -1004,7 +973,6 @@ static MARK htmltagload(tagname, from)
 	{
 		i = (int)CHARlen(anchorname);
 		scanalloc(&p, &retmark);
-#if 0
 		for (token = htmlget(&p);
 		     token
 			&& (CHARncmp(token->text, toCHAR("<a name="), 8)
@@ -1014,41 +982,6 @@ static MARK htmltagload(tagname, from)
 		     token = htmlget(&p))
 		{
 		}
-#else
-		for (token = htmlget(&p); token; token = htmlget(&p))
-		{
-			/* ignore if not markup or not a possible target */
-			if (!token->markup || token->markup->DEST == '-')
-				continue;
-
-			/* scan for "id=" or "name=" */
-			for (j = 2; j < token->nchars; j++)
-			{
-				if (!CHARncmp(&token->text[j], toCHAR(" name="), 6))
-				{
-					j += 6;
-					break;
-				}
-				if (!CHARncmp(&token->text[j], toCHAR(" id="), 4))
-				{
-					j += 4;
-					break;
-				}
-			}
-			if (j >= token->nchars)
-				continue;
-
-			/* compare to sought name.  Beware of quotes */
-			if (token->text[j] == '"'
-			    ? !CHARncmp(&token->text[++j], anchorname, (size_t)i)
-				&& token->text[j + i] == '"'
-			    : !CHARncmp(&token->text[j], anchorname, (size_t)i)
-				&& (token->text[j + i] == '>' || token->text[j + i] == ' '))
-			{
-				break;
-			}
-		}
-#endif
 
 		/* skip to the following non-whitespace text token */
 		while (token && (token->markup || isspace(token->text[0])))
@@ -1140,12 +1073,12 @@ static void manescape(token)
 	for (i = j = 0; i < token->nchars; i++)
 	{
 		/* most characters are copied as-is */
-		if (token->text[i] != '\\')
+		if (token->text[i] != '\\' || i + 1 >= token->nchars)
 		{
 			token->text[j] = token->text[i];
 			token->offset[j++] = token->offset[i];
 		}
-		else if (i + 1 < token->nchars)
+		else
 		{
 			switch (token->text[++i])
 			{
@@ -1258,17 +1191,6 @@ static int manarg(token, start, font, spc)
 				manlen++;
 				break;
 
-			  case 's':
-				/* delete the \s and the number that follows */
-				if (token->text[start] == '+' || token->text[start] == '-')
-					start++;
-				do
-				{
-					start++;
-				} while (isdigit(token->text[start]));
-				start--;
-				break;
-
 			  default:
 				mantext[manlen] = token->text[start];
 				manoffset[manlen] = token->offset[start];
@@ -1312,7 +1234,7 @@ static BOOLEAN manput()
 	int	i, start;
 
 	/* if it won't fit, then output a newline instead */
-	if (!first && col + manlen > textwidth - listind)
+	if (!first && col + manlen > linelength - listind)
 	{
 		/* output a newline */
 		(*drawchar)(&newline, 1, 'n', anyspc ? spcoffset : -1);
@@ -1591,8 +1513,6 @@ static void manmarkup(token)
 		{ "sp",		"N1-----"			},
 		{ "nf",		"N0---N-"			},
 		{ "fi",		"N0---Y-"			},
-		{ "DS",		"N1---N-"			},
-		{ "DE",		"N0---Y-"			},
 		{ (char *)0,	"N------"			}
 	};
 	MARKUP	*scan;	/* used for scanning the tbl[] array */
@@ -1636,7 +1556,7 @@ static TOKEN *manget(refp)
 	scannext(refp);
 
 	/* If '.' at the start of a line, then token is a markup */
-	if ((rettok.text[0] == '.' || rettok.text[0] == '\'') && !midline)
+	if (rettok.text[0] == '.' && !midline)
 	{
 		/* This is a markup.  Collect characters up to next '\n' */
 		for (;
@@ -2048,7 +1968,7 @@ static BOOLEAN put(token)
 	/* if not a markup, and it won't fit on the current line, then output
 	 * a '\n' and return True so it'll appear on the next line.
 	 */
-	else if (col + token->width + 1 > textwidth - 4
+	else if (col + token->width + 1 > linelength - 4
 		&& !prefmt && col != 0 && !token->markup)
 	{
 		(*drawchar)(&newline, 1, 'n', anyspc ? spcoffset : -1);
@@ -2308,7 +2228,7 @@ static MARK move(w, from, linedelta, column, cmd)
 	{
 		outoffset = mui->line[i].offset;
 	}
-	else if (column >= textwidth)
+	else if (column >= linelength)
 	{
 		if (i < mui->nlines - 1)
 			outoffset = mui->line[i + 1].offset - 1;
@@ -2477,17 +2397,17 @@ static void header(w, pagenum, info, draw)
 	/* find the lengths of things */
 	slen = CHARlen(sides);
 	mlen = CHARlen(middle);
-	if (slen + 2 + mlen + 2 + slen > textwidth)
+	if (slen + 2 + mlen + 2 + slen > linelength)
 	{
-		mlen = textwidth - 4 - 2 * slen;
+		mlen = linelength - 4 - 2 * slen;
 		if (mlen <= 0)
 		{
 			mlen = CHARlen(middle);
-			slen = (textwidth - 4 - mlen) / 2;
+			slen = (linelength - 4 - mlen) / 2;
 		}
 	}
-	gap1 = (textwidth - mlen) / 2 - slen;
-	gap2 = textwidth - mlen - 2 * slen - gap1;
+	gap1 = (linelength - mlen) / 2 - slen;
+	gap2 = linelength - mlen - 2 * slen - gap1;
 
 	/* Output the parts of the headings */
 	(*draw)(sides, slen, 'n', 0);
@@ -2600,11 +2520,11 @@ static int start(win, from, draw)
 	title = False; /* nothing that causes a linebreak can appear in title */
 	list = o_list(win);
 #if 1
-	textwidth = o_columns(win);
+	linelength = o_columns(win);
 #else
-	textwidth = o_textwidth(markbuffer(win->cursor));
-	if (textwidth == 0)
-		textwidth = 80;
+	linelength = o_linelength(markbuffer(win->cursor));
+	if (linelength == 0)
+		linelength = 80;
 #endif
 	tabstop = o_tabstop(markbuffer(win->cursor));
 	listind = o_shiftwidth(markbuffer(win->cursor)) / 2;
