@@ -1,7 +1,7 @@
 /* dmnormal.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_dmnormal[] = "$Id: dmnormal.c,v 2.29 1996/09/21 01:21:36 steve Exp $";
+char id_dmnormal[] = "$Id: dmnormal.c,v 2.25 1996/05/27 07:56:20 steve Exp $";
 
 #include "elvis.h"
 
@@ -461,7 +461,7 @@ static MARK image(w, line, info, draw)
 	/* end the line */
 	if (o_list(w) && !w->state->acton && *cp == '\n')
 	{
-		(*draw)(toCHAR("$"), 1, 'n', offset);
+		(*draw)(toCHAR("$"), 1, 'n', -1);
 	}
 	(*draw)(cp ? cp : toCHAR("\n"), 1, 'n', offset);
 	if (cp)
@@ -517,7 +517,7 @@ static void indent(w, line, linedelta)
 		return;
 	}
 	to = *scanmark(&cp);
-	srcblank = (BOOLEAN)(*cp == '\n');
+	srcblank = (*cp == '\n');
 	scanfree(&cp);
 
 	if (markoffset(&from) != markoffset(&to))
@@ -593,7 +593,7 @@ static MARK tagload(tagname, from)
  static MARKBUF	retmark;	/* the return value */
 	BUFFER	buf;		/* the buffer containing the tag */
 	CHAR	tagline[1000];	/* a line from the tags file */
-	int	bytes;		/* number of bytes in I/O buffer */
+	int	bytes;		/* number of bytes in tagline */
 	char	*pathname;	/* full pathname of current "tags" file */
 	int	len;		/* significant length of tagname */
 	BOOLEAN	fulllen;	/* len is the full length of the tag */
@@ -644,7 +644,7 @@ static MARK tagload(tagname, from)
 			/* if the next line is incomplete, read some more text
 			 * from the tags file.
 			 */
-			if (!allnext || bytes <= len)
+			if (!allnext)
 			{
 				bytes += ioread(dst, (int)QTY(tagline) - bytes - 1);
 			}
@@ -678,7 +678,7 @@ Found:
 	/* if the filename isn't a full pathname, then assume it is in the
 	 * same directory as the "tags" file.
 	 */
-	if (!isalnum(*src))
+	if (isalnum(*src))
 	{
 		loadname = tochar8(src);
 	}
@@ -688,25 +688,9 @@ Found:
 	}
 
 	/* find a buffer containing the file, or load the file into a buffer */
-	buf = buffind(toCHAR(loadname));
+	buf = bufload(NULL, loadname, False);
 	if (!buf)
-	{
-		if (dirperm(loadname) == DIR_NEW)
-		{
-			msg(MSG_ERROR, "[s]$1 doesn't exist", loadname);
-			return NULL;
-		}
-		buf = bufload(NULL, loadname, False);
-		if (!buf)
-		{
-			/* bufload() already gave error message */
-			return NULL;
-		}
-	}
-	if (o_bufchars(buf) == 0)
-	{
-		goto NotFound;
-	}
+		return NULL;
 
 	/* locate the tag address field */
 	for (src = dst; src < &tagline[bytes] && isspace(*src); src++)
@@ -718,27 +702,23 @@ Found:
 	*dst = '\0';
 
 	/* convert the tag address into a line number */
-	scanstring(&dst, src);
+	scanstring(&src, src);
 	memset((char *)&xinfb, 0, sizeof xinfb);
 	(void)marktmp(xinfb.defaddr, buf, 0);
 	wasmagic = o_magic;
 	o_magic = False;
-	if (!exparseaddress(&dst, &xinfb))
+	if (!exparseaddress(&src, &xinfb))
 	{
-		scanfree(&dst);
+		scanfree(&src);
 		o_magic = wasmagic;
-		goto NotFound;
+		return NULL;
 	}
-	scanfree(&dst);
+	scanfree(&src);
 	o_magic = wasmagic;
-	(void)marktmp(retmark, buf, lowline(bufbufinfo(buf), xinfb.to));
-	exfree(&xinfb);
+	exfree(&xinfb); /* does not affect "to" */
 
-	return &retmark;
-
-NotFound:
-	msg(MSG_WARNING, "tag address out of date");
-	return marktmp(retmark, buf, 0L);
+	/* convert the line number to an offset, and return the mark there */
+	return marktmp(retmark, buf, lowline(bufbufinfo(buf), xinfb.to));
 }
 
 
