@@ -1,25 +1,26 @@
 /* dmsyntax.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_dmsyntax[] = "$Id: dmsyntax.c,v 2.24 1996/09/21 01:21:36 steve Exp $";
+char id_dmsyntax[] = "$Id: dmsyntax.c,v 2.11 1996/06/03 17:26:25 steve Exp $";
 
 #include "elvis.h"
 #ifdef DISPLAY_SYNTAX
 
 #if USE_PROTOTYPES
-static CHAR *iskeyword(CHAR *word);
-static void addkeyword(CHAR *word, _char_ font, BOOLEAN doesregexp, BOOLEAN doesregsub);
+static BOOLEAN iskeyword(CHAR *word);
 static CHAR **fetchline(void);
 static DMINFO *init(WINDOW win);
 static void term(DMINFO *info);
 static MARK setup(MARK top, long cursor, MARK bottom, DMINFO *info);
 static MARK image(WINDOW w, MARK line, DMINFO *info, void (*draw)(CHAR *p, long qty, _char_ font, long offset));
-static CHAR *tagatcursor(WINDOW win, MARK cursor);
-static MARK tagload(CHAR *tagname, MARK from);
 #endif
 
+#ifndef DISPLAY_C
 /* These are the font names, used in the option descriptions below */
-char fontnames[] = "normal bold emphasized italic underlined fixed";
+char fontnames[] = "normal bold emphasized italics underlined fixed";
+#else
+extern char fontnames[];
+#endif
 
 /* These are the descriptions and values of some global options */
 static OPTDESC globdesc[] =
@@ -31,7 +32,6 @@ static OPTDESC globdesc[] =
 	{"variablefont","vfont",opt1string,	optisoneof,	fontnames },
 	{"prepfont", "pfont",	opt1string,	optisoneof,	fontnames },
 	{"otherfont", "ofont",	opt1string,	optisoneof,	fontnames },
-	{"includepath", "inc",	optsstring,	optisstring }
 };
 static OPTVAL globval[QTY(globdesc)];
 #define o_commentfont	globval[0].value.character
@@ -41,19 +41,13 @@ static OPTVAL globval[QTY(globdesc)];
 #define o_variablefont	globval[4].value.character
 #define o_prepfont	globval[5].value.character
 #define o_otherfont	globval[6].value.character
-#define o_includepath	globval[7].value.string
 
 /* This data type is used to denote a token type.  Values of this type will
  * be used as indicies into the cfont[] array, below, to determine which
  * font each language element should use.  The last symbol in the list must
  * be PUNCT, because the declaration of cfont[] depends on this.
  */
-typedef enum
-{
-	COMMENT, COMMENT2, STRING, CHARACTER, REGEXP, REGSUB, KEYWORD,
-	FIRSTPUNCT, SECONDPUNCT, FUNCTION, VARIABLE, PREP, PREPWORD, PREPQUOTE,
-	OTHER, PUNCT
-} TOKENTYPE;
+typedef enum { COMMENT, COMMENT2, STRING, CHARACTER, KEYWORD, FUNCTION, VARIABLE, PREP, PREPWORD, OTHER, PUNCT } TOKENTYPE;
 
 /* This structure stores information about the current language's syntax.
  * Each window has its own copy of this, so different windows which happen
@@ -65,15 +59,14 @@ typedef struct
 	TOKENTYPE token;	/* used during parsing */
 
 	/* info from the "elvis.syn" file */
-	CHAR	**keyword[256];	/* hash table of keyword names */
+	int	nkeywords;	/* number of keywords in the keyword array */
+	CHAR	*keyword[100];	/* array of keyword names */
 	CHAR	function;	/* character used for function calls */
 	CHAR	strbegin;	/* string start-quote character */
 	CHAR	strend;		/* string end-quote character */
 	CHAR	charbegin;	/* character quote character */
 	CHAR	charend;	/* character quote character */
 	CHAR	preprocessor;	/* first character of preprocessor directives */
-	CHAR	pqbegin;	/* preprocessor start-quote character */
-	CHAR	pqend;		/* preprocessor start-quote character */
 	CHAR	comment[2];	/* start of one-line comment */
 	CHAR	combegin[2];	/* start of multi-line comment */
 	CHAR	comend[2];	/* end of multi-line comment */
@@ -85,21 +78,10 @@ typedef struct
 	BOOLEAN	ignorecase;	/* keywords may be uppercase */
 	char	wordbits[256];	/* which chars can appear in a word */
 } SINFO;
-#define STARTWORD		0x01	/* can start a word */
-#define INWORD			0x02	/* can occur within a word */
-#define PUNCTWORD		0x04	/* may be first punct of 2-punct word */
-#define DELIMREGEXP		0x08	/* can delimit regular expression */
-#define USEREGEXP		0x10	/* occurs before possible regexp */
-#define USEREGSUB		0x20	/* occurs before possible regsub */
+#define STARTWORD		0x01
+#define INWORD			0x02
 #define isstartword(si,c)	((si)->wordbits[(CHAR)(c)] & STARTWORD)
 #define isinword(si,c)		((si)->wordbits[(CHAR)(c)] & INWORD)
-#define ispunctword(si,c)	((si)->wordbits[(CHAR)(c)] & PUNCTWORD)
-#define isregexp(si,c)		((si)->wordbits[(CHAR)(c)] & DELIMREGEXP)
-#define isbeforeregexp(si,c)	((si)->wordbits[(CHAR)(c)] & USEREGEXP)
-#define isbeforeregsub(si,c)	((si)->wordbits[(CHAR)(c)] & USEREGSUB)
-#define wordbeforeregexp(w)	((w)[-2] & USEREGEXP)
-#define wordbeforeregsub(w)	((w)[-2] & USEREGSUB)
-#define wordfont(w)		((char)((w)[-1]))
 
 
 
@@ -110,120 +92,43 @@ typedef struct
 static char cfont[PUNCT + 1];
 static SINFO *sinfo;
 
-/* This macro computes a hash value for a word, which is used for looking
- * the word up in the SINFO.keyword[] table.  The word is known to be at least
- * one character long and terminated with a '\0', so word[1] is guaranteed to
- * be valid and consistent.
- */
-#define KWHASH(word)	(((word)[0] & 0x1f) ^ (((word)[1] & 0x03) << 5))
 
-
-/* This function returns a pointer to the hashed keyword description if the
- * given word is in fact a keyword.  Otherwise it returns NULL.
+/* This function returns True if the given word is a keyword, or False
+ * otherwise.
  */
-static CHAR *iskeyword(word)
+static BOOLEAN iskeyword(word)
 	CHAR	*word;	/* pointer to word */
 {
-	int	hash;
 	int	i, j;
 
-	/* compute a hash value for this word */
-	hash = KWHASH(word);
-
-	/* if no keywords have that hash value, then return False right away */
-	if (!sinfo->keyword[hash])
-		return NULL;
-
 	/* try to find the word in the list of keywords */
 	if (sinfo->ignorecase)
 	{
 		/* not case sensitive */
-		for (i = 0; sinfo->keyword[hash][i]; i++)
+		for (i = 0; i < sinfo->nkeywords; i++)
 		{
-			for (j = 0; toupper(sinfo->keyword[hash][i][j]) == toupper(word[j]); j++)
+			for (j = 0; toupper(sinfo->keyword[i][j]) == toupper(word[j]); j++)
 			{
-				if (!sinfo->keyword[hash][i][j])
-					return sinfo->keyword[hash][i];
+				if (!sinfo->keyword[i][j])
+					return True;
 			}
 		}
 	}
 	else
 	{
 		/* case sensitive */
-		for (i = 0; sinfo->keyword[hash][i]; i++)
+		for (i = 0; i < sinfo->nkeywords; i++)
 		{
-			if (!CHARcmp(sinfo->keyword[hash][i], word))
+			if (*sinfo->keyword[i] == *word
+			 && !CHARcmp(sinfo->keyword[i], word))
 			{
-				return sinfo->keyword[hash][i];
+				return True;
 			}
 		}
 	}
 
 	/* we didn't find it in the keyword list */
-	return NULL;
-}
-
-static void addkeyword(word, font, doesregexp, doesregsub)
-	CHAR	*word;		/* a keyword */
-	_char_	font;		/* font, or '\0' for default */
-	BOOLEAN	doesregexp;	/* can keyword be followed by a regexp? */
-	BOOLEAN	doesregsub;	/* can keyword be followed by regexp+regsub? */
-{
-	int	hash;		/* hash value of word */
-	CHAR	*keyword;	/* entry describing the current keyword */
-	CHAR	**hashed;	/* new list for a hash table slot */
-	int	i;
-
-	/* see if the keyword is already in the list */
-	keyword = iskeyword(word);
-	if (!keyword)
-	{
-		/* no, we need to add it... */
-		hash = KWHASH(word);
-		if (sinfo->keyword[hash])
-		{
-			/* allocate a new, larger copy of the hash array */
-			for (i = 0; sinfo->keyword[hash][i]; i++)
-			{
-			}
-			hashed = safealloc(i + 2, sizeof(CHAR *));
-			for (i = 0; sinfo->keyword[hash][i]; i++)
-			{
-				hashed[i] = sinfo->keyword[hash][i];
-			}
-			safefree(sinfo->keyword[hash]);
-			sinfo->keyword[hash] = hashed;
-		}
-		else
-		{
-			/* create the first hash list */
-			sinfo->keyword[hash] = safealloc(2, sizeof(CHAR *));
-			i = 0;
-		}
-
-		/* append the new keyword to the list */
-		keyword = safealloc(CHARlen(word) + 3, sizeof(CHAR));
-		*keyword++ = '\0'; /* no special bits yet */
-		*keyword++ = '\0'; /* no special font yet */
-		CHARcpy(keyword, word);
-		sinfo->keyword[hash][i] = keyword;
-		sinfo->keyword[hash][i + 1] = NULL;
-	}
-
-	/* set the word's attributes */
-	if (font)
-		keyword[-1] = font;
-	if (doesregexp)
-		keyword[-2] |= USEREGEXP;
-	if (doesregsub)
-		keyword[-2] |= USEREGEXP | USEREGSUB;
-
-	/* if not a normal word, then mark the character for extra checking */
-	if (!isstartword(sinfo, keyword[0]) ||
-		(keyword[1] && !isinword(sinfo, keyword[1])))
-	{
-		sinfo->wordbits[keyword[0]] |= PUNCTWORD;
-	}
+	return False;
 }
 
 
@@ -308,7 +213,7 @@ BOOLEAN dmsknown(filename)
 static DMINFO *init(win)
 	WINDOW	win;
 {
-	char	*pathname, *str;
+	char	*pathname;
 	CHAR	*cp, **values;
 	int	i, j;
 
@@ -322,6 +227,8 @@ static DMINFO *init(win)
 		dmsyntax.move = dmnormal.move;
 		dmsyntax.wordmove = dmnormal.wordmove;
 		dmsyntax.indent = dmnormal.indent; /* !!! really a good idea? */
+		dmsyntax.tagatcursor = dmnormal.tagatcursor;
+		dmsyntax.tagload = dmnormal.tagload;
 		dmsyntax.tagnext = dmnormal.tagnext;
 
 		/* initialize the mode's global options */
@@ -332,12 +239,6 @@ static DMINFO *init(win)
 		optpreset(o_variablefont, 'n', OPT_REDRAW);
 		optpreset(o_prepfont, 'e', OPT_REDRAW);
 		optpreset(o_otherfont, 'b', OPT_REDRAW);
-		str = getenv("INCLUDE");
-#ifdef OSINCLUDEPATH
-		if (!str)
-			str = OSINCLUDEPATH;
-#endif
-		o_includepath = toCHAR(str);
 
 		/* if no real window, then we're done! */
 		if (!win)
@@ -382,7 +283,7 @@ static DMINFO *init(win)
 			{
 				if (CHARcmp(values[0], toCHAR("extension")))
 					continue;
-				for (i = 1; values[i] && (CHARlen(values[i]) > (unsigned)j || CHARcmp(values[i], cp + j - CHARlen(values[i]))); i++)
+				for (i = 1; values[i] && (CHARlen(values[i]) > j || CHARcmp(values[i], cp + j - CHARlen(values[i]))); i++)
 				{
 				}
 				if (values[i])
@@ -400,38 +301,14 @@ static DMINFO *init(win)
 			while ((values = fetchline()) != NULL
 				&& CHARcmp(values[0], toCHAR("language")))
 			{
-				str = tochar8(values[0]);
-				if (!strcmp(str, "keyword"))
+				if (!CHARcmp(values[0], toCHAR("keyword")))
 				{
-					for (i = 1; values[i]; i++)
+					for (i = 1; sinfo->nkeywords < QTY(sinfo->keyword) && values[i]; i++)
 					{
-						addkeyword(values[i], '\0', False, False);
+						sinfo->keyword[sinfo->nkeywords++] = CHARdup(values[i]);
 					}
 				}
-				else if (!strcmp(str, "font") && values[1])
-				{
-					str = tochar8(values[1]);
-					if (!strcmp(str, "normal")
-					 || !strcmp(str, "n")
-					 || !strcmp(str, "fixed")
-					 || !strcmp(str, "f")
-					 || !strcmp(str, "bold")
-					 || !strcmp(str, "b")
-					 || !strcmp(str, "emphasized")
-					 || !strcmp(str, "e")
-					 || !strcmp(str, "italic")
-					 || !strcmp(str, "i")
-					 || !strcmp(str, "underlined")
-					 || !strcmp(str, "u"))
-					{
-						for (i = 2; values[i]; i++)
-						{
-							addkeyword(values[i], *str, False, False);
-						}
-					}
-					/* else invalid font */
-				}
-				else if (!strcmp(str, "comment"))
+				else if (!CHARcmp(values[0], toCHAR("comment")))
 				{
 					for (i = 1; values[i]; i++)
 					{
@@ -446,7 +323,7 @@ static DMINFO *init(win)
 						}
 					}
 				}
-				else if (!strcmp(str, "string"))
+				else if (!CHARcmp(values[0], toCHAR("string")))
 				{
 					if (values[1] && values[2])
 					{
@@ -464,7 +341,7 @@ static DMINFO *init(win)
 						sinfo->strend = *values[1];
 					}
 				}
-				else if (!strcmp(str, "character"))
+				else if (!CHARcmp(values[0], toCHAR("character")))
 				{
 					if (values[1] && values[2])
 					{
@@ -482,107 +359,34 @@ static DMINFO *init(win)
 						sinfo->charend = *values[1];
 					}
 				}
-				else if (!strcmp(str, "regexp"))
-				{
-					for (i = 1; values[i]; i++)
-					{
-						for (j = 0; values[i][j]; j++)
-						{
-							sinfo->wordbits[values[i][j]] |= DELIMREGEXP;
-						}
-					}
-				}
-				else if (!strcmp(str, "useregexp"))
-				{
-					for (i = 1; values[i]; i++)
-					{
-						cp = iskeyword(values[i]);
-						if (cp)
-						{
-							cp[-2] |= USEREGEXP;
-						}
-						else if (isalpha(values[i][0]))
-						{
-							addkeyword(values[i], '\0', True, False);
-						}
-						else /* character list */
-						{
-							for (j = 0; values[i][j]; j++)
-							{
-								sinfo->wordbits[values[i][j]] |= USEREGEXP;
-							}
-						}
-					}
-				}
-				else if (!strcmp(str, "useregsub"))
-				{
-					for (i = 1; values[i]; i++)
-					{
-						cp = iskeyword(values[i]);
-						if (cp)
-						{
-							cp[-2] |= USEREGEXP | USEREGSUB;
-						}
-						else if (isalpha(values[i][0]))
-						{
-							addkeyword(values[i], '\0', True, True);
-						}
-						else /* character list */
-						{
-							for (j = 0; values[i][j]; j++)
-							{
-								sinfo->wordbits[values[i][j]] |= USEREGEXP | USEREGSUB;
-							}
-						}
-					}
-				}
-				else if (!strcmp(str, "preprocessor"))
+				else if (!CHARcmp(values[0], toCHAR("preprocessor")))
 				{
 					if (values[1])
 						sinfo->preprocessor = *values[1];
 				}
-				else if (!strcmp(str, "prepquote"))
-				{
-					if (values[1] && values[2])
-					{
-						sinfo->pqbegin = *values[1];
-						sinfo->pqend = *values[2];
-					}
-					else if (values[1] && values[1][1])
-					{
-						sinfo->pqbegin = values[1][0];
-						sinfo->pqend = values[1][1];
-					}
-					else if (values[1])
-					{
-						sinfo->pqbegin =
-						sinfo->pqend = *values[1];
-					}
-				}
-				else if (!strcmp(str, "function"))
+				else if (!CHARcmp(values[0], toCHAR("function")))
 				{
 					if (values[1])
 						sinfo->function = *values[1];
 				}
-				else if (!strcmp(str, "other"))
+				else if (!CHARcmp(values[0], toCHAR("other")))
 				{
 					for (i = 1; values[i]; i++)
 					{
-						str = tochar8(values[i]);
-						if (!strcmp(str, "allcaps"))
+						if (!CHARcmp(values[i], toCHAR("allcaps")))
 							sinfo->allcaps = (BOOLEAN)!sinfo->allcaps;
-						else if (!strcmp(str, "initialcaps"))
+						else if (!CHARcmp(values[i], toCHAR("initialcaps")))
 							sinfo->initialcaps = (BOOLEAN)!sinfo->initialcaps;
-						else if (!strcmp(str, "mixedcaps"))
+						else if (!CHARcmp(values[i], toCHAR("mixedcaps")))
 							sinfo->mixedcaps = (BOOLEAN)!sinfo->mixedcaps;
-						else if (!strcmp(str, "final_t"))
+						else if (!CHARcmp(values[i], toCHAR("final_t")))
 							sinfo->finalt = (BOOLEAN)!sinfo->finalt;
-						else if (!strcmp(str, "initialpunct"))
-							sinfo->initialpunct = (BOOLEAN)!sinfo->initialpunct;
+						else if (!CHARcmp(values[i], toCHAR("initialpunct")))
+							sinfo->finalt = (BOOLEAN)!sinfo->initialpunct;
 						/* else unknown type */
 					}
 				}
-				else if (!strcmp(str, "startword"))
+				else if (!CHARcmp(values[0], toCHAR("startword")))
 				{
 					for (i = 1; values[i]; i++)
 					{
@@ -592,7 +396,7 @@ static DMINFO *init(win)
 						}
 					}
 				}
-				else if (!strcmp(str, "inword"))
+				else if (!CHARcmp(values[0], toCHAR("inword")))
 				{
 					for (i = 1; values[i]; i++)
 					{
@@ -602,7 +406,7 @@ static DMINFO *init(win)
 						}
 					}
 				}
-				else if (!strcmp(str, "ignorecase"))
+				else if (!CHARcmp(values[0], toCHAR("ignorecase")))
 				{
 					if (values[1])
 						sinfo->ignorecase = calctrue(values[1]);
@@ -626,28 +430,16 @@ static DMINFO *init(win)
 static void term(info)
 	DMINFO	*info;	/* window-specific information about mode */
 {
-	int	hash, i;
+	int	i;
 	SINFO	*si = (SINFO *)info;
 
-	for (hash = 0; hash < 256; hash++)
+	for (i = 0; i < si->nkeywords; i++)
 	{
-		/* skip empty hash lists */
-		if (!si->keyword[hash]) continue;
-
-		/* free all words in this hash list, and then free the list */
-		for (i = 0; si->keyword[hash][i]; i++)
-		{
-			safefree(si->keyword[hash][i] - 2);
-				/* "- 2" because we sneak in two extra chars
-				 * to store attributes of the keyword.
-				 */
-		}
-		safefree(si->keyword[hash]);
+		safefree(si->keyword[i]);
 	}
 	safefree(info);
 }
 
-
 /* Choose a line to appear at the top of the screen, and return its mark.
  * Also, initialize the info for the next line.
  */
@@ -667,8 +459,8 @@ static MARK setup(top, cursor, bottom, info)
 
 	/* copy the values of the "font" options into the cfont[] array */
 	cfont[COMMENT] = cfont[COMMENT2] = o_commentfont;
-	cfont[STRING] = cfont[CHARACTER] = cfont[REGEXP] = cfont[REGSUB] = cfont[PREPQUOTE] = o_stringfont;
-	cfont[KEYWORD] = cfont[FIRSTPUNCT] = cfont[SECONDPUNCT] = o_keywordfont;
+	cfont[STRING] = cfont[CHARACTER] = o_stringfont;
+	cfont[KEYWORD] = o_keywordfont;
 	cfont[FUNCTION] = o_functionfont;
 	cfont[VARIABLE] = o_variablefont;
 	cfont[PREP] = cfont[PREPWORD] = o_prepfont;
@@ -765,7 +557,6 @@ static MARK image(w, line, info, draw)
 	int	col;
 	CHAR	*cp;
 	CHAR	tmpchar;
-	CHAR	regexpdelim;	/* delimiter for current regexp */
 	long	offset;
 	static MARKBUF tmp;
 	CHAR	undec[40];	/* characters of undecided font */
@@ -774,21 +565,12 @@ static MARK image(w, line, info, draw)
 	BOOLEAN	quote;		/* True after a backslash in STRING or CHARACTER */
 	int	upper, lower;	/* counts letters of each case */
 	BOOLEAN	indent;
-	CHAR	*kp;		/* pointer to a keyword */
-	BOOLEAN	expectregexp;	/* allow a regular expression to start next */
-	BOOLEAN	expectregsub;	/* allow substitution text to follow regexp */
-	BOOLEAN	expectprepq;	/* allow preprocessor quotes */
 	int	i;
 
 	/* initially, we'll assume we continue the font of the previous line */
 	sinfo = (SINFO *)info;
 	quote = False;
 	indent = True;
-	expectregexp = (BOOLEAN)(sinfo->token == PUNCT);
-	expectregsub = expectprepq = False;
-
-	/* this is just to silence a compiler warning */
-	regexpdelim = '/';
 
 	/* for each character in the line... */
 	for (prev = ' ', col = 0, offset = markoffset(line), scanalloc(&cp, line);
@@ -844,22 +626,15 @@ static MARK image(w, line, info, draw)
 		}
 		else /* normal printable character */
 		{
-			/* ending a keyword/function/variable? */
-			if (!isinword(sinfo, *cp)
-				&& (sinfo->token == FUNCTION
-					|| sinfo->token == VARIABLE
-					|| sinfo->token == KEYWORD
-					|| sinfo->token == OTHER))
+			/* any non-whitespace ends indent */
+			if (*cp != ' ')
 			{
-				sinfo->token = PUNCT;
+				indent = False;
 			}
 
 			/* starting a keyword/function/variable? */
 			if (sinfo->token == PUNCT && isstartword(sinfo, *cp))
 			{
-				/* this isn't regexp */
-				expectregexp = expectregsub = False;
-
 				/* store first letter of possible keyword */
 				lower = upper = 0;
 				undec[0] = *cp;
@@ -883,15 +658,9 @@ static MARK image(w, line, info, draw)
 				undec[i] = '\0';
 
 				/* did we find a keyword? */
-				kp = iskeyword(undec);
-				if (kp)
+				if (iskeyword(undec))
 				{
 					sinfo->token = KEYWORD;
-					cfont[KEYWORD] = wordfont(kp);
-					if (!cfont[KEYWORD])
-						cfont[KEYWORD] = o_keywordfont;
-					expectregexp = (BOOLEAN)wordbeforeregexp(kp);
-					expectregsub = (BOOLEAN)wordbeforeregsub(kp);
 				}
 				else /* must be function, variable, or other */
 				{
@@ -916,10 +685,6 @@ static MARK image(w, line, info, draw)
 					{
 						sinfo->token = FUNCTION;
 					}
-					else if (sinfo->finalt && prev2 == '_' && prev == 't')
-					{
-						sinfo->token = OTHER;
-					}
 					else if (sinfo->initialpunct && !isalnum(undec[0]))
 					{
 						sinfo->token = OTHER;
@@ -928,6 +693,10 @@ static MARK image(w, line, info, draw)
 					{
 						sinfo->token = OTHER;
 					}
+					else if (sinfo->finalt && prev2 == '_' && prev == 't')
+					{
+						sinfo->token = OTHER;
+					}
 					else if (sinfo->initialcaps && lower > 0 && isupper(undec[0]))
 					{
 						sinfo->token = OTHER;
@@ -936,13 +705,10 @@ static MARK image(w, line, info, draw)
 					{
 						sinfo->token = OTHER;
 					}
-					else if (i > 1 || isalnum(*cp))
+					else /* plain names are usually variables */
 					{
 						sinfo->token = VARIABLE;
 					}
-					/* else leave it set to PUNCT so we can
-					 * recognize two-punctuation keywords.
-					 */
 				}
 				scanfree(&up);
 			}
@@ -953,47 +719,20 @@ static MARK image(w, line, info, draw)
 			else if (sinfo->token == PREPWORD && !isalnum(*cp))
 			{
 				sinfo->token = PUNCT;
-				expectprepq = True;
 			}
-
-			/* start of preprocessor quote? */
-			if (sinfo->token == PUNCT && expectprepq && *cp == sinfo->pqbegin)
+			else if (!isinword(sinfo, *cp)
+				&& (sinfo->token == FUNCTION
+					|| sinfo->token == VARIABLE
+					|| sinfo->token == KEYWORD
+					|| sinfo->token == OTHER))
 			{
-				sinfo->token = PREPQUOTE;
-				expectprepq = False;
-			}
-
-			/* start of a two-punctuation keyword? */
-			if (sinfo->token == PUNCT && ispunctword(sinfo, *cp))
-			{
-				/* maybe... check the other punct character */
-				undec[0] = *cp;
-				undec[1] = '\0';
-				kp = iskeyword(undec);
-				if (!kp)
-				{
-					tmp = *scanmark(&cp);
-					markaddoffset(&tmp, 1);
-					undec[1] = scanchar(&tmp);
-					undec[2] = '\0';
-					kp = iskeyword(undec);
-				}
-				if (kp)
-				{
-					sinfo->token = kp[1] ? FIRSTPUNCT : SECONDPUNCT;
-					cfont[FIRSTPUNCT] = cfont[SECONDPUNCT] = wordfont(kp);
-					if (!cfont[FIRSTPUNCT])
-						cfont[FIRSTPUNCT] = cfont[SECONDPUNCT] = o_keywordfont;
-					expectregexp = (BOOLEAN)wordbeforeregexp(kp);
-					expectregsub = (BOOLEAN)wordbeforeregsub(kp);
-				}
+				sinfo->token = PUNCT;
 			}
 
 			/* start of a string? */
 			if (sinfo->strbegin && sinfo->token == PUNCT && *cp == sinfo->strbegin)
 			{
 				sinfo->token = STRING;
-				expectregexp = expectregsub = False;
 
 				/* make sure the initial quote character
 				 * isn't going to be mistaken for the
@@ -1006,7 +745,6 @@ static MARK image(w, line, info, draw)
 			if (sinfo->charbegin && sinfo->token == PUNCT && *cp == sinfo->charbegin)
 			{
 				sinfo->token = CHARACTER;
-				expectregexp = expectregsub = False;
 
 				/* make sure the initial quote character
 				 * isn't going to be mistaken for the
@@ -1023,46 +761,16 @@ static MARK image(w, line, info, draw)
 					|| scanchar(marktmp(tmp, markbuffer(line), offset + 1)) == sinfo->combegin[1]))
 			{
 				sinfo->token = COMMENT;
-				expectregexp = expectregsub = False;
 			}
 
 			/* start of a one-line comment? */
 			if (sinfo->token == PUNCT
 				&& sinfo->comment[0]
 				&& *cp == sinfo->comment[0]
-				&& (indent || !expectregexp)
 				&& (!sinfo->comment[1]
 					|| scanchar(marktmp(tmp, markbuffer(line), offset + 1)) == sinfo->comment[1]))
 			{
 				sinfo->token = COMMENT2;
-				expectregexp = expectregsub = False;
-			}
-
-			/* start of a regular expression? */
-			if (sinfo->token == PUNCT && expectregexp && isregexp(sinfo, *cp))
-			{
-				sinfo->token = REGEXP;
-				expectregexp = False;
-				regexpdelim = *cp;
-
-				/* make sure the initial quote character
-				 * isn't going to be mistaken for the
-				 * terminating quote character.
-				 */
-				quote = True;
-			}
-
-			/* would a regexp be allowed after this char? */
-			if (sinfo->token == PUNCT && !isspace(*cp))
-			{
-				expectregexp = (BOOLEAN)isbeforeregexp(sinfo, *cp);
-				expectregsub = (BOOLEAN)isbeforeregsub(sinfo, *cp);
-			}
-
-			/* any non-whitespace ends indent and disabled prepquote */
-			if (!isspace(*cp))
-			{
-				indent = expectprepq = False;
 			}
 
 			/* draw the character */
@@ -1090,43 +798,10 @@ static MARK image(w, line, info, draw)
 				sinfo->token = PUNCT;
 			}
 
-			/* end of a regexp or substitution text? */
-			if ((sinfo->token == REGEXP || sinfo->token == REGSUB)
-				&& *cp == regexpdelim && !quote)
-			{
-				if (expectregsub)
-				{
-					sinfo->token = REGSUB;
-					quote = True;
-					expectregsub = False;
-				}
-				else
-				{
-					sinfo->token = PUNCT;
-				}
-			}
-
-			/* middle/end of a two-punctuation character keyword? */
-			if (sinfo->token == FIRSTPUNCT)
-			{
-				sinfo->token = SECONDPUNCT;
-			}
-			else if (sinfo->token == SECONDPUNCT)
-			{
-				sinfo->token = PUNCT;
-			}
-
-			/* end of a prepquote? */
-			if (sinfo->token == PREPQUOTE && *cp == sinfo->pqend)
-			{
-				sinfo->token = PUNCT;
-			}
-
-			/* in a STRING, CHARACTER, REGEXP, or REGSUB constant,
-			 * backslash is used to quote the following character.
+			/* in a STRING or CHARACTER constant, backslash
+			 * is used to quote the following character.
 			 */
-			if ((sinfo->token==STRING || sinfo->token==CHARACTER
-			 || sinfo->token == REGEXP || sinfo->token == REGSUB)
+			if ((sinfo->token==STRING || sinfo->token==CHARACTER)
 				&& *cp == '\\' && !quote)
 			{
 				quote = True;
@@ -1170,141 +845,6 @@ static MARK image(w, line, info, draw)
 }
 
 
-/* This function considers the possibility that the cursor may be on a quoted
- * filename.  If so, it returns the name, with the quotes.  Otherwise it calls
- * dmnormal.tagatcursor() for the traditional tags.
- *
- * The return value is a dynamically-allocated string; the calling function
- * is responsible for freeing it when it is no longer required.
- */
-static CHAR *tagatcursor(win, cursor)
-	WINDOW win;	/* window, used for finding mode-dependent info */
-	MARK cursor;	/* where the desired tag name can be found */
-{
-	CHAR	*ret;	/* return value */
-	CHAR	*cp;	/* used for scanning */
-
-	/* initialization */
-	sinfo = (SINFO *)win->mi;
-	ret = NULL;
-
-	/* search backward for first quote or whitespace */
-	for (scanalloc(&cp, cursor);
-	     cp && !isspace(*cp) && *cp != sinfo->pqbegin && *cp != sinfo->strbegin;
-	     scanprev(&cp))
-	{
-	}
-
-	/* did we find some kind of quote? */
-	if (cp && *cp == sinfo->strbegin)
-	{
-		/* string - search forward for closing quote, collecting chars
-		 * along the way.  Beware of whitespace.
-		 */
-		do
-		{
-			buildCHAR(&ret, *cp);
-		} while (scannext(&cp) && !isspace(*cp) && *cp != sinfo->strend);
-		if (cp && *cp == sinfo->strend)
-		{
-			buildCHAR(&ret, *cp);
-			scanfree(&cp);
-			return ret;
-		}
-	}
-	else if (cp && *cp == sinfo->pqbegin)
-	{
-		/* prepquote - search forward for closing quote, collecting
-		 * chars along the way.  Beware of whitespace.
-		 */
-		do
-		{
-			buildCHAR(&ret, *cp);
-		} while (scannext(&cp) && !isspace(*cp) && *cp != sinfo->pqend);
-		if (cp && *cp == sinfo->pqend)
-		{
-			buildCHAR(&ret, *cp);
-			scanfree(&cp);
-			return ret;
-		}
-	}
-
-	/* cleanup */
-	scanfree(&cp);
-	if (ret)
-		safefree(ret);
-
-	/* use dmnormal's tagatcursor() function */
-	return (*dmnormal.tagatcursor)(win, cursor);
-}
-
-
-/* This function checks the tagname.  If it is a quoted filename, then it
- * attempts to load the file.  Otherwise it calls dmnormal.tagload() to try
- * and load a normal tag.
- */
-static MARK tagload(tagname, from)
-	CHAR *tagname;	/* name of tag to move to */
-	MARK from;	/* where we're coming from */
-{
-	char	*name;
-	char	*tmp;
-	unsigned len;
-	DIRPERM	perms;
-	BUFFER	buf;
- static	MARKBUF	retb;
-
-	/* is it a quoted filename? */
-	len = CHARlen(tagname);
-	if (len >= 3 && (*tagname == '"' || (sinfo && sinfo->pqbegin && *tagname == sinfo->pqbegin)))
-	{
-		/* make an unquoted, 8-bit version of the name */
-		name = strdup(tochar8(tagname + 1));
-		if ((*tagname == '"' && tagname[len - 1] == '"')
-		 || (sinfo && sinfo->pqbegin && *tagname == sinfo->pqbegin && tagname[len - 1] == sinfo->pqend))
-		{
-			name[len - 2] = '\0';
-		}
-
-		/* if plain old quote character, then look for it first in
-		 * the current directory.
-		 */
-		if (*tagname == '"')
-		{
-			perms = dirperm(name);
-			if (perms == DIR_READONLY || perms == DIR_READWRITE)
-			{
-				buf = bufload(NULL, name, False);
-				assert(buf != NULL);
-				safefree(name);
-				return marktmp(retb, buf, buf->changepos);
-			}
-		}
-
-		/* search through the include path */
-		if (o_includepath)
-		{
-			tmp = iopath(tochar8(o_includepath), name, False);
-			if (tmp)
-			{
-				buf = bufload(NULL, tmp, False);
-				assert(buf != NULL);
-				safefree(name);
-				return marktmp(retb, buf, buf->changepos);
-			}
-		}
-
-		/* Failed!  Complain, clean up, and return NULL */
-		msg(MSG_ERROR, "[s]header file $1 not found", name);
-		safefree(name);
-		return NULL;
-	}
-
-	/* use dmnormal's tagload() function */
-	return (*dmnormal.tagload)(tagname, from);
-}
-
-
 DISPMODE dmsyntax =
 {
 	"syntax",
@@ -1324,9 +864,6 @@ DISPMODE dmsyntax =
 	setup,
 	image,
 	NULL,	/* doesn't need a header */
-	NULL,	/* init() sets this to be identical to dmnormal's indent() */
-	tagatcursor,
-	tagload,
-	NULL	/* init() sets this to be identical to dmnormal's tagnext() */
+	NULL	/* init() sets this to be identical to dmnormal's indent() */
 };
 #endif /* DISPLAY_SYNTAX */
