@@ -1,16 +1,10 @@
 /* draw.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_draw[] = "$Id: draw.c,v 2.65 1996/09/26 01:05:52 steve Exp $";
+char id_draw[] = "$Id: draw.c,v 2.54 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
-#if defined (GUI_WIN32)
-# define SLOPPY_ITALICS
-#else
-# undef SLOPPY_ITALICS
-#endif
-
 #if USE_PROTOTYPES
 static void insimage(CHAR *ch, char *font, int qty, int extent);
 static void delimage(CHAR *ch, char *font, int qty, int extent);
@@ -98,8 +92,7 @@ void drawexpose(win, top, left, bottom, right)
 	int	right;	/* right edge to be redrawn */
 {
 	int	row, column, same, base, nonblank;
-	MARKBUF m;
-	long	firstline, lastline;
+	MARKBUF first, last;
 
 	assert(win != NULL && top >= 0 && left >= 0 && bottom < o_lines(win)
 		&& right < o_columns(win) && top <= bottom && left <= right);
@@ -182,25 +175,9 @@ void drawexpose(win, top, left, bottom, right)
 	{
 		if (win->md->move == dmnormal.move)
 		{
-			/* find line numbers of first and last lines shown */
-			firstline = markline(marktmp(m, markbuffer(win->cursor), win->di->topline)) - 1;
-			lastline = markline(marktmp(m, markbuffer(win->cursor), win->di->bottomline)) - 1;
-
-			/* Some scrolling commands temporarily set bottomline
-			 * to the end of the buffer.  Ordinarily this causes
-			 * no problems, but if an expose event occurs before
-			 * the screen is redrawn, the the scrollbar's "thumb"
-			 * will briefly extend to the bottom of the scrollbar.
-			 * If we seem to be in that situation, then make a more
-			 * reasonable guess about the screen bottom line number.
-			 */
-			if (lastline - firstline >= o_lines(win))
-			{
-				lastline = firstline + o_lines(win) - 1;
-			}
-
-			/* update the scrollbar */
-			(*gui->scrollbar)(win->gw, firstline, lastline,
+			(*gui->scrollbar)(win->gw,
+				markline(marktmp(first, markbuffer(win->cursor), win->di->topline)) - 1,
+				markline(marktmp(last, markbuffer(win->cursor), win->di->bottomline)) - 1,
 				o_buflines(markbuffer(win->cursor)));
 		}
 		else
@@ -280,7 +257,7 @@ static void delimage(ch, font, qty, extent)
 	int	qty;	/* number of characters to delete */
 	int	extent;	/* number of characters after the deletion point */
 {
-	register int i;
+	register i;
 
 	/* shift the characters */
 	for (i = 0; i < extent - qty; i++)
@@ -367,7 +344,7 @@ static void fillcell(ch, font, offset)
 
 		/* adjust limits, taking o_sidescroll into consideration */
 		thiscell -= o_columns(thiswin);
-		if (o_wrap(thiswin))
+		if (o_sidescroll(thiswin) == 0)
 		{
 			rightcol += o_columns(thiswin);
 		}
@@ -533,9 +510,9 @@ static void drawchar(p, qty, font, offset)
 		/* if this is a newline, then treat it as a bunch of spaces */
 		if (ch == '\n')
 		{
-			/* remember this line's width.  If a single "line"
-			 * involves multiple newlines, then the width of the
-			 * last non-empty phsyical line is kept.
+			/* remember this line's width.  If a single "line" involves
+			 * multiple newlines, then the width of the last non-empty
+			 * phsyical line is kept.
 			 */
 			if (thiscol != 0)
 			{
@@ -561,7 +538,6 @@ static void drawchar(p, qty, font, offset)
 			 * will be there.  This test must be made outside the
 			 * loop.
 			 */
-#if 0
 			if (offset == markoffset(thiswin->state->cursor)
 				&& thiscol % o_columns(thiswin) == 0)
 			{
@@ -571,25 +547,6 @@ static void drawchar(p, qty, font, offset)
 			{
 				fillcell(' ', hifont, offset);
 			}
-#else
-			i = o_columns(thiswin);
-			if ((offset == markoffset(thiswin->state->cursor)
-				&& thiscol % i == 0) || thiscol == leftcol)
-			{
-				fillcell(' ', hifont, offset);
-			}
-			/* Note: thiscell may've changed since previous "if" */
-			if (thiscell % i != 0)
-			{
-				for (i -= thiscell % i; i > 0; i--)
-				{
-					di->newchar[thiscell] = ' ';
-					di->newfont[thiscell] = hifont;
-					di->offsets[thiscell] = offset;
-					thiscol++, thiscell++;
-				}
-			}
-#endif
 
 			/* reset the virtual column number */
 			thiscol = 0;
@@ -773,53 +730,50 @@ printf("---------------------------------------------------------------------\n"
 static void updateimage(win)
 	WINDOW	win;	/* window to be updated */
 {
-	int	row, column, same, used;
+	int	row, column, base, same, i, used;
 	int	ncols = o_columns(win);
-	register int	  i, base;
-	register DRAWINFO *di = win->di;
-	register int	  rowXncols;
 
 	/* for each row... */
-	for (row = rowXncols = 0; row < o_lines(win); row++, rowXncols += ncols)
+	for (row = 0; row < o_lines(win); row++)
 	{
-		assert(di->newrow[row].insrows == 0 || di->logic != DRAW_SCRATCH);
+		assert(win->di->newrow[row].insrows == 0 || win->di->logic != DRAW_SCRATCH);
 
 		/* do we want to insert/delete rows? */
-		if (di->newrow[row].insrows != 0)
+		if (win->di->newrow[row].insrows != 0)
 		{
 			/* try to perform the insert/delete for the GUI */
 			guimoveto(win, 0, row);
-			if (guiscroll(win, di->newrow[row].insrows, (BOOLEAN)(maxrow < o_lines(win))))
+			if (guiscroll(win, win->di->newrow[row].insrows, (BOOLEAN)(maxrow < o_lines(win))))
 			{
 				/* perform the insert/delete on the "current" image */
-				if (di->newrow[row].insrows > 0)
+				if (win->di->newrow[row].insrows > 0)
 				{
 					/* insert some rows */
-					insimage(&di->curchar[rowXncols],
-						&di->curfont[rowXncols],
-						di->newrow[row].insrows * ncols,
+					insimage(&win->di->curchar[row * ncols],
+						&win->di->curfont[row * ncols],
+						win->di->newrow[row].insrows * ncols,
 						(scrollrows - row) * ncols);
 
 					/* shift amount of new lines is always 0 */
-					for (i = row; i < row + di->newrow[row].insrows; i++)
+					for (i = row; i < row + win->di->newrow[row].insrows; i++)
 					{
-						di->newrow[i].shiftright =
-							di->newrow[i].inschars = 0;
+						win->di->newrow[i].shiftright =
+							win->di->newrow[i].inschars = 0;
 					}
 				}
 				else
 				{
 					/* delete some rows */
-					delimage(&di->curchar[rowXncols],
-						&di->curfont[rowXncols],
-						-di->newrow[row].insrows * ncols,
+					delimage(&win->di->curchar[row * ncols],
+						&win->di->curfont[row * ncols],
+						-win->di->newrow[row].insrows * ncols,
 						(scrollrows - row) * ncols);
 
 					/* shift amount of new lines is always 0 */
-					for (i = maxrow + di->newrow[row].insrows; i < maxrow; i++)
+					for (i = maxrow + win->di->newrow[row].insrows; i < maxrow; i++)
 					{
-						di->newrow[i].shiftright =
-							di->newrow[i].inschars = 0;
+						win->di->newrow[i].shiftright =
+							win->di->newrow[i].inschars = 0;
 					}
 				}
 			}
@@ -828,47 +782,47 @@ static void updateimage(win)
 				/* don't do any more scrolling after this */
 				for (i = row; i < o_lines(win); i++)
 				{
-					di->newrow[i].insrows = 0;
+					win->di->newrow[i].insrows = 0;
 				}
 			}
 		}
 
 		/* perform shifting, if any */
-		if (di->newrow[row].shiftright != 0)
+		if (win->di->newrow[row].shiftright != 0)
 		{
 			/* see how many following rows have the same shift */
 			for (same = 1;
 			     gui->shiftrows && 
-			        row + same < di->rows - 1 &&
-				di->newrow[row].shiftright == di->newrow[row + same].shiftright;
+			        row + same < win->di->rows - 1 &&
+				win->di->newrow[row].shiftright == win->di->newrow[row + same].shiftright;
 			     same++)
 			{
 			}
 
 			/* make the GUI do its shift */
 			guimoveto(win, 0, row);
-			if (guishift(win, di->newrow[row].shiftright, same))
+			if (guishift(win, win->di->newrow[row].shiftright, same))
 			{
 				/* shift the current image, too */
 				for (i = row; i < row + same; i++)
 				{
-					if (di->newrow[i].shiftright > 0)
+					if (win->di->newrow[i].shiftright > 0)
 					{
 						/* shift right by inserting */
-						insimage(&di->curchar[i * ncols],
-							&di->curfont[i * ncols],
-							di->newrow[i].shiftright,
+						insimage(&win->di->curchar[i * ncols],
+							&win->di->curfont[i * ncols],
+							win->di->newrow[i].shiftright,
 							ncols);
 					}
 					else 
 					{
 						/* shift left by deleting */
-						delimage(&di->curchar[i * ncols],
-							&di->curfont[i * ncols],
-							-di->newrow[i].shiftright,
+						delimage(&win->di->curchar[i * ncols],
+							&win->di->curfont[i * ncols],
+							-win->di->newrow[i].shiftright,
 							ncols);
 					}
-					di->newrow[i].shiftright = 0;
+					win->di->newrow[i].shiftright = 0;
 				}
 			}
 		}
@@ -877,29 +831,18 @@ static void updateimage(win)
 		base = ncols * row;
 		for (used = ncols;
 		     used > 0
-			&& di->newchar[base + used - 1] == ' '
-			&& di->newfont[base + used - 1] == 'n';
+			&& win->di->newchar[base + used - 1] == ' '
+			&& win->di->newfont[base + used - 1] == 'n';
 		     used--)
 		{
 		}
 
-#ifdef SLOPPY_ITALICS
-		/* If the line ends with an italic character, then pretend
-		 * we're using one additional character.  This is so the last
-		 * italic character's right edge isn't clipped off.
-		 */
-		if (used < ncols && used >= 1 && di->newfont[base + used - 1] == 'i')
-		{
-			used++;
-		}
-#endif
-
 		/* look for mismatched segments */
 		for (column = 0; column < used; base += same, column += same)
 		{
 			/* if this cell matches, then skip to next */
-			if (di->newchar[base] == di->curchar[base]
-			 && di->newfont[base] == di->curfont[base])
+			if (win->di->newchar[base] == win->di->curchar[base]
+			 && win->di->newfont[base] == win->di->curfont[base])
 			{
 				same = 1;
 				continue;
@@ -913,43 +856,43 @@ static void updateimage(win)
 			 * we're still on the inserted blanks at the start
 			 * of the row.
 			 */
-			if (column >= di->newrow[row].shiftright
-			  && di->newrow[row].inschars != 0)
+			if (column >= win->di->newrow[row].shiftright
+			  && win->di->newrow[row].inschars != 0)
 			{
-				if (guishift(win, di->newrow[row].inschars, 1))
+				if (guishift(win, win->di->newrow[row].inschars, 1))
 				{
 					/* shift the current image, too */
-					if (di->newrow[row].inschars > 0)
+					if (win->di->newrow[row].inschars > 0)
 					{
 						/* shift right by inserting */
-						insimage(&di->curchar[rowXncols + column],
-							&di->curfont[rowXncols + column],
-							di->newrow[row].inschars,
+						insimage(&win->di->curchar[row * ncols + column],
+							&win->di->curfont[row * ncols + column],
+							win->di->newrow[row].inschars,
 							ncols - column);
 					}
 					else 
 					{
 						/* shift left by deleting */
-						delimage(&di->curchar[rowXncols + column],
-							&di->curfont[rowXncols + column],
-							-di->newrow[row].inschars,
+						delimage(&win->di->curchar[row * ncols + column],
+							&win->di->curfont[row * ncols + column],
+							-win->di->newrow[row].inschars,
 							ncols - column);
 					}
 
 					/* for other rows of the same line, insertions
 					 * will now appear to be shifts.
 					 */
-					for (i = row + 1; i < maxrow && di->newrow[i].lineoffset == di->newrow[row].lineoffset; i++)
+					for (i = row + 1; i < maxrow && win->di->newrow[i].lineoffset == win->di->newrow[row].lineoffset; i++)
 					{
-						di->newrow[i].shiftright += di->newrow[i].inschars;
-						di->newrow[i].inschars = 0;
+						win->di->newrow[i].shiftright += win->di->newrow[i].inschars;
+						win->di->newrow[i].inschars = 0;
 					}
 				}
 
 				/* we have now done all the inserting/deleting
 				 * that we're ever going to do for this row.
 				 */
-				di->newrow[row].inschars = 0;
+				win->di->newrow[row].inschars = 0;
 			}
 
 			/* See how many mismatched cells we can find with same
@@ -959,18 +902,14 @@ static void updateimage(win)
 			 */
 			for (same = 1, i = 0;
 			     column + same < used
-				&& (di->newchar[base + same] != di->curchar[base + same]
-					|| di->newfont[base + same] != di->curfont[base + same]
+				&& (win->di->newchar[base + same] != win->di->curchar[base + same]
+					|| win->di->newfont[base + same] != win->di->curfont[base + same]
 					|| i < gui->movecost)
-				&& (di->newfont[base] == di->newfont[base + same]
-#ifdef SLOPPY_ITALICS
-					|| (di->newfont[base + same - 1] == 'i' && di->newfont[base + same] == 'n' && di->newchar[base + same] == ' ')
-#endif
-										 );
+				&& win->di->newfont[base] == win->di->newfont[base + same];
 			     same++)
 			{
-				if (di->newchar[base + same] != di->curchar[base + same]
-					|| di->newfont[base + same] != di->curfont[base + same])
+				if (win->di->newchar[base + same] != win->di->curchar[base + same]
+					|| win->di->newfont[base + same] != win->di->curfont[base + same])
 				{
 					i = 0;
 				}
@@ -981,32 +920,14 @@ static void updateimage(win)
 			}
 			same -= i;
 
-#ifdef SLOPPY_ITALICS
-			/* if the change is in italics, and the preceding
-			 * characters were also italic, then include those
-			 * preceding characters in the redrawn span; otherwise 
-			 * they would be distorted by the new text.
-			 */
-			while (column > 0
-			    && di->newfont[base] == 'i'
-			    && di->newfont[base - 1] == 'i'
-			    && di->newchar[base - 1] != ' ')
-			{
-				base--;
-				column--;
-				same++;
-			}
-			guimoveto(win, column, row);
-#endif
-
 			/* write the cells to the GUI */
-			guidraw(win, di->newfont[base], &di->newchar[base], same);
+			guidraw(win, win->di->newfont[base], &win->di->newchar[base], same);
 
 			/* write the cells to the "current" image */
 			for (i = 0; i < same; i++)
 			{
-				di->curchar[base + i] = di->newchar[base + i];
-				di->curfont[base + i] = di->newfont[base + i];
+				win->di->curchar[base + i] = win->di->newchar[base + i];
+				win->di->curfont[base + i] = win->di->newfont[base + i];
 			}
 		}
 
@@ -1014,8 +935,8 @@ static void updateimage(win)
 		base = ncols * row;
 		for (i = used;
 		     i < ncols
-			&& di->curchar[base + i] == ' '
-			&& di->curfont[base + i] == 'n';
+			&& win->di->curchar[base + i] == ' '
+			&& win->di->curfont[base + i] == 'n';
 		     i++)
 		{
 		}
@@ -1038,8 +959,8 @@ static void updateimage(win)
 			guiclrtoeol(win);
 			while (i < ncols)
 			{
-				di->curchar[base + i] = ' ';
-				di->curfont[base + i] = 'n';
+				win->di->curchar[base + i] = ' ';
+				win->di->curfont[base + i] = 'n';
 				i++;
 			}
 		}
@@ -1053,7 +974,7 @@ static void updateimage(win)
 static void genlastrow(win)
 	WINDOW	win;	/* window whose last row is to be generated */
 {
-	int	i, j, base;
+	int	i, base;
 	char	*scan;
 	char	buf[25];
 
@@ -1091,9 +1012,7 @@ static void genlastrow(win)
 	if (gui->status)
 	{
 		/* yes, call it with status info */
-		(*gui->status)(win->gw,
-			win->cmdchars,
-			markline(win->state->cursor),
+		(*gui->status)(markline(win->state->cursor),
 			(*win->md->mark2col)(win, win->state->cursor, viiscmd(win)) + 1,
 			maplrnchar(o_modified(markbuffer(win->cursor)) ? '*' : ','),
 			win->state->modename);
@@ -1141,22 +1060,6 @@ static void genlastrow(win)
 				win->di->newchar[i] = ' ';
 			win->di->newfont[i] = 'n';
 		}
-
-		/* if "showcmd" then show it */
-		if (o_showcmd(win))
-		{
-			i = base + o_columns(win) - 22 - QTY(win->cmdchars);
-			for (j = 0; win->cmdchars[j]; j++, i++)
-			{
-				win->di->newchar[i] = win->cmdchars[j];
-				win->di->newfont[i] = 'n';
-			}
-			for ( ; j < QTY(win->cmdchars) - 1; j++, i++)
-			{
-				win->di->newchar[i] = ' ';
-				win->di->newfont[i] = 'n';
-			}
-		}
 	}
 
 	/* if "showstack", then show it */
@@ -1214,7 +1117,7 @@ static BOOLEAN drawquick(win)
 	col = di->curscol;
 	if (o_number(win))
 		col += 8;
-	if (o_wrap(win))
+	if (o_sidescroll(win) == 0)
 		col %= di->columns;
 	else
 		col -= di->skipped;
@@ -1235,11 +1138,13 @@ static BOOLEAN drawquick(win)
 	}
 
 	/* scan all rows at that column, looking for the cursor's offset */
-	for (i = 0, j = col; di->offsets[j] != cursoff; i++, j += di->columns)
+	for (i = 0, j = col;
+	     i < di->rows && di->offsets[j] < cursoff; /*!!! changed != to < */
+	     i++, j += di->columns)
 	{
-		if (i >= di->rows - 2)
-			return False;
 	}
+	if (i >= di->rows)
+		return False;
 
 	/* Hooray!  We can skip generating a new image */
 	di->cursrow = i;
@@ -1338,7 +1243,7 @@ void drawimage(win)
 	{
 		win->di->curscol += 8; /* since line numbers will push text to the right */
 	}
-	if (o_wrap(win))
+	if (o_sidescroll(win) == 0)
 	{
 		/* we're doing traditional vi line wrapping */
 		leftcol = 0;
@@ -1394,7 +1299,7 @@ void drawimage(win)
 	for (row = 0; thiscell < maxcell || win->di->cursrow < 0 || (win->di->cursrow > wantcurs && markoffset(next) < o_bufchars(markbuffer(win->cursor))); )
 	{
 		/* set column limits, taking o_sidescroll into consideration */
-		if (o_wrap(win))
+		if (o_sidescroll(win) == 0)
 		{
 			rightcol = maxcell - thiscell;
 		}
@@ -1465,24 +1370,6 @@ void drawimage(win)
 	win->di->bottomline = markoffset(next);
 	assert(linesshown > 0 && linesshown <= maxrow);
 
-	/* update the scrollbar */
-	if (gui->scrollbar)
-	{
-		if (win->md->move == dmnormal.move)
-		{
-			(*gui->scrollbar)(win->gw,
-				markline(marktmp(first, markbuffer(win->cursor), win->di->newline[0].start)) - 1,
-				markline(marktmp(last, markbuffer(win->cursor), markoffset(next))) - 1,
-				o_buflines(markbuffer(win->cursor)));
-		}
-		else
-		{
-			(*gui->scrollbar)(win->gw, win->di->newline[0].start,
-				markoffset(next), o_bufchars(markbuffer(next)));
-		}
-		guiflush();
-	}
-
 	/* compute insert/delete info */
 	compareimage(win);
 
@@ -1511,6 +1398,23 @@ void drawimage(win)
 
 	/* leave the cursor in the right place */
 	guimoveto(win, win->di->curscol, win->di->cursrow);
+
+	/* update the scrollbar */
+	if (gui->scrollbar)
+	{
+		if (win->md->move == dmnormal.move)
+		{
+			(*gui->scrollbar)(win->gw,
+				markline(marktmp(first, markbuffer(win->cursor), win->di->topline)) - 1,
+				markline(marktmp(last, markbuffer(win->cursor), win->di->bottomline)) - 1,
+				o_buflines(markbuffer(win->cursor)));
+		}
+		else
+		{
+			(*gui->scrollbar)(win->gw, win->di->topline,
+				win->di->bottomline, win->di->curnbytes);
+		}
+	}
 }
 
 /*----------------------------------------------------------------------------*/
