@@ -4,7 +4,6 @@
 
 #define drawscratch(di)	((di)->mustredraw = True)
 
-BEGIN_EXTERNC
 extern DRAWINFO *drawalloc P_((int rows, int columns));
 extern void drawfree P_((DRAWINFO *di));
 extern void drawimage P_((WINDOW win));
@@ -15,4 +14,3 @@ extern void drawfinish P_((WINDOW win));
 extern void drawopenedit P_((WINDOW win));
 extern void drawopencomplete P_((WINDOW win));
 extern void drawextext P_((WINDOW win, CHAR *text, int len));
-END_EXTERNC
