@@ -1,7 +1,7 @@
 /* ex.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_ex[] = "$Id: ex.c,v 2.95 1996/09/21 01:21:36 steve Exp $";
+char id_ex[] = "$Id: ex.c,v 2.78 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
@@ -31,58 +31,46 @@ static RESULT execute(EXINFO *xinf);
 /* These are the possible arguments for a command.  These may be combined
  * via the bitwise OR operator to describe arbitrary combinations of these.
  */
-#define a_Line	  0x00000001L	/* single line specifier */
-#define a_Range	  0x00000002L	/* range of lines */
-#define a_Multi	  0x00000004L	/* command character may be stuttered */
-#define a_Bang	  0x00000008L	/* '!' after the command name */
-#define a_Target  0x00000010L	/* extra line specifier after the command name */
-#define a_Lhs	  0x00000020L	/* single word after the command */
-#define a_Rhs	  0x00000040L	/* extra words (after Lhs, if Lhs allowed) */
-#define a_Buffer  0x00000080L	/* cut buffer name */
-#define a_Count	  0x00000100L	/* number of lines affected */
-#define a_Pflag	  0x00000200L	/* print flag (p, l, or #) */
-#define a_Plus	  0x00000400L	/* line offset (as for ":e +20 foo") */
-#define a_Append  0x00000800L	/* ">>" indicating append mode */
-#define a_Filter  0x00001000L	/* "!cmd", where cmd may include '|' characters */
-#define a_File	  0x00002000L	/* a single file name */
-#define a_Files	  0x00004000L	/* Zero or more filenames */
-#define a_Cmds	  0x00008000L	/* text which may contain '|' characters */
-#define a_RegExp  0x00010000L	/* regular expression */
-#define a_RegSub  0x00020000L	/* substitution text (requires RegExp) */
-#define a_Text	  0x00040000L	/* If no Rhs, following lines are part of cmd */
+#define a_Line	  0x00000001	/* single line specifier */
+#define a_Range	  0x00000002	/* range of lines */
+#define a_Multi	  0x00000004	/* command character may be stuttered */
+#define a_Bang	  0x00000008	/* '!' after the command name */
+#define a_Target  0x00000010	/* extra line specifier after the command name */
+#define a_Lhs	  0x00000020	/* single word after the command */
+#define a_Rhs	  0x00000040	/* extra words (after Lhs, if Lhs allowed) */
+#define a_Buffer  0x00000080	/* cut buffer name */
+#define a_Count	  0x00000100	/* number of lines affected */
+#define a_Pflag	  0x00000200	/* print flag (p, l, or #) */
+#define a_Plus	  0x00000400	/* line offset (as for ":e +20 foo") */
+#define a_Append  0x00000800	/* ">>" indicating append mode */
+#define a_Filter  0x00001000	/* "!cmd", where cmd may include '|' characters */
+#define a_File	  0x00002000	/* a single file name */
+#define a_Files	  0x00004000	/* Zero or more filenames */
+#define a_Cmds	  0x00008000	/* text which may contain '|' characters */
+#define a_RegExp  0x00010000	/* regular expression */
+#define a_RegSub  0x00020000	/* substitution text (requires RegExp) */
+#define a_Text	  0x00040000	/* If no Rhs, following lines are part of cmd */
 
 /* The following values indicate the default values of arguments */
-#define d_All	  0x00080000L	/* default range is all lines */
-#define d_None	  0x00100000L	/* default range is no lines (else current line) */
-#define d_File	  0x00200000L	/* default file is current file (else no default) */
+#define d_All	  0x00080000	/* default range is all lines */
+#define d_None	  0x00100000	/* default range is no lines (else current line) */
+#define d_File	  0x00200000	/* default file is current file (else no default) */
 
 /* The following indicate other quirks of commands */
-#define q_Unsafe  0x00400000L	/* command can't be executed in .exrc script */
-#define q_Autop	  0x00800000L	/* autoprint current line */
-#define q_Zero	  0x01000000L	/* allow Target or Line to be 0 */
-#define q_Exrc	  0x02000000L	/* command may be run in a .exrc file */
-#define q_Undo	  0x04000000L	/* save an "undo" version before this command */
-#define q_Custom  0x08000000L	/* save options/maps after command */
-#define q_Ex	  0x10000000L	/* only allowed in ex mode, not vi's <:> cmd */
-#define q_CtrlV	  0x20000000L	/* use ^V for quote char, instead of \ */
-#define q_MayQuit 0x40000000L	/* may cause window to disappear */
+#define q_Unsafe  0x00400000	/* command can't be executed in .exrc script */
+#define q_Autop	  0x00800000	/* autoprint current line */
+#define q_Zero	  0x01000000	/* allow Target or Line to be 0 */
+#define q_Exrc	  0x02000000	/* command may be run in a .exrc file */
+#define q_Undo	  0x04000000	/* save an "undo" version before this command */
+#define q_Custom  0x08000000	/* save options/maps after command */
+#define q_Ex	  0x10000000	/* only allowed in ex mode, not vi's <:> cmd */
+#define q_CtrlV	  0x20000000	/* use ^V for quote char, instead of \ */
 
 
 /* This array maps ex command names to command codes. The order in which
- * command names are listed below is significant --  ambiguous abbreviations
+ * command names are listed below is significant -- ambiguous abbreviations
  * are always resolved to be the first possible match.  (e.g. "r" is taken
  * to mean "read", not "rewind", because "read" comes before "rewind")
- *
- * Also, commands which share the same first letter are grouped together.
- * Similarly, within each one-letter group, the commands are ordered so that
- * the commands with the same two letters are grouped together, and those
- * groups are then divided into 3-letter groups, and so on.  This allows
- * the command list to be searched faster.
- * 
- * The comment at the start of each line below gives the shortest abbreviation.
- * HOWEVER, YOU CAN'T SIMPLY SORT THOSE ABBREVIATIONS to produce the correct
- * order for the commands.  Consider the change/chdir/calculate commands, for
- * an example of why that wouldn't work.
  */
 static struct
 {
@@ -93,113 +81,117 @@ static struct
 	long	flags;	/* command line arguments permitted/needed/used */
 }
 	cmdnames[] =
-{	   /*	 cmd name	cmd code	function	arguments */
-/*!   */{"!",		EX_BANG,	ex_bang,	a_Range | a_Cmds | d_None | q_Exrc | q_Unsafe | q_Undo			},
-/*#   */{"#",		EX_NUMBER,	ex_print,	a_Range | a_Count | a_Pflag						},
-/*&   */{"&",		EX_SUBAGAIN,	ex_substitute,	a_Range | q_Undo							},
-/*<   */{"<",		EX_SHIFTL,	ex_shift,	a_Range | a_Multi | a_Bang | a_Count | q_Autop | q_Undo			},
-/*=   */{"=",		EX_EQUAL,	ex_file,	a_Range									},
-/*>   */{">",		EX_SHIFTR,	ex_shift,	a_Range | a_Multi | a_Bang | a_Count | q_Autop | q_Undo			},
-/*@   */{"@",		EX_AT,		ex_at,		a_Buffer								},
-/*N   */{"Next",	EX_PREVIOUS,	ex_next,	a_Bang | q_Unsafe							},
-/*"   */{"\"",		EX_COMMENT,	ex_comment,	a_Cmds | q_Exrc								},
-/*a   */{"append",	EX_APPEND,	ex_append,	a_Line | a_Bang | a_Cmds | a_Text | q_Zero | q_Undo | q_Ex		},
-/*ab  */{"abbreviate",	EX_ABBR,	ex_map,		a_Bang | a_Lhs | a_Rhs | q_Exrc | q_Custom | q_Unsafe | q_CtrlV		},
-/*al  */{"all",		EX_ALL,		ex_all,		a_Bang | a_Cmds								},
-/*ar  */{"args",	EX_ARGS,	ex_args,	a_Files | q_Exrc | q_Unsafe						},
-/*b   */{"buffer",	EX_BUFFER,	ex_buffer,	a_Bang | a_Rhs								},
-/*br  */{"break",	EX_BREAK,	ex_map,		a_Bang | a_Lhs | q_CtrlV						},
-/*c   */{"change",	EX_CHANGE,	ex_append,	a_Range | a_Bang | a_Count | a_Text | q_Undo | q_Ex			},
-/*cha */{"chdir",	EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
-/*ca  */{"calculate",	EX_CALC,	ex_comment,	a_Cmds | q_Exrc								},
-/*cc  */{"cc",		EX_CC,		ex_make,	a_Bang | a_Rhs | q_Unsafe						},
-/*cd  */{"cd",		EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
-/*cl  */{"close",	EX_CLOSE,	ex_xit,		a_Bang | q_MayQuit							},
-/*co  */{"copy",	EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
-/*col */{"color",	EX_COLOR,	ex_color,	a_Rhs | q_Exrc | q_Custom						},
-/*d   */{"delete",	EX_DELETE,	ex_delete,	a_Range | a_Buffer | a_Count | a_Pflag | q_Undo | q_Autop		},
-/*di  */{"display",	EX_DISPLAY,	ex_display,	a_Rhs									},
-/*dis */{"digraph",	EX_DIGRAPH,	ex_digraph,	a_Bang | a_Rhs | q_Exrc | q_Custom					},
-/*e   */{"edit",	EX_EDIT,	ex_edit,	a_Bang | a_Plus | a_File | d_File					},
-/*ec  */{"echo",	EX_ECHO,	ex_comment,	a_Rhs | q_Exrc								},
-/*el  */{"else",	EX_ELSE,	ex_then,	a_Cmds | q_Exrc								},
-/*er  */{"errlist",	EX_ERRLIST,	ex_errlist,	a_Bang | a_File | a_Filter						},
-/*ev  */{"eval",	EX_EVAL,	ex_if,		a_Cmds | q_Exrc								},
-/*ex  */{"ex",		EX_EDIT,	ex_edit,	a_Bang | a_Plus | a_File | d_File | q_Unsafe				},
-/*f   */{"file",	EX_FILE,	ex_file,	a_File | q_Unsafe							},
-/*g   */{"global",	EX_GLOBAL,	ex_global,	a_Range | a_Bang | a_RegExp | a_Cmds | d_All | q_Undo			},
-/*go  */{"goto",	EX_GOTO,	ex_comment,	a_Line | q_Zero | q_Autop						},
-/*gu  */{"gui",		EX_GUI,		ex_gui,		a_Rhs | q_Exrc								},
-/*h   */{"help",	EX_HELP,	ex_help,	a_Lhs | a_Rhs								},
-/*i   */{"insert",	EX_INSERT,	ex_append,	a_Line | a_Bang | a_Cmds | a_Text | q_Undo | q_Ex			},
-/*if  */{"if",		EX_IF,		ex_if,		a_Cmds | q_Exrc								},
-/*j   */{"join",	EX_INSERT,	ex_join,	a_Range | a_Bang | q_Autop | q_Undo					},
-/*k   */{"k",		EX_MARK,	ex_mark,	a_Line | a_Lhs								},
-/*l   */{"list",	EX_LIST,	ex_print,	a_Range | a_Count | a_Pflag						},
-/*la  */{"last",	EX_LAST,	ex_next,	q_Unsafe								},
-/*le  */{"let",		EX_LET,		ex_set,		a_Bang | a_Cmds | q_Exrc | q_Custom					},
-/*lp  */{"lpr",		EX_LPR,		ex_lpr,		a_Range | a_Bang | a_File | a_Filter | d_All | q_Unsafe			},
-/*m   */{"move",	EX_MOVE,	ex_move,	a_Range | a_Target | q_Autop | q_Undo					},
-/*ma  */{"mark",	EX_MARK,	ex_mark,	a_Line | a_Lhs								},
-/*mak */{"make",	EX_MAKE,	ex_make,	a_Bang | a_Rhs | q_Unsafe						},
-/*map */{"map",		EX_MAP,		ex_map,		a_Bang | a_Lhs | a_Rhs | q_Exrc | q_Custom | q_Unsafe | q_CtrlV		},
-/*mk  */{"mkexrc",	EX_MKEXRC,	ex_mkexrc,	a_Bang | a_File | q_Unsafe						},
-/*n   */{"next",	EX_NEXT,	ex_next,	a_Bang | a_Files | q_Unsafe						},
-/*ne  */{"new",		EX_SNEW,	ex_split,	q_Unsafe								},
-/*no  */{"normal",	EX_NORMAL,	ex_display										},
-/*nu  */{"number",	EX_NUMBER,	ex_print,	a_Range | a_Count | a_Pflag						},
-/*o   */{"open",	EX_OPEN,	ex_edit,	a_Bang | a_Plus | a_File						},
-/*p   */{"print",	EX_PRINT,	ex_print,	a_Range | a_Count | a_Pflag						},
-/*pre */{"previous",	EX_PREVIOUS,	ex_next,	a_Bang | q_Unsafe							},
-/*pres*/{"preserve",	EX_PRESERVE,	ex_qall,	d_None | q_Unsafe | q_MayQuit						},
-/*po  */{"pop",		EX_POP,		ex_pop,		a_Bang | q_Unsafe							},
-/*pu  */{"put",		EX_PUT,		ex_put,		a_Line | q_Zero | a_Buffer | q_Undo					},
-/*q   */{"quit",	EX_QUIT,	ex_xit,		a_Bang | q_MayQuit							},
-/*qa  */{"qall",	EX_QALL,	ex_qall,	a_Bang | q_MayQuit							},
-/*r   */{"read",	EX_READ,	ex_read,	a_Line | a_File | a_Filter | q_Zero | q_Undo				},
-/*red */{"redo",	EX_REDO,	ex_undo,	a_Lhs | q_Autop								},
-/*rew */{"rewind",	EX_REWIND,	ex_next,	a_Bang | q_Unsafe							},
-/*s   */{"substitute",	EX_SUBSTITUTE,	ex_substitute,	a_Range|a_RegExp|a_RegSub|a_Rhs|a_Count|a_Pflag|q_Autop|q_Undo|q_Exrc	},
-/*sN  */{"sNext",	EX_SPREVIOUS,	ex_next,	q_Unsafe								},
-/*sa  */{"sall",	EX_SALL,	ex_sall,	q_Unsafe								},
-/*saf */{"safer",	EX_SAFER,	ex_source,	a_Bang | a_File | q_Exrc						},
-/*se  */{"set",		EX_SET,		ex_set,		a_Bang | a_Rhs | q_Exrc | q_Custom					},
-/*sh  */{"shell",	EX_SHELL,	ex_suspend,	q_Unsafe								},
-/*sl  */{"slast",	EX_SLAST,	ex_next,	q_Unsafe								},
-/*sn  */{"snext",	EX_SNEXT,	ex_next,	a_Files									},
-/*snew*/{"snew",	EX_SNEW,	ex_split,	q_Unsafe								},
-/*so  */{"source",	EX_SOURCE,	ex_source,	a_Bang | a_File | q_Exrc						},
-/*sp  */{"split",	EX_SPLIT,	ex_split,	a_Line | a_File								},
-/*sr  */{"srewind",	EX_SREWIND,	ex_next,	a_Bang | q_Unsafe							},
-/*st  */{"stop",	EX_STOP,	ex_suspend,	a_Bang | q_Unsafe							},
-/*sta */{"stag",	EX_STAG,	ex_tag,		a_Lhs									},
-/*stac*/{"stack",	EX_STACK,	ex_stack,	d_None									},
-/*su  */{"suspend",	EX_SUSPEND,	ex_suspend,	a_Bang | q_Unsafe							},
-/*t   */{"to",		EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
-/*ta  */{"tag",		EX_TAG,		ex_tag,		a_Bang | a_Lhs | q_Unsafe						},
-/*th  */{"then",	EX_THEN,	ex_then,	a_Cmds | q_Exrc			 					},
-/*u   */{"undo",	EX_UNDO,	ex_undo,	a_Lhs | q_Autop								},
-/*una */{"unabbreviate",EX_UNABBR,	ex_map,		a_Bang | a_Lhs | q_Exrc | q_Custom | q_CtrlV				},
-/*unb */{"unbreak",	EX_UNBREAK,	ex_map,		a_Bang | a_Lhs | q_CtrlV						},
-/*unm */{"unmap",	EX_UNMAP,	ex_map,		a_Bang | a_Lhs | q_Exrc | q_Custom | q_CtrlV				},
-/*v   */{"vglobal",	EX_VGLOBAL,	ex_global,	a_Range | a_Bang | a_RegExp | a_Cmds | d_All | q_Undo			},
-/*ve  */{"version",	EX_VERSION,	ex_version,	q_Exrc									},
-/*vi  */{"visual",	EX_VISUAL,	ex_edit,	a_Bang | a_Plus | a_File						},
-/*w   */{"write",	EX_WRITE,	ex_write,	a_Range | a_Bang | a_Append | a_File | a_Filter | d_All | q_Unsafe	},
-/*wi  */{"window",	EX_WINDOW,	ex_window,	a_Lhs									},
-/*wn  */{"wnext",	EX_WNEXT,	ex_next,	a_Bang | q_Unsafe							},
-/*wq  */{"wquit",	EX_WQUIT,	ex_xit,		a_Bang | a_File | d_File | q_MayQuit					},
-/*x   */{"xit",		EX_XIT,		ex_xit,		a_Bang | a_File  | q_MayQuit						},
-/*y   */{"yank",	EX_YANK,	ex_delete,	a_Range | a_Buffer | a_Count						},
-/*z   */{"z",		EX_Z,		ex_z,		a_Line | a_Rhs								},
-/*~   */{"~",		EX_SUBAGAIN,	ex_substitute,	a_Range | q_Undo							},
+{   /*	cmd name	cmd code	function	arguments */
+	/* The following can be abbreviated to 1 character... */
+	{"append",	EX_APPEND,	ex_append,	a_Line | a_Bang | a_Cmds | a_Text | q_Zero | q_Undo | q_Ex		},
+	{"buffer",	EX_BUFFER,	ex_buffer,	a_Bang | a_Rhs								},
+	{"change",	EX_CHANGE,	ex_append,	a_Range | a_Bang | a_Count | a_Text | q_Undo | q_Ex			},
+	{"delete",	EX_DELETE,	ex_delete,	a_Range | a_Buffer | a_Count | a_Pflag | q_Undo | q_Autop		},
+	{"edit",	EX_EDIT,	ex_edit,	a_Bang | a_Plus | a_File | d_File					},
+	{"file",	EX_FILE,	ex_file,	a_File | q_Unsafe							},
+	{"global",	EX_GLOBAL,	ex_global,	a_Range | a_Bang | a_RegExp | a_Cmds | d_All | q_Undo			},
+	{"help",	EX_HELP,	ex_help,	a_Lhs | a_Rhs								},
+	{"insert",	EX_INSERT,	ex_append,	a_Line | a_Bang | a_Cmds | a_Text | q_Undo | q_Ex			},
+	{"join",	EX_INSERT,	ex_join,	a_Range | a_Bang | q_Autop | q_Undo					},
+	{"k",		EX_MARK,	ex_mark,	a_Line | a_Lhs								},
+	{"list",	EX_LIST,	ex_print,	a_Range | a_Count | a_Pflag						},
+	{"move",	EX_MOVE,	ex_move,	a_Range | a_Target | q_Autop | q_Undo					},
+	{"next",	EX_NEXT,	ex_next,	a_Bang | a_Files | q_Unsafe						},
+	{"Next",	EX_PREVIOUS,	ex_next,	a_Bang | q_Unsafe							},
+	{"open",	EX_OPEN,	ex_edit,	a_Bang | a_Plus | a_File						},
+	{"print",	EX_PRINT,	ex_print,	a_Range | a_Count | a_Pflag						},
+	{"quit",	EX_QUIT,	ex_xit,		a_Bang									},
+	{"read",	EX_READ,	ex_read,	a_Line | a_File | a_Filter | q_Zero | q_Undo				},
+	{"substitute",	EX_SUBSTITUTE,	ex_substitute,	a_Range|a_RegExp|a_RegSub|a_Rhs|a_Count|a_Pflag|q_Autop|q_Undo|q_Exrc	},
+	{"to",		EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
+	{"undo",	EX_UNDO,	ex_undo,	a_Lhs | q_Autop								},
+	{"vglobal",	EX_VGLOBAL,	ex_global,	a_Range | a_Bang | a_RegExp | a_Cmds | d_All | q_Undo			},
+	{"write",	EX_WRITE,	ex_write,	a_Range | a_Bang | a_Append | a_File | a_Filter | d_All | q_Unsafe	},
+	{"xit",		EX_XIT,		ex_xit,		a_Bang | a_File 							},
+	{"yank",	EX_YANK,	ex_delete,	a_Range | a_Buffer | a_Count						},
+	{"z",		EX_Z,		ex_z,		a_Line | a_Rhs								},
+
+	{"!",		EX_BANG,	ex_bang,	a_Range | a_Cmds | d_None | q_Exrc | q_Unsafe | q_Undo			},
+	{"\"",		EX_COMMENT,	ex_comment,	a_Cmds | q_Exrc								},
+	{"#",		EX_NUMBER,	ex_print,	a_Range | a_Count | a_Pflag						},
+	{"<",		EX_SHIFTL,	ex_shift,	a_Range | a_Multi | a_Bang | a_Count | q_Autop | q_Undo			},
+	{">",		EX_SHIFTR,	ex_shift,	a_Range | a_Multi | a_Bang | a_Count | q_Autop | q_Undo			},
+	{"=",		EX_EQUAL,	ex_file,	a_Range									},
+	{"&",		EX_SUBAGAIN,	ex_substitute,	a_Range | q_Undo							},
+	{"~",		EX_SUBAGAIN,	ex_substitute,	a_Range | q_Undo							},
+	{"@",		EX_AT,		ex_at,		a_Buffer								},
+
+	/* The following can be abbreviated to 2 characters... */
+	{"abbreviate",	EX_ABBR,	ex_map,		a_Bang | a_Lhs | a_Rhs | q_Exrc | q_Custom | q_Unsafe			},
+	{"all",		EX_ALL,		ex_all,		a_Bang | a_Cmds								},
+	{"args",	EX_ARGS,	ex_args,	a_Files | q_Exrc | q_Unsafe						},
+	{"break",	EX_BREAK,	ex_map,		a_Bang | a_Lhs | q_CtrlV						},
+	{"calculate",	EX_CALC,	ex_comment,	a_Cmds | q_Exrc								},
+	{"cc",		EX_CC,		ex_make,	a_Bang | a_Rhs | q_Unsafe						},
+	{"cd",		EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
+	{"close",	EX_CLOSE,	ex_xit,		a_Bang									},
+	{"copy",	EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
+	{"display",	EX_DISPLAY,	ex_display,	a_Rhs									},
+	{"echo",	EX_ECHO,	ex_comment,	a_Rhs | q_Exrc								},
+	{"else",	EX_ELSE,	ex_then,	a_Cmds | q_Exrc								},
+	{"errlist",	EX_ERRLIST,	ex_errlist,	a_Bang | a_File								},
+	{"ex",		EX_EDIT,	ex_edit,	a_Bang | a_Plus | a_File | d_File | q_Unsafe				},
+	{"eval",	EX_EVAL,	ex_if,		a_Cmds | q_Exrc								},
+	{"goto",	EX_GOTO,	ex_comment,	a_Line | q_Zero | q_Autop						},
+	{"gui",		EX_GUI,		ex_gui,		a_Rhs | q_Exrc								},
+	{"if",		EX_IF,		ex_if,		a_Cmds | q_Exrc								},
+	{"last",	EX_LAST,	ex_next,	q_Unsafe								},
+	{"let",		EX_LET,		ex_set,		a_Bang | a_Cmds | q_Exrc | q_Custom					},
+	{"lpr",		EX_LPR,		ex_lpr,		a_Range | a_Bang | a_File | a_Filter | d_All | q_Unsafe			},
+	{"mark",	EX_MARK,	ex_mark,	a_Line | a_Lhs								},
+	{"mkexrc",	EX_MKEXRC,	ex_mkexrc,	a_Bang | a_File | q_Unsafe						},
+	{"normal",	EX_NORMAL,	ex_display										},
+	{"number",	EX_NUMBER,	ex_print,	a_Range | a_Count | a_Pflag						},
+	{"pop",		EX_POP,		ex_pop,		a_Bang | q_Unsafe							},
+	{"put",		EX_PUT,		ex_put,		a_Line | q_Zero | a_Buffer | q_Undo					},
+	{"qall",	EX_QALL,	ex_qall,	a_Bang									},
+	{"sall",	EX_SALL,	ex_sall,	q_Unsafe								},
+	{"set",		EX_SET,		ex_set,		a_Bang | a_Rhs | q_Exrc | q_Custom					},
+	{"shell",	EX_SHELL,	ex_suspend,	q_Unsafe								},
+	{"slast",	EX_SLAST,	ex_next,	q_Unsafe								},
+	{"source",	EX_SOURCE,	ex_source,	a_Bang | a_File | q_Exrc						},
+	{"snext",	EX_SNEXT,	ex_next,	a_Files									},
+	{"sNext",	EX_SPREVIOUS,	ex_next,	q_Unsafe								},
+	{"split",	EX_SPLIT,	ex_split,	a_Line | a_File								},
+	{"stop",	EX_STOP,	ex_suspend,	a_Bang | q_Unsafe							},
+	{"tag",		EX_TAG,		ex_tag,		a_Bang | a_Lhs | q_Unsafe						},
+	{"then",	EX_THEN,	ex_then,	a_Cmds | q_Exrc			 					},
+	{"version",	EX_VERSION,	ex_version,	q_Exrc									},
+	{"visual",	EX_VISUAL,	ex_edit,	a_Bang | a_Plus | a_File						},
+	{"window",	EX_WINDOW,	ex_window,	a_Lhs									},
+	{"wnext",	EX_WNEXT,	ex_next,	a_Bang | q_Unsafe							},
+	{"wquit",	EX_WQUIT,	ex_xit,		a_Bang | a_File | d_File						},
+
+	/* The following can be abbreviated to 3 characters... */
+	{"chdir",	EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
+	{"color",	EX_COLOR,	ex_color,	a_Rhs | q_Exrc | q_Custom						},
+	{"digraph",	EX_DIGRAPH,	ex_digraph,	a_Bang | a_Rhs | q_Exrc | q_Custom					},
+	{"make",	EX_MAKE,	ex_make,	a_Bang | a_Rhs | q_Unsafe						},
+	{"map",		EX_MAP,		ex_map,		a_Bang | a_Lhs | a_Rhs | q_Exrc | q_Custom | q_Unsafe | q_CtrlV		},
+	{"new",		EX_SNEW,	ex_split,	q_Unsafe								},
+	{"previous",	EX_PREVIOUS,	ex_next,	a_Bang | q_Unsafe							},
+	{"redo",	EX_REDO,	ex_undo,	a_Lhs | q_Autop								},
+	{"rewind",	EX_REWIND,	ex_next,	a_Bang | q_Unsafe							},
+	{"safer",	EX_SAFER,	ex_source,	a_Bang | a_File | q_Exrc						},
+	{"snew",	EX_SNEW,	ex_split,	q_Unsafe								},
+	{"sNext",	EX_SPREVIOUS,	ex_next,	q_Unsafe								},
+	{"srewind",	EX_SREWIND,	ex_next,	a_Bang | q_Unsafe							},
+	{"stag",	EX_STAG,	ex_tag,		a_Lhs									},
+	{"suspend",	EX_SUSPEND,	ex_suspend,	a_Bang | q_Unsafe							},
+	{"unbreak",	EX_UNBREAK,	ex_map,		a_Bang | a_Lhs | q_CtrlV						},
+	{"unmap",	EX_UNMAP,	ex_map,		a_Bang | a_Lhs | q_Exrc | q_Custom | q_CtrlV				},
+	{"unabbreviate",EX_UNABBR,	ex_map,		a_Bang | a_Lhs | q_Exrc | q_Custom					},
+
+	/* The following can be abbreviated to 4 characters... */
+	{"stack",	EX_STACK,	ex_stack,	d_None									},
+	{"preserve",	EX_PRESERVE,	ex_qall,	d_None									}
 };
 
-
-/* This variable is used for detecting nested global statements */
-static int 	globaldepth;
-
-
 /* This function discards info from an EXINFO struct.  The struct itself
  * is not freed, since it is usually just a local variable in some function.
  */
@@ -507,15 +499,8 @@ static BOOLEAN parseregexp(refp, xinf, flags)
 		}
 		if (!xinf->lhs)
 		{
-			/* zero-length string, if nothing else */
 			xinf->lhs = (CHAR *)safealloc(1, sizeof(CHAR));
 		}
-
-		/* consume the closing delimiter */
-		if (*refp && **refp == delim)
-		{
-			scannext(refp);
-		}
 	}
 
 	return True;
@@ -536,7 +521,6 @@ BOOLEAN exparseaddress(refp, xinf)
 	long	start;	/* where search stared, so we can detect wrap */
 	long	buflines;/* number of lines in buffer */
 	MARKBUF	m;	/* start of a line */
-	CHAR	ch;
 
 	/* if nothing, do nothing */
 	if (!*refp)
@@ -545,7 +529,6 @@ BOOLEAN exparseaddress(refp, xinf)
 	/* find the default line number */
 	bi = bufbufinfo(markbuffer(&xinf->defaddr));
 	lnum = markline(&xinf->defaddr);
-	xinf->fromoffset = 0;
 
 	/* parse the address */
 	switch (**refp)
@@ -564,18 +547,6 @@ BOOLEAN exparseaddress(refp, xinf)
 			return False;
 		}
 
-		/* allow the visual 'n' and 'N' commands to search for the
-		 * same regexp.
-		 */
-		if (searchre)
-		{
-			safefree(searchre);
-		}
-		searchre = xinf->re;
-		searchforward = (BOOLEAN)(delta > 0);
-		searchhasdelta = False;
-		xinf->re = NULL;
-
 		/* search for the regular expression */
 		start = lnum;
 		buflines = o_buflines(markbuffer(&xinf->defaddr));
@@ -602,7 +573,7 @@ BOOLEAN exparseaddress(refp, xinf)
 			}
 
 			/* see if this line contains the regexp */
-			if (regexec(searchre, marktmp(m, markbuffer(&xinf->defaddr), lowline(bi, lnum)), True))
+			if (regexec(xinf->re, marktmp(m, markbuffer(&xinf->defaddr), lowline(bi, lnum)), True))
 			{
 				delta = 0;
 			}
@@ -625,14 +596,13 @@ BOOLEAN exparseaddress(refp, xinf)
 		} while (delta != 0);
 
 		/* If we get here, then we've found a match, and lnum is set
-		 * to its line number.  Good!
+		 * to its line number.  Good!  Now discard the compiled regexp.
 		 */
-		xinf->fromoffset = (searchre->leavep >= 0 ? searchre->leavep : searchre->startp[0]);
+		safefree(xinf->re);
+		xinf->re = (regexp *)0;
 		break;
 
 	  case '\'':
-	  case '`':
-		ch = **refp;
 		if (!scannext(refp) || **refp < 'a' || **refp > 'z')
 		{
 			msg(MSG_ERROR, "bad mark name");
@@ -653,8 +623,6 @@ BOOLEAN exparseaddress(refp, xinf)
 			xinf->defaddr = *namedmark[**refp - 'a'];
 		}
 		lnum = markline(namedmark[**refp - 'a']);
-		if (ch == '`')
-			xinf->fromoffset = markoffset(namedmark[**refp - 'a']);
 		scannext(refp);
 		break;
 
@@ -693,9 +661,6 @@ BOOLEAN exparseaddress(refp, xinf)
 	skipwhitespace(refp);
 	if (*refp && (**refp == '+' || **refp == '-'))
 	{
-		/* delta cancels implies whole-line addressing */
-		xinf->fromoffset = 0;
-
 		/* find the sign & magnitude of the delta */
 		sign = **refp;
 		for (delta = 1; scannext(refp) && **refp == sign; delta++)
@@ -736,7 +701,6 @@ static BOOLEAN parsecommandname(refp, xinf)
 	CHAR	**refp;	/* pointer to the (CHAR *) used for scanning command */
 	EXINFO	*xinf;	/* info about the command being parsed */
 {
-	BOOLEAN	anymatch;
 	int	matches;	/* number of commands that match so far */
 	int	firstmatch;	/* first command that matched */
 	char	cmdname[20];	/* command name */
@@ -763,28 +727,16 @@ static BOOLEAN parsecommandname(refp, xinf)
 	 */
 	len = 0;
 	start = scanmark(refp);
-	firstmatch = 0;
-	anymatch = False;
+	firstmatch = -1;
 	do
 	{
 		/* copy the first (or next) character of the name into a buffer */
 		cmdname[len++] = **refp;
 
 		/* see how many commands match this command so far */
-		for (matches = 0, i = firstmatch; i < QTY(cmdnames); i++)
+		for (matches = i = 0; i < QTY(cmdnames); i++)
 		{
-			/* In the cmdnames[] array, commands with the same
-			 * initial letters are grouped together.  Have we
-			 * reached the end of the group that interests us?
-			 */
-			if (len > 1 && (cmdnames[i].name[0] != cmdname[0] || strncmp(cmdnames[i].name, cmdname, (size_t)(len - 1))))
-			{
-				/* no more matches are possible -- we reached
-				 * the end of this (len-1) letter group.
-				 */
-				break;
-			}
-			if ((CHAR)cmdnames[i].name[len - 1] == **refp)
+			if (!strncmp(cmdnames[i].name, cmdname, (size_t)len))
 			{
 				/* Partial match, but keep looking.  If this
 				 * was the first match with this length then
@@ -794,7 +746,6 @@ static BOOLEAN parsecommandname(refp, xinf)
 				if (matches == 1)
 				{
 					firstmatch = i;
-					anymatch = True;
 				}
 			}
 		}
@@ -806,8 +757,8 @@ static BOOLEAN parsecommandname(refp, xinf)
 	 * so we need to adjust the position of the *refp.
 	 */
 	if (matches == 0
-		&& anymatch
-		&& (strlen(cmdnames[firstmatch].name) == (unsigned)(len - 1)
+		&& firstmatch >= 0
+		&& (strlen(cmdnames[firstmatch].name) == len - 1
 			|| !*refp
 			|| !isalpha(**refp)))
 	{
@@ -984,15 +935,9 @@ NoFlag:	/* see if we're supposed to autoprint this command.  If so, then
 	 */
 	if ((flags & q_Autop) != 0
 	 && o_autoprint
-	 && globaldepth == 0
 	 && xinf->window
-	 && xinf->window->state
-	 && (xinf->window->state->flags & (ELVIS_POP|ELVIS_1LINE)) == 0
-	 && xinf->window->state->enter
-	 && markbuffer(scanmark(refp)) == buffind(toCHAR(EX_BUF)))
+	 && (xinf->window->state->flags & (ELVIS_POP|ELVIS_ONCE)) == 0)
 	{
-		if (pflag == PF_NONE)
-			pflag = PF_PRINT;
 		xinf->pflag = pflag;
 	}
 }
@@ -1148,7 +1093,6 @@ static BOOLEAN parsecmds(refp, xinf, flags)
 	long	flags;	/* bitmap of command attributes */
 {
 	BOOLEAN	shell;	/* is this command for a shell? */
-	BOOLEAN	bol;	/* at front of a line? */
 	CHAR	*scan;
 	CHAR	ch;
 	MARKBUF	end;
@@ -1162,38 +1106,6 @@ static BOOLEAN parsecmds(refp, xinf, flags)
 	/* determine whether this command is for a shell or for ex */
 	shell = (BOOLEAN)(xinf->command == EX_BANG || (flags & a_Filter) != 0);
 
-	/* skip whitespace */
-	skipwhitespace(refp);
-
-	/* ex commands allow a { ... } syntax */
-	if (!shell && *refp && **refp == '{')
-	{
-		/* collect characters up to next } character which appears
-		 * at the front of a line.  Ignore blank lines.
-		 */
-		for (bol = True; scannext(refp) && !(bol && **refp == '}'); )
-		{
-			if (!bol || **refp != '\n')
-				buildCHAR(&xinf->rhs, **refp);
-			if (**refp == '\n')
-				bol = True;
-			if (!isspace(**refp))
-				bol = False;
-		}
-
-		/* if we hit end without finding } then complain. */
-		if (!*refp)
-		{
-			msg(MSG_ERROR, "missing }");
-			return False;
-		}
-
-		/* eat the closing } */
-		scannext(refp);
-
-		return True;
-	}
-
 	/* collect characters up to end of command */
 	while (*refp && **refp && **refp != '\n')
 	{
@@ -1259,7 +1171,7 @@ static BOOLEAN parsecmds(refp, xinf, flags)
 				/* fall through for all but \@ ... */
 
 			  default:
-				buildCHAR(&xinf->rhs, **refp);
+				buildCHAR(&xinf->rhs, ch);
 				scan = xinf->rhs;
 			}
 
@@ -1316,7 +1228,7 @@ BOOLEAN exaddfilearg(file, nfiles, filename, wild)
 	char	*match;		/* a name matching a wildcard */
 	char	**old;		/* previous value of xinf->file pointer */
 	int	start;		/* index into xinf->file of wildcard's expansion */
-	BOOLEAN	mustfree=False;	/* if True, then free "match" before returning*/
+	CHAR	*args[2];	/* arguments for environment variable expansion */
 	int	i;
 
 
@@ -1324,16 +1236,20 @@ BOOLEAN exaddfilearg(file, nfiles, filename, wild)
 	if (wild)
 	{
 		/* expand any environment variables */
-		filename = tochar8(calculate(toCHAR(filename), NULL, True));
+		args[0] = toCHAR(dirpath("a", "b") + 1);
+			assert(args[0] && CHARlen(args[0]) > 0);
+			args[0][CHARlen(args[0]) - 1] = '\0';
+		args[1] = NULL;
+		filename = tochar8(calculate(toCHAR(filename), args, True));
 		if (!filename)
 			return False;
 
 		/* If this operating system uses backslashes between names,
 		 * then replace any forward slashes with backslashes.  This is
-		 * intended to allow MS-DOS users to type forward slashes in
+		 * intended to allow MS-DOS uses to type forward slashes in
 		 * their filenames, if they so prefer.
 		 */
-		if (dirpath("a", "b")[1] == '\\')
+		if (args[0][0] == '\\')
 		{
 			for (i = 0; filename[i]; i++)
 			{
@@ -1347,10 +1263,11 @@ BOOLEAN exaddfilearg(file, nfiles, filename, wild)
 		/* replace ~ with the name of the home directory */
 		if (filename[0] == '~' && !isalpha(filename[1]))
 		{
-			filename = strdup(filename[1]
-				    ? dirpath(tochar8(o_home), &filename[2])
-				    : tochar8(o_home));
-			mustfree = True;
+			match = safealloc((int)(CHARlen(o_home) + strlen(filename)), sizeof(CHAR));
+			strcpy(match, tochar8(o_home));
+			strcat(match, filename + 1);
+			filename = match;
+			args[1] = (CHAR *)filename; /* the "must-free" flag */
 		}
 
 		/* If there are any matches... */
@@ -1376,7 +1293,7 @@ BOOLEAN exaddfilearg(file, nfiles, filename, wild)
 		}
 
 		/* if necessary, free the filename */
-		if (mustfree)
+		if (args[1])
 		{
 			safefree(filename);
 		}
@@ -1384,7 +1301,7 @@ BOOLEAN exaddfilearg(file, nfiles, filename, wild)
 	else /* literal filename */
 	{
 		/* [re-]allocate the *files array if necessary.  Note that
-		 * we've arranged it so there will always be at least one
+		 * we've arrange it so there will always be at least one
 		 * NULL pointer at the end of the list.
 		 */
 		if (*nfiles == 0 || (*nfiles + 1) % FILEGRAN == 0)
@@ -1437,12 +1354,7 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 
 	/* An initial backslash could have been used to quote a ! or > at
 	 * the start of a filename.  If the first character is backslash,
-	 * and the second is either ! or > then skip the backslash.  Also,
-	 * if the second is another backslash and backslash isn't used as
-	 * the directory separator on this operating system, then skip the
-	 * backslash; this should allow "\\foo" to be "\foo" under UNIX,
-	 * while still allowing "\\machine\dir\file" to be parsed as a UNC
-	 * name under Win32.
+	 * and the second is either ! or > or another \ then skip the backslash.
 	 */
 	if (*refp && **refp == '\\')
 	{
@@ -1452,7 +1364,7 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 			msg(MSG_ERROR, "oops");
 			return False;
 		}
-		if (**refp != '!' && **refp != '>' && (**refp != '\\' || dirpath("a", "b")[1] == '\\'))
+		if (**refp != '\\' && **refp != '!' && **refp != '>')
 		{
 			scanprev(refp);
 		}
@@ -1621,8 +1533,6 @@ static RESULT parse(win, refp, xinf)
 	long	rngto;	/* "to" line number of range */
 	long	flags;	/* bitmap of command attributes */
 	BOOLEAN	sel;	/* did address come from selection? */
-	CHAR	*p2;	/* a second scanning variable */
-	RESULT	result;	/* result of parsing */
 	int	i;
 
 	/* set defaults */
@@ -1638,20 +1548,6 @@ static RESULT parse(win, refp, xinf)
 		scannext(refp);
 	}
 
-	/* If no command was entered, then assume the command line should be
-	 * "+p", so the user can simply hit <Enter> to step through the file.
-	 * Only do this for interactively entered commands, though!
-	 */
-	skipwhitespace(refp);
-	if ((!*refp || **refp == '\n')
-		&& markbuffer(&orig) == buffind(toCHAR(EX_BUF)))
-	{
-		scanstring(&p2, toCHAR("+p"));
-		result = parse(win, &p2, xinf);
-		scanfree(&p2);
-		return result;
-	}
-
 	/* parse the window id */
 	if (win)
 	{
@@ -1790,7 +1686,7 @@ static RESULT parse(win, refp, xinf)
 		if (sel)
 		{
 			/* visible selection; pretend no addresses were given */
-			xinf->anyaddr = False;
+			xinf->anyaddr = 0;
 		}
 		else
 		{
@@ -2036,8 +1932,8 @@ static RESULT execute(xinf)
 	BUFFER		custom;	/* the CUSTOM_BUF buffer */
 	MARKBUF		top, bottom;
 	RESULT		ret;
+	static int 	globaldepth;
 	BUFFER		origdef;/* original value of bufdefault */
-	struct state_s	*state;
 
 	/* If we need to save an "undo" version of the buffer, then
 	 * remember that fact.
@@ -2058,15 +1954,6 @@ static RESULT execute(xinf)
 		globaldepth++;
 	}
 
-	/* if quit command, then cause "wrote..." messages to be displayed
-	 * as INFO instead of the normal STATUS.  This is so they'll be queued
-	 * and can be printed someplace else after the window is closed.
-	 */
-	if (cmdnames[xinf->cmdidx].flags & q_MayQuit)
-	{
-		bufmsgtype = MSG_INFO;
-	}
-
 	/* if the command's default buffer isn't the window's default buffer,
 	 * then make the command's default buffer be the one used by :set.
 	 */
@@ -2098,11 +1985,6 @@ static RESULT execute(xinf)
 	/* move the cursor to where it wants to be */
 	if (xinf->newcurs)
 	{
-		/* find the window's main state */
-		for (state = xinf->window->state; state->acton; state = state->acton)
-		{
-		}
-
 		/* if we're switching buffers, then we need to
 		 * switch names, too.
 		 */
@@ -2114,21 +1996,14 @@ static RESULT execute(xinf)
 			{
 				(*gui->retitle)(xinf->window->gw, tochar8(o_bufname(markbuffer(xinf->newcurs))));
 			}
-			marksetbuffer(state->cursor, markbuffer(xinf->newcurs));
-			marksetbuffer(state->top, markbuffer(xinf->newcurs));
-			marksetbuffer(state->bottom, markbuffer(xinf->newcurs));
+			marksetbuffer(xinf->window->cursor, markbuffer(xinf->newcurs));
 			dispset(xinf->window, tochar8(o_bufdisplay(markbuffer(xinf->newcurs))));
 		}
 
 		/* other stuff is easy */
-		marksetoffset(state->cursor, markoffset(xinf->newcurs));
-		marksetoffset(state->top, markoffset(xinf->newcurs));
-		marksetoffset(state->bottom, markoffset(xinf->newcurs));
+		marksetoffset(xinf->window->cursor, markoffset(xinf->newcurs));
 		markfree(xinf->newcurs);
-		xinf->window->wantcol = state->wantcol = (*xinf->window->md->mark2col)(xinf->window, xinf->window->cursor, True);
-
-		assert(markbuffer(state->cursor) == markbuffer(state->top));
-		assert(markbuffer(state->cursor) == markbuffer(state->bottom));
+		xinf->window->wantcol = dispmark2col(xinf->window);
 	}
 
 	/* if the new cursor position is off the end of the buffer,
@@ -2321,8 +2196,6 @@ CHAR *exname(name)
 		  case '&':	return toCHAR("AMP");
 		  case '~':	return toCHAR("TILDE");
 		  case '@':	return toCHAR("AT");
-		  case '(':	return toCHAR("OPEN"); /* not a real command */
-		  case '{':	return toCHAR("OCUR"); /* not a real command */
 		}
 	}
 
@@ -2481,3 +2354,5 @@ long exprintlines(win, mark, qty, pflag)
 	scanfree(&scan);
 	return last;
 }
+
+
