@@ -1,7 +1,7 @@
 /* exaction.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_exaction[] = "$Id: exaction.c,v 2.59 1996/09/04 18:30:16 steve Exp $";
+char id_exaction[] = "$Id: exaction.c,v 2.56 1996/06/03 17:24:28 steve Exp $";
 
 #include "elvis.h"
 
@@ -188,7 +188,7 @@ RESULT ex_window(xinf)
 		{
 		}
 	}
-	else if (xinf->lhs[0] == '+')
+	else if (xinf->lhs[0] == '+' && !xinf->lhs[1])
 	{
 		/* move up 1 window */
 		for (win = NULL; winofbuf(win, NULL) != xinf->window; )
@@ -197,21 +197,11 @@ RESULT ex_window(xinf)
 			if (!win)
 				break;
 		}
-		if (!win && xinf->lhs[1])
-		{
-			for (win = winofbuf(NULL, NULL);
-			     winofbuf(win, NULL);
-			     win = winofbuf(win, NULL))
-			{
-			}
-		}
 	}
-	else if (xinf->lhs[0] == '-')
+	else if (xinf->lhs[0] == '-' && !xinf->lhs[1])
 	{
 		/* move down 1 window */
 		win = winofbuf(xinf->window, NULL);
-		if (!win && xinf->lhs[1])
-			win = winofbuf(NULL, NULL);
 	}
 	else
 	{
@@ -354,16 +344,6 @@ RESULT	ex_edit(xinf)
 			o_previousfileline = 1L;
 		}
 
-		/* If the buffer we'll be loading already exists, and has been
-		 * modified, then fail unless "!" was given.
-		 */
-		newbuf = buffind(toCHAR(xinf->file[0]));
-		if (newbuf && o_modified(newbuf) && !xinf->bang)
-		{
-			msg(MSG_ERROR, "[S]$1 modified, not saved", o_bufname(newbuf));
-			return RESULT_ERROR;
-		}
-
 		/* None of these commands are ever supposed to save the old
 		 * buffer.  When we eventually switch to the new buffer,
 		 * the old buffer will be unloaded, so if it has been
@@ -377,7 +357,7 @@ RESULT	ex_edit(xinf)
 				o_modified(oldbuf) = False;
 			else if (!o_autowrite || !bufsave(oldbuf, False, False))
 			{
-				msg(MSG_ERROR, "[S]$1 modified, not saved", o_bufname(oldbuf));
+				msg(MSG_ERROR, "[S]modified, not saved", o_bufname(oldbuf));
 				return RESULT_ERROR;
 			}
 		}
@@ -980,7 +960,6 @@ RESULT	ex_tag(xinf)
 	CHAR	*fromtag;
 	MARK	tagdefn;
 	RESULT	result = RESULT_COMPLETE;
-	BUFFER	oldbuf;
 	int	i;
 
 	assert(xinf->command == EX_TAG || xinf->command == EX_STAG);
@@ -1018,30 +997,6 @@ RESULT	ex_tag(xinf)
 		goto Finish;
 	}
 
-	/* if switching buffers, and the current buffer has been modified,
-	 * and no other window is also showing this buffer, then either
-	 * complain or write the buffer.
-	 */
-	oldbuf = markbuffer(xinf->window->cursor);
-	if (!xinf->bang
-	 && markbuffer(tagdefn) != oldbuf
-	 && o_modified(oldbuf)
-	 && winofbuf(NULL, oldbuf) == xinf->window
-	 && winofbuf(xinf->window, oldbuf) == NULL)
-	{
-		if (!o_autowrite)
-		{
-			msg(MSG_ERROR, "[S]$1 modified, not saved", o_bufname(oldbuf));
-			result = RESULT_ERROR;
-			goto Finish;
-		}
-		else if (!bufsave(oldbuf, False, False))
-		{
-			result = RESULT_ERROR;
-			goto Finish;
-		}
-	}
-
 	/* push the current cursor position and display mode onto tag stack */
 	if (o_tagstack)
 	{
