@@ -1,7 +1,7 @@
 /* exconfig.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_exconfig[] = "$Id: exconfig.c,v 2.50 1996/09/21 02:12:31 steve Exp $";
+char id_exconfig[] = "$Id: exconfig.c,v 2.44 1996/05/27 07:57:10 steve Exp $";
 
 #include "elvis.h"
 
@@ -76,7 +76,7 @@ static struct
 	CHAR	*bg;	/* name of background color */
 } colortbl[] =
 {
-	{"normal"}, {"bold"}, {"emphasized"}, {"italic"},
+	{"normal"}, {"bold"}, {"emphasized"}, {"italics"},
 	{"underlined"}, {"fixed"}, {"cursor"}, {"standout"},
 	{"scrollbar"} /* "scrollbar" must be last */
 };
@@ -123,6 +123,13 @@ RESULT	ex_color(xinf)
 	{
 		i = QTY(colortbl) - 1;
 	}
+	else if (!CHARncmp(args, toCHAR("italic "), 7))
+	{
+		/* allow "italic" as an alias for "italics" (with an 's') */
+		for (i = 0; colortbl[i].name[0] != 'i'; i++)
+		{
+		}
+	}
 	else
 	{
 		for (i = 0;
@@ -254,11 +261,7 @@ RESULT	ex_comment(xinf)
 	}
 	else if (xinf->command == EX_GOTO && xinf->fromaddr)
 	{
-		if (xinf->fromoffset > markoffset(xinf->fromaddr)
-		 && xinf->fromoffset <= markoffset(xinf->toaddr))
-			xinf->newcurs = markalloc(markbuffer(xinf->fromaddr), xinf->fromoffset);
-		else
-			xinf->newcurs = markdup(xinf->fromaddr);
+		xinf->newcurs = markdup(xinf->fromaddr);
 	}
 	return RESULT_COMPLETE;
 }
@@ -283,7 +286,6 @@ RESULT	ex_display(xinf)
 	{
 		return RESULT_ERROR;
 	}
-	xinf->window->di->logic = DRAW_CHANGED;
 	return RESULT_COMPLETE;
 }
 
@@ -297,7 +299,7 @@ RESULT	ex_gui(xinf)
 		return RESULT_ERROR;
 	}
 
-	return (*gui->guicmd)(xinf->window ? xinf->window->gw : NULL, tochar8(xinf->rhs))
+	return (*gui->guicmd)(xinf->window->gw, tochar8(xinf->rhs))
 		? RESULT_COMPLETE
 		: RESULT_ERROR;
 }
@@ -483,29 +485,12 @@ RESULT	ex_if(xinf)
 RESULT	ex_then(xinf)
 	EXINFO	*xinf;
 {
-	BOOLEAN origthen;
-	RESULT	result;
-
-	origthen = thenflag;
-	result = RESULT_COMPLETE;
-
 	/* Execute commands, if "thenflag" is set appropriately */
 	if (xinf->command == EX_THEN ? thenflag : !thenflag)
 	{
-		result = exstring(xinf->window, xinf->rhs);
+		exstring(xinf->window, xinf->rhs);
 	}
-
-	/* If the command line was enclosed in { .... } (as indicated by the
-	 * presence of a newline; without curlies the command couldn't possibly
-	 * contain any newlines) then restore the "thenflag" to its original
-	 * value.
-	 */
-	if (CHARchr(xinf->rhs, '\n'))
-	{
-		thenflag = origthen;
-	}
-
-	return result;
+	return RESULT_COMPLETE;
 }
 
 
@@ -649,18 +634,9 @@ MissingRHS:
 RESULT	ex_version(xinf)
 	EXINFO	*xinf;
 {
-	msg(MSG_INFO, "[s]elvis $1", VERSION);
-#ifdef COPY1
-	msg(MSG_INFO, "[s]$1", COPY1);
-#endif
-#ifdef COPY2
-	msg(MSG_INFO, "[s]$1", COPY2);
-#endif
-#ifdef COPY3
-	msg(MSG_INFO, "[s]$1", COPY3);
-#endif
-#ifdef COPY4
-	msg(MSG_INFO, "[s]$1", COPY4);
+	msg(MSG_INFO, "[s]$1", VERSION);
+#ifdef COPYING
+	msg(MSG_INFO, "[s]$1", COPYING);
 #endif
 #ifdef PORTEDBY
 	msg(MSG_INFO, "[s]$1", PORTEDBY);
