@@ -1,7 +1,7 @@
 /* exedit.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_exedit[] = "$Id: exedit.c,v 2.36 1996/10/01 19:46:29 steve Exp $";
+char id_exedit[] = "$Id: exedit.c,v 2.30 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
@@ -72,11 +72,8 @@ RESULT	ex_global(xinf)
 	EXINFO	*xinf;
 {
 	CHAR	*cp;
-	long	lenln, endln;
+	MARK	endln;
 	RESULT	ret = RESULT_COMPLETE;
-	MARK	cursor;
-	MARK	orig;
-	MARKBUF	thisln;
 
 	assert(xinf->command == EX_GLOBAL || xinf->command == EX_VGLOBAL);
 
@@ -98,61 +95,44 @@ RESULT	ex_global(xinf)
 	if (xinf->bang)
 		xinf->command = EX_VGLOBAL;
 
-	/* remember the cursor's original position.  Inside the following loop,
-	 * we'll force the cursor onto each matching line; when we're done,
-	 * we'll move the cursor back and return the final position so the
-	 * experform() function can move the cursor there in the conventional
-	 * way -- that will be important if we switch buffers.
-	 */
-	if (xinf->window->state->acton)
-		cursor = xinf->window->state->acton->cursor;
-	else
-		cursor = xinf->window->cursor;
-	if (markbuffer(cursor) != markbuffer(xinf->fromaddr))
-	{
-		marksetbuffer(cursor, markbuffer(xinf->fromaddr));
-		marksetoffset(cursor, markoffset(xinf->fromaddr));
-	}
-	orig = markdup(cursor);
-
 	/* for each line... */
 	(void)scanalloc(&cp, xinf->fromaddr);
 	ret = RESULT_COMPLETE;
 	while (cp && markoffset(xinf->fromaddr) < markoffset(xinf->toaddr))
 	{
 		/* find the end of this line */
-		for (lenln = 0; cp && *cp != '\n'; lenln++)
+		while (cp && *cp != '\n')
 			(void)scannext(&cp);
 		if (cp)
-		{
 			(void)scannext(&cp);
-			lenln++;
-		}
-
-		/* move "fromaddr" to end of this line (start of next line) */
-		thisln = *xinf->fromaddr;
-		marksetoffset(xinf->fromaddr,
-		    cp ? markoffset(scanmark(&cp)) : markoffset(xinf->toaddr));
 
 		/* is this a selected line? */
-		if ((regexec(xinf->re, &thisln, True) ? EX_GLOBAL : EX_VGLOBAL)
+		if ((regexec(xinf->re, xinf->fromaddr, True) ? EX_GLOBAL : EX_VGLOBAL)
 			== xinf->command)
 		{
-
-			/* move the cursor to the matching line */
-			endln = markoffset(scanmark(&cp));
-			marksetoffset(cursor, endln - lenln);
+			/* set a DYNAMIC mark at the end of the line */
+			endln = markdup(cp ? scanmark(&cp) : xinf->toaddr);
 
 			/* free the scan pointer -- can't make changes
 			 * while scanning.
 			 */
 			scanfree(&cp);
 
-			/* execute the command */
-			ret = exstring(xinf->window, xinf->rhs);
+			/* set the window's selection to contain the line */
+			xinf->window->seltop = markdup(xinf->fromaddr);
+			xinf->window->selbottom = markdup(endln);
+			markaddoffset(xinf->window->selbottom, -1);
+			xinf->window->seltype = 'l';
 
-			/* reallocate the scan pointer */
-			(void)scanalloc(&cp, xinf->fromaddr);
+			/* execute the command (this has the side-effect of
+			 * freeing the selection marks after they've been used)
+			 */
+			ret = exstring(xinf->window, xinf->rhs);
+			assert(!xinf->window->seltop && !xinf->window->selbottom);
+
+			/* reallocate the scan pointer, free dynamic mark */
+			(void)scanalloc(&cp, endln);
+			markfree(endln);
 
 			/* if the ex command failed, then exit */
 			if (ret != RESULT_COMPLETE)
@@ -162,18 +142,13 @@ RESULT	ex_global(xinf)
 		/* if user wants to abort operation, then exit */
 		if (guipoll(False))
 			break;
+
+		/* move "fromaddr" to end of this line; start of next line */
+		marksetoffset(xinf->fromaddr,
+		    cp ? markoffset(scanmark(&cp)) : markoffset(xinf->toaddr));
 	}
 	scanfree(&cp);
 
-	/* move the cursor back to its original position, and then return
-	 * the final position so the cursor will be moved there in a graceful
-	 * way.
-	 */
-	xinf->newcurs = markdup(cursor);
-	if (markbuffer(cursor) != markbuffer(orig))
-		marksetbuffer(cursor, markbuffer(orig));
-	marksetoffset(cursor, markoffset(orig));
-	markfree(orig);
 	return ret;
 }
 
@@ -187,15 +162,11 @@ RESULT	ex_join(xinf)
 	long	nspaces;	/* number of spaces to insert */
 	MARK	start, end;	/* region around a newline */
 	long	offset;		/* position of last change */
-	CHAR	*endlist;	/* string of sentence ending punctuation */
- static CHAR	spaces[3] = {' ', ' ', ' '};
+ static CHAR	spaces[2] = {' ', ' '};
 
 	/* initialize "offset" just to silence a compiler warning */
 	offset = 0;
 
-	/* initialize endlist from options (if set) or literals */
-	endlist = (o_sentenceend ? o_sentenceend : toCHAR(".!?"));
-
 	/* We're going to be replacing newlines with blanks.  The number of
 	 * newlines we want to replace is equal to the number lines affected
 	 * minus one... except that if the user requested a "join" of a single
@@ -222,8 +193,8 @@ RESULT	ex_join(xinf)
 				/* figure out how many spaces to insert */
 				if (xinf->bang || *cp == ')' || isspace(prevchar))
 					nspaces = 0;
-				else if (CHARchr(toCHAR(endlist), prevchar))
-					nspaces = o_sentencegap;
+				else if (CHARchr(toCHAR(".?!"), prevchar))
+					nspaces = 2;
 				else
 					nspaces = 1;
 
@@ -246,10 +217,7 @@ RESULT	ex_join(xinf)
 				 * with a given number of spaces.
 				 */
 				bufreplace(start, end, spaces, nspaces);
-				marksetoffset(end, offset + nspaces - 1);
-					/* NOTE: the "- 1" is to compensate for
-					 * the scannext() at the top of the loop
-					 */
+				marksetoffset(end, markoffset(start) + nspaces);
 
 				/* resume scanning */
 				scanalloc(&cp, end);
@@ -269,7 +237,7 @@ RESULT	ex_join(xinf)
 			/* remember the character.  When we hit a newline,
 			 * the previous character will be checked to determine
 			 * how many spaces to insert.  Note that we ignore
-			 * quote and parenthesis characters except after a
+			 * quote * and parenthesis characters except after a
 			 * blank.
 			 */
 			prevchar = *cp;
@@ -299,17 +267,9 @@ RESULT	ex_move(xinf)
 	}
 
 	/* copy the text */
-	xinf->newcurs = markdup(xinf->destaddr);
-	markaddoffset(xinf->newcurs, 1);
 	oldto = markoffset(xinf->toaddr);
 	bufpaste(xinf->destaddr, xinf->fromaddr, xinf->toaddr);
 
-	/* leave the cursor on the last line of the destination */
-	if (markoffset(xinf->newcurs) > 1)
-		markaddoffset(xinf->newcurs, -2);
-	marksetoffset(xinf->newcurs, markoffset(
-		(*dmnormal.move)(xinf->window, xinf->newcurs, 0L, 0L, False)));
-
 	/* If moving (not copying) then delete source */
 	if (xinf->command == EX_MOVE)
 	{
@@ -325,6 +285,14 @@ RESULT	ex_move(xinf)
 		}
 
 		bufreplace(xinf->fromaddr, xinf->toaddr, NULL, 0);
+
+		/* leave the cursor at the start of the destination */
+		xinf->newcurs = markdup(xinf->destaddr);
+	}
+	else /* :copy or :to */
+	{
+		/* leave the cursor at the start of the source */
+		xinf->newcurs = markdup(xinf->fromaddr);
 	}
 
 	return RESULT_COMPLETE;
@@ -372,15 +340,8 @@ RESULT	ex_print(xinf)
 RESULT	ex_put(xinf)
 	EXINFO	*xinf;
 {
-	MARK	newcurs;
-
-	newcurs = cutput(xinf->cutbuf, xinf->window, xinf->fromaddr, (BOOLEAN)(xinf->from > 0), False, False);
-	if (newcurs)
-	{
-		xinf->newcurs = markdup(newcurs);
-		return RESULT_COMPLETE;
-	}
-	return RESULT_ERROR;
+	xinf->newcurs = markdup(cutput(xinf->cutbuf, xinf->window, xinf->fromaddr, (BOOLEAN)(xinf->from > 0), False, False));
+	return RESULT_COMPLETE;
 }
 
 
@@ -531,6 +492,7 @@ RESULT	ex_substitute(xinf)
 	long	cursoff;/* offset where cursor should be moved to */
  static PFLAG	pflag;	/* printing flag */
  static BOOLEAN	optg;	/* boolean option: substitute globally in line? */
+ static BOOLEAN	optc;	/* boolean option: confirm before subst? */
  static BOOLEAN	optx;	/* boolean option: execute instead of substitute? */
  static long	count;	/* numeric option: which instance in each line to sub */
 	MARKBUF	posn;	/* position within a line to be replaced */
@@ -566,6 +528,7 @@ RESULT	ex_substitute(xinf)
 		if (xinf->bang)
 		{
 			pflag= PF_NONE;
+			optc = False;
 		}
 	}
 	else /* xinf->command == CMD_SUBSTITUTE */
@@ -577,7 +540,7 @@ RESULT	ex_substitute(xinf)
 		if (!o_edcompatible)
 		{
 			pflag = xinf->pflag;
-			optg = optx = False;
+			optg = optc = optx = False;
 			count = 0;
 		}
 		for (opt = xinf->rhs; opt && *opt; opt++)
@@ -588,6 +551,10 @@ RESULT	ex_substitute(xinf)
 				optg = (BOOLEAN)!optg;
 				break;
 
+			  case 'c':
+				optc = (BOOLEAN)!optc;
+				break;
+
 			  case 'x':
 				optx = (BOOLEAN)!optx;
 				break;
@@ -621,13 +588,6 @@ RESULT	ex_substitute(xinf)
 				} while (isdigit(*opt));
 				opt--;
 				break;
-
-			  default:
-				if (!isspace(*opt))
-				{
-					msg(MSG_ERROR, "[C]unsupported flag '$1'", *opt);
-					return RESULT_ERROR;
-				}
 			}
 		}
 
@@ -638,15 +598,10 @@ RESULT	ex_substitute(xinf)
 			return RESULT_ERROR;
 		}
 
-		/* default behavior is to either replace only first instance,
-		 * or all instances, depending on the "gdefault" option.
-		 */
+		/* default behavior is to replace only first instance */
 		if (count == 0 && !optg)
 		{
-			if (o_gdefault && !o_edcompatible)
-				optg = True;
-			else
-				count = 1;
+			count = 1;
 		}
 	}
 
@@ -680,7 +635,7 @@ RESULT	ex_substitute(xinf)
 				/* Either execute the replacement, or perform
 				 * the substitution
 				 */
-				opt = regsub(xinf->re, newp, (BOOLEAN)!optx);
+				opt = regsub(xinf->re, newp, !optx);
 				if (!opt)
 				{
 					return RESULT_ERROR;
