@@ -3,11 +3,11 @@
 
 
 
+#define guibeep(w)		if (gui && gui->beep != NULL) (*gui->beep)((w)->gw); else (void)0
 #define guiscrollbar(w,t,b,n)	if (gui->scrollbar != NULL) \
 					(*scrollbar)((w)->gw, t, b, n); else (void)0
 #define guiflush()		if (gui->flush) (*gui->flush)()
 
-BEGIN_EXTERNC
 extern void	guimoveto P_((WINDOW win, int column, int row));
 extern void	guidraw P_((WINDOW win, _char_ font, CHAR *text, int len));
 extern BOOLEAN	guishift P_((WINDOW win, int qty, int rows));
@@ -15,8 +15,6 @@ extern BOOLEAN	guiscroll P_((WINDOW win, int qty, BOOLEAN notlast));
 extern void	guiclrtoeol P_((WINDOW win));
 extern void	guireset P_((void));
 extern BOOLEAN	guipoll P_((BOOLEAN reset));
-extern void	guibeep P_((WINDOW win));
-END_EXTERNC
 
 extern GUI	*gui;
 
@@ -40,7 +38,3 @@ extern GUI	guitermcap;
 extern GUI	guiopen;
 extern GUI	guiquit;
 #endif
-
-#ifdef GUI_WIN32
-extern GUI	guiwin32;
-#endif
