@@ -64,7 +64,7 @@ typedef struct twin_s
 	char	startfixed[30], endfixed[30];
 	char	startbold[30], endbold[30];
 	char	startemph[30], endemph[30];
-	char	startitalic[30], enditalic[30];
+	char	startitalics[30], enditalics[30];
 	char	startunderline[30], endunderline[30];
 	char	starthilite[30], endhilite[30]; /* regardless of font */
 } TWIN;
@@ -738,7 +738,7 @@ static BOOLEAN ansi_color(tw, font, fgname, bgname)
 	  case 'f':	build = tw->startfixed;		break;
 	  case 'b':	build = tw->startbold;		break;
 	  case 'e':	build = tw->startemph;		break;
-	  case 'i':	build = tw->startitalic;	break;
+	  case 'i':	build = tw->startitalics;	break;
 	  case 'u':	build = tw->startunderline;	break;
 	  default:	build = tw->starthilite;	break;
 	}
@@ -788,8 +788,8 @@ static BOOLEAN ansi_color(tw, font, fgname, bgname)
 				/* emphasized is same as bold */
 				strcpy(tw->startemph, tw->startbold);
 
-				/* italic is a dim version of normal */
-				sprintf(tw->startitalic, "\033[0;%d;%dm", 30 + fg, 40 + bg);
+				/* italics is a dim version of normal */
+				sprintf(tw->startitalics, "\033[0;%d;%dm", 30 + fg, 40 + bg);
 
 				/* underline is dim underlined version of normal */
 				sprintf(tw->startunderline, "\033[0;%d;%dm%s",
@@ -816,8 +816,8 @@ static BOOLEAN ansi_color(tw, font, fgname, bgname)
 				/* emphasized is same as bold */
 				strcpy(tw->startemph, tw->startbold);
 
-				/* italic is a dim version of normal */
-				sprintf(tw->startitalic, "\033[0;%dm", 30 + fg);
+				/* italics is a dim version of normal */
+				sprintf(tw->startitalics, "\033[0;%dm", 30 + fg);
 
 				/* underline is dim underlined version of normal */
 				sprintf(tw->startunderline, "\033[0;%dm%s",
@@ -837,7 +837,7 @@ static BOOLEAN ansi_color(tw, font, fgname, bgname)
 		/* copy the normal string to each font's endXXX string */
 		strcpy(tw->endbold, tw->startnormal);
 		strcpy(tw->endemph, tw->startnormal);
-		strcpy(tw->enditalic, tw->startnormal);
+		strcpy(tw->enditalics, tw->startnormal);
 		*tw->endunderline = '\0';
 			if (UE) strcpy(tw->endunderline, UE);
 			strcat(tw->endunderline, tw->startnormal);
@@ -1299,7 +1299,7 @@ fprintf(stderr, "draw(\"%.*s\"), tw->cursy=%d, tw->height=%d, physy=%d, afterscr
 		{
 		  case 'b':	startf = tw->startbold; endf = tw->endbold; break;
 		  case 'e':	startf = tw->startemph; endf = tw->endemph; break;
-		  case 'i':	startf = tw->startitalic; endf = tw->enditalic; break;
+		  case 'i':	startf = tw->startitalics; endf = tw->enditalics; break;
 		  case 'u':	startf = tw->startunderline; endf = tw->endunderline; break;
 		  case 'f':	startf = tw->startfixed; endf = tw->endfixed; break;
 			/* 'g' is handled separately, above */
@@ -1752,8 +1752,8 @@ static BOOLEAN creategw(name, attributes)
 		strcpy(newp->startnormal, defcolors.startnormal);
 		strcpy(newp->startbold, defcolors.startbold);
 		strcpy(newp->endbold, defcolors.endbold);
-		strcpy(newp->startitalic, defcolors.startitalic);
-		strcpy(newp->enditalic, defcolors.enditalic);
+		strcpy(newp->startitalics, defcolors.startitalics);
+		strcpy(newp->enditalics, defcolors.enditalics);
 		strcpy(newp->startunderline, defcolors.startunderline);
 		strcpy(newp->endunderline, defcolors.endunderline);
 		strcpy(newp->starthilite, defcolors.starthilite);
@@ -1764,7 +1764,7 @@ static BOOLEAN creategw(name, attributes)
 		/* set the initial font-switch strings from termcap strings */
 		if (MD) strcpy(newp->startbold, MD), strcpy(newp->endbold, ME);
 		if (US) strcpy(newp->startunderline, US), strcpy(newp->endunderline, UE);
-		if (MH) strcpy(newp->startitalic, MH), strcpy(newp->enditalic, ME);
+		if (MH) strcpy(newp->startitalics, MH), strcpy(newp->enditalics, ME);
 		if (SO) strcpy(newp->starthilite, SO), strcpy(newp->endhilite, SE);
 	}
 	strcpy(newp->startfixed, defcolors.startfixed);
@@ -2020,8 +2020,8 @@ static BOOLEAN color(gw, font, fg, bg)
 			strcpy(other->endbold, tw->endbold);
 			strcpy(other->startemph, tw->startemph);
 			strcpy(other->endemph, tw->endemph);
-			strcpy(other->startitalic, tw->startitalic);
-			strcpy(other->enditalic, tw->enditalic);
+			strcpy(other->startitalics, tw->startitalics);
+			strcpy(other->enditalics, tw->enditalics);
 			strcpy(other->startunderline, tw->startunderline);
 			strcpy(other->endunderline, tw->endunderline);
 			strcpy(other->starthilite, tw->starthilite);
@@ -2141,12 +2141,6 @@ static RESULT stop(alwaysfork)
 {
 	RESULT	result;
 
-	/* move the cursor to the bottom of the screen, and scroll up */
-	tputs(tgoto(CM, 0, (int)o_ttyrows - 1), 0, ttych);
-	ttych('\n');
-	reset();
-	ttyflush();
-
 	/* if we want to fork, then the default processing is good enough */
 	if (alwaysfork)
 		return RESULT_MORE;
