@@ -1,7 +1,7 @@
 /* input.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_input[] = "$Id: input.c,v 2.46 1996/09/21 05:26:35 steve Exp $";
+char id_input[] = "$Id: input.c,v 2.35 1996/05/27 07:58:16 steve Exp $";
 
 #include "elvis.h"
 
@@ -45,7 +45,6 @@ typedef struct
 #if USE_PROTOTYPES
 static void cleanup(WINDOW win, BOOLEAN del, BOOLEAN backsp, BOOLEAN yank);
 static void addchar(MARK cursor, MARK top, MARK bottom, INPUTINFO *info);
-static BOOLEAN tryabbr(WINDOW win, _CHAR_ nextc);
 static RESULT perform(WINDOW win);
 static RESULT parse(_CHAR_ key, void *info);
 static ELVCURSOR shape(WINDOW win);
@@ -75,12 +74,6 @@ static void cleanup(win, del, backsp, yank)
 	}
 	assert(markoffset(win->state->cursor) == markoffset(win->state->bottom));
 
-	/* save the newly input text in the "previous input" buffer */
-	if (yank)
-	{
-		cutyank('.', win->state->top, win->state->bottom, 'c', False);
-	}
-
 	/* move the cursor back one character, unless it is already at the
 	 * start of a line.
 	 */
@@ -91,8 +84,12 @@ static void cleanup(win, del, backsp, yank)
 		{
 			markaddoffset(win->state->cursor, 1);
 		}
-		marksetoffset(win->state->top, markoffset(win->state->cursor));
-		marksetoffset(win->state->bottom, markoffset(win->state->cursor));
+	}
+
+	/* save the newly input text in the "previous input" buffer */
+	if (yank)
+	{
+		cutyank('.', win->state->top, win->state->bottom, 'c', False);
 	}
 
 	/* Force the screen to be regenerated */
@@ -136,71 +133,6 @@ static void addchar(cursor, top, bottom, info)
 	}
 }
 
-
-/* This function attempts to expand an abbreviation at the cursor location,
- * if there is one.  If so, it deletes the short form, and pushes the long
- * form an following character back onto the type-ahead buffer.  Else it
- * returns False.
- */
-static BOOLEAN tryabbr(win, nextc)
-	WINDOW	win;	/* window where abbreviation may need expanding */
-	_CHAR_	nextc;	/* character after the word */
-{
-	MARKBUF	from, to;
-	CHAR	*cp, *build;
-	long	slen, llen;
-	CHAR	cbuf[1];
-
-	/* Try to do an abbreviation.  To do this, we collect
-	 * characters backward to the preceding whitespace.  We
-	 * go to the preceding whitespace because abbreviations
-	 * can't contain whitespace; we know we'll never need
-	 * more characters to recognize an abbreviation.  We
-	 * collect the characters backwards just because it is
-	 * easier.
-	 */
-	for (scanalloc(&cp, win->state->cursor), build = NULL;
-	     cp && scanprev(&cp) && !isspace(*cp);
-	     )
-	{
-		buildCHAR(&build, *cp);
-	}
-	scanfree(&cp);
-	if (build)
-	{
-		cp = mapabbr(build, &slen, &llen, (BOOLEAN)(win->state->acton != NULL));
-		if (cp)
-		{
-			/* yes, we have an abbreviation! */
-
-			/* delete the short form */
-			cleanup(win, True, False, False);
-			(void)marktmp(from, markbuffer(win->state->cursor), markoffset(win->state->cursor) - slen);
-			(void)marktmp(to, markbuffer(win->state->cursor), markoffset(win->state->cursor));
-			bufreplace(&from, &to, NULL, 0);
-
-			/* stuff the long form, and the user's
-			 * non-alnum character, into the queue
-			 */
-			if (nextc)
-			{
-				cbuf[0] = nextc;
-				mapunget(cbuf, 1, False);
-			}
-			mapunget(cp, (int)llen, False);
-
-			/* move cursor to where word goes */
-			marksetoffset(win->state->bottom, markoffset(&from));
-			marksetoffset(win->state->cursor, markoffset(&from));
-			safefree(build);
-			return True;
-		}
-		safefree(build);
-	}
-	return False;
-}
-
-
 /* This function performs an input-mode command.  Usually, this will be a
  * character to insert/replace in the text.
  */
@@ -212,7 +144,7 @@ static RESULT perform(win)
 	MARK	  cursor = state->cursor;
 	MARK	  tmp;
 	MARKBUF	  from, to;
-	CHAR	  *cp;
+	CHAR	  *cp, *build;
 	char	  *littlecp, ch;
 	EXINFO	  xinfb;
 	VIINFO	  vinfb;
@@ -265,8 +197,45 @@ static RESULT perform(win)
 		 && ii->arg != '_'
 		 && (state->acton == NULL || o_inputtab(markbuffer(cursor)) == 'f'))
 		{
-			if (tryabbr(win, ii->arg))
-				break;
+			/* Try to do an abbreviation.  To do this, we collect
+			 * characters backward to the preceding whitespace.  We
+			 * go to the preceding whitespace because abbreviations
+			 * can't contain whitespace; we know we'll never need
+			 * more characters to recognize an abbreviation.  We
+			 * collect the characters backwards just because it is
+			 * easier.
+			 */
+			for (scanalloc(&cp, cursor), build = NULL;
+			     cp && scanprev(&cp) && !isspace(*cp);
+			     )
+			{
+				buildCHAR(&build, *cp);
+			}
+			scanfree(&cp);
+			if (build)
+			{
+				cp = mapabbr(build, &col, &len, (BOOLEAN)(state->acton != NULL));
+				if (cp)
+				{
+					cleanup(win, True, False, False);
+					(void)marktmp(from, markbuffer(cursor), markoffset(cursor) - col);
+					(void)marktmp(to, markbuffer(cursor), markoffset(cursor));
+#if 0
+					bufreplace(&from, &to, cp, len);
+					marksetoffset(state->bottom, markoffset(&from) + len);
+					marksetoffset(cursor, markoffset(&from) + len);
+#else
+					bufreplace(&from, &to, NULL, 0);
+					mapunget(&ii->arg, 1, False);
+					mapunget(cp, (int)len, False);
+					marksetoffset(state->bottom, markoffset(&from));
+					marksetoffset(cursor, markoffset(&from));
+					safefree(build);
+					break;
+#endif
+				}
+				safefree(build);
+			}
 		}
 		/* fall through... */
 
@@ -282,8 +251,8 @@ static RESULT perform(win)
 		if (!isspace(ii->arg)
 		 && cursor == win->cursor
 		 && win->md->wordwrap
-		 && o_textwidth(markbuffer(cursor)) > 0
-		 && dispmark2col(win) >= o_textwidth(markbuffer(cursor)))
+		 && o_linelength(markbuffer(cursor)) > 0
+		 && dispmark2col(win) >= o_linelength(markbuffer(cursor)))
 		{
 			/* Figure out where the current word started */
 			for (scanalloc(&cp, cursor);
@@ -355,96 +324,89 @@ static RESULT perform(win)
 
 	  case INP_NEWLINE:
 		cleanup(win, True, False, False);
-		if (!tryabbr(win, '\n'))
-		{
-			ii->arg = '\n';
-			ii->cmd = INP_NORMAL;
-			perform(win);
-			dispindent(win, cursor, -1);
-		}
+		ii->arg = '\n';
+		ii->cmd = INP_NORMAL;
+		perform(win);
+		dispindent(win, cursor, -1);
 		ii->backsp = ii->prev = '\0';
 		break;
 
 	  case INP_TAB:
-		if (!tryabbr(win, '\n'))
+		switch (o_inputtab(markbuffer(cursor)))
 		{
-			switch (o_inputtab(markbuffer(cursor)))
+		  case 't':
+			/* insert a tab */
+			addchar(cursor, state->top, state->bottom, ii);
+			break;
+
+		  case 's':
+			/* insert enough spaces to look like a tab */
+			col = dispmark2col(win);
+			ii->arg = ' ';
+			do
 			{
-			  case 't':
-				/* insert a tab */
 				addchar(cursor, state->top, state->bottom, ii);
+			} while ((++col) % o_tabstop(markbuffer(cursor)) != 0);
+			break;
+
+		  case 'f':
+			/* filename completion */
+
+			/* if at start of input, then fail */
+			if (markoffset(cursor) == markoffset(state->top))
+			{
+				guibeep(win);
 				break;
-
-			  case 's':
-				/* insert enough spaces to look like a tab */
-				col = dispmark2col(win);
-				ii->arg = ' ';
-				do
-				{
-					addchar(cursor, state->top, state->bottom, ii);
-				} while ((++col) % o_tabstop(markbuffer(cursor)) != 0);
-				break;
-
-			  case 'f':
-				/* filename completion */
-
-				/* if at start of input, then fail */
-				if (markoffset(cursor) == markoffset(state->top))
-				{
-					guibeep(win);
-					break;
-				}
-
-				/* locate the previous character */
-				tmp = markdup(cursor);
-				markaddoffset(tmp, -1);
-
-				/* if previous is whitespace, then fail */
-				if (isspace(scanchar(tmp)) || scanchar(tmp) == '(')
-				{
-					markfree(tmp);
-					guibeep(win);
-					break;
-				}
-
-				/* collect the partial name into char array */
-				littlecp = &name.partial[QTY(name.partial)];
-				*--littlecp = '\0';
-				ch = scanchar(tmp);
-				do
-				{
-					*--littlecp = ch;
-					markaddoffset(tmp, -1);
-					ch = (markoffset(tmp) >= markoffset(state->top)) ? scanchar(tmp) : ' ';
-				} while (!isspace(ch) && ch != '(');
-				markaddoffset(tmp, 1);
-
-				/* try to expand the filename */
-				littlecp = iofilename(littlecp, (ch == '(') ? ')' : '\t');
-				if (!littlecp)
-				{
-					markfree(tmp);
-					guibeep(win);
-					break;
-				}
-
-				/* name found -- replace old word with expanded name */
-				for (cp = name.full, col = 0;
-				     (*cp++ = *littlecp++) != '\0'; /* yes, ASSIGNMENT! */
-				     col++)
-				{
-				}
-				bufreplace(tmp, win->state->bottom, name.full, col);
-				marksetoffset(cursor, markoffset(tmp) + col);
-				marksetoffset(win->state->bottom, markoffset(cursor));
-				markfree(tmp);
 			}
+
+			/* locate the previous character */
+			tmp = markdup(cursor);
+			markaddoffset(tmp, -1);
+
+			/* if previous is whitespace, then fail */
+			if (isspace(scanchar(tmp)) || scanchar(tmp) == '(')
+			{
+				markfree(tmp);
+				guibeep(win);
+				break;
+			}
+
+			/* collect the partial name into char array */
+			littlecp = &name.partial[QTY(name.partial)];
+			*--littlecp = '\0';
+			ch = scanchar(tmp);
+			do
+			{
+				*--littlecp = ch;
+				markaddoffset(tmp, -1);
+				ch = (markoffset(tmp) >= markoffset(state->top)) ? scanchar(tmp) : ' ';
+			} while (!isspace(ch) && ch != '(');
+			markaddoffset(tmp, 1);
+
+			/* try to expand the filename */
+			littlecp = iofilename(littlecp, (ch == '(') ? ')' : '\t');
+			if (!littlecp)
+			{
+				markfree(tmp);
+				guibeep(win);
+				break;
+			}
+
+			/* name found -- replace old word with expanded name */
+			for (cp = name.full, col = 0;
+			     (*cp++ = *littlecp++) != '\0'; /* yes, ASSIGNMENT! */
+			     col++)
+			{
+			}
+			bufreplace(tmp, cursor, name.full, col);
+			marksetoffset(cursor, markoffset(tmp) + col);
+			markfree(tmp);
 		}
 		ii->backsp = ii->prev = '\0';
 		break;
 
 	  case INP_ONEVI:
-		cleanup(win, True, False, True);
+		cleanup(win, False, False, True);
 		vipush(win, ELVIS_ONCE, NULL);
 		ii->backsp = ii->prev = '\0';
 		ii->setbottom = True;
@@ -464,7 +426,7 @@ static RESULT perform(win)
 			     scannext(&cp))
 			{
 			}
-			if (cp && *cp == '\n')
+			if (cp)
 			{
 				to = *scanmark(&cp);
 				scanfree(&cp);
@@ -589,12 +551,8 @@ static RESULT perform(win)
 		xinfb.bang = True;
 
 		/* execute the command */
-		len = o_bufchars(markbuffer(cursor)) - markoffset(cursor);
-		assert(len >= 0);
 		(void)ex_shift(&xinfb);
 		ii->backsp = ii->prev = '\0';
-		assert(o_bufchars(markbuffer(cursor)) >= len);
-		marksetoffset(cursor, o_bufchars(markbuffer(cursor)) - len);
 		break;
 
 	  case INP_EXPOSE:
@@ -628,9 +586,6 @@ static RESULT perform(win)
 
 	}
 
-	/* set wantcol to the cursor's current column */
-	win->wantcol = dispmark2col(win);
-
 	/* prepare for next command */
 	ii->cmd = INP_NORMAL;
 
@@ -796,7 +751,6 @@ void inputpush(win, flags, mode)
 	_char_		mode;	/* 'R' for replace, or anything else for insert */
 {
 	/* push the state */
-	flags |= ELVIS_REGION;
 	statepush(win, flags);
 
 	/* initialize the state */
