@@ -1,7 +1,7 @@
 /* lowbuf.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_lowbuf[] = "$Id: lowbuf.c,v 2.22 1996/09/21 03:50:57 steve Exp $";
+char id_lowbuf[] = "$Id: lowbuf.c,v 2.17 1996/05/23 00:19:50 steve Exp $";
 
 /* This file contains functions which divide the session file into several
  * buffers.  These functions use session.c, and are used by buffer.c
@@ -9,10 +9,6 @@ char id_lowbuf[] = "$Id: lowbuf.c,v 2.22 1996/09/21 03:50:57 steve Exp $";
 
 #include "elvis.h"
 
-/* These control the sizes of some tiny caches for storing line offsets and
- * info about random offsets.  You can disable these by making them 0
- */
-#define LINECACHE	4
 
 #if USE_PROTOTYPES
 static short checksum(BLK *blk);
@@ -21,10 +17,6 @@ static BLKNO insblock(_BLKNO_ blklist, _LBLKNO_ before, _BLKNO_ chars, _COUNT_ n
 static BLKNO lockchars(_BLKNO_ bufinfo, _LBLKNO_ lblkno, _BLKNO_ blkno);
 static void unlockchars(_BLKNO_ bufinfo, _LBLKNO_ lblkno, _BLKNO_ blkno, int chgchars, int chglines);
 static void helpinit(_BLKNO_ bufinfo, void (*bufproc)(_BLKNO_ bufinfo, long nchars, long nlines, long changes, long prevloc, CHAR *name));
-
-# if LINECACHE
-static void clobbercache(_BLKNO_ dst);
-# endif
 #endif
 
 /******************************************************************************/
@@ -47,15 +39,7 @@ static short checksum(blk)
 	     scan != blk->sumshorts;
 	     )
 	{
-		/* We know that o_blksize is a power of 2, >= 256.  We can
-		 * safely assume that sizeof(short) is also a power of 2, <=16,
-		 * so we can safely unroll this loop.
-		 */
-		sum += scan[-1] + scan[-2] + scan[-3] + scan[-4]
-			+ scan[-5] + scan[-6] + scan[-7] + scan[-8]
-			+ scan[-9] + scan[-10] + scan[-11] + scan[-12]
-			+ scan[-13] + scan[-14] + scan[-15] + scan[-16];
-		scan -= 16;
+		sum += *--scan;
 	}
 
 	/* restore the old sum, but then return the new sum */
@@ -708,29 +692,6 @@ void lowfree(bufinfo)
 
 /******************************************************************************/
 
-#if LINECACHE
-static struct
-{
-	BLKNO	bufinfo;/* bufinfo of a known line, or 0 for none */
-	long	lineno;	/* line number */
-	long	offset;	/* offset of the start of the line */
-} linecache[LINECACHE];
-static int lineidx;	/* index into linecache[] of slot to recycle */
-#endif /* LINECACHE */
-
-
-#if LINECACHE
-static void clobbercache(dst)
-	_BLKNO_	dst;	/* bufinfo of a changed buffer */
-{
-	register int	i;
-
-	for (i = 0; i < LINECACHE; i++)
-		if (linecache[i].bufinfo == dst)
-			linecache[i].bufinfo = 0;
-}
-#endif /* LINECACHE */
-
 /* convert a line number to an offset, for a given buffer */
 long lowline(bufinfo, lineno)
 	_BLKNO_	bufinfo;	/* BUFINFO of the buffer */
@@ -741,19 +702,6 @@ long lowline(bufinfo, lineno)
 	BLKNO	next;	/* the next blklist block, after this one */
 	long	offset;	/* total offset seen */
 	int	i;	/* used for scanning through blklist.blk[] */
-#if LINECACHE
-	long	origline;/* a copy of lineno */
-
-	/* check the cache first */
-	for (i = 0; i < LINECACHE; i++)
-	{
-		if (linecache[i].bufinfo == bufinfo && linecache[i].lineno == lineno)
-		{
-			return linecache[i].offset;
-		}
-	}
-	origline = lineno;
-#endif /* LINECACHE */
 
 	/* outside this function, line numbers start with 1, but in here they start at 0 */
 	lineno--;
@@ -798,14 +746,6 @@ long lowline(bufinfo, lineno)
 					}
 				}
 
-#if LINECACHE
-				/* stuff information into the cache */
-				linecache[lineidx].bufinfo = bufinfo;
-				linecache[lineidx].offset = offset;
-				linecache[lineidx].lineno = origline;
-				lineidx = (lineidx + 1) % LINECACHE;
-#endif /* LINECACHE */
-
 				/* return the offset */
 				sesunlock(next, False);
 				return offset;
@@ -857,12 +797,12 @@ BLKNO lowoffset(bufinfo, offset, left, right, lptr, linenum)
 	LBLKNO	*lptr;		/* output: logical block number of CHARS block */
 	long	*linenum;	/* output: line number */
 {
+	BLK	*blk;	/* contents of a bufinfo block or blklist block */
 	BLKNO	blklist;/* used for stepping from one blklist to next */
 	BLKNO	next;	/* the next blklist block, after this one */
 	LBLKNO	lblkno;	/* counts overall LBLKNO of the given offset */
 	long	lnum;	/* counts newlines */
-	register int	i;	/* used for scanning through blklist.blk[] */
-	register BLK	*blk;	/* contents of a bufinfo or blklist block */
+	int	i;	/* used for scanning through blklist.blk[] */
 
 	/* treat negative offsets as 0 */
 	if (offset < 0)
@@ -952,10 +892,10 @@ BLKNO lowoffset(bufinfo, offset, left, right, lptr, linenum)
  * the number of lines in the buffer (i.e., the number of newline characters
  * in the "new" string.
  */
-long lowinsert(dst, dsttop, newp, newlen)
+long lowinsert(dst, dsttop, new, newlen)
 	_BLKNO_	dst;	/* BUFINFO of lowbuf */
 	long	dsttop;	/* offset where insertion should begin */
-register CHAR	*newp;	/* new text to be inserted */
+register CHAR	*new;	/* new text to be inserted */
 	long	newlen;	/* length of new text */
 {
 	BLKNO	blkno;
@@ -974,10 +914,6 @@ register int	i;
 
 	safeinspect();
 
-#if LINECACHE
-	clobbercache(dst);
-#endif
-
 	/* figure out where insertion should begin */
 	blkno = lowoffset(dst, dsttop, &left, &right, &lblkno, (long *)0);
 
@@ -998,8 +934,8 @@ register int	i;
 		nlines = 0;
 		for (i = 0; i < newlen; i++)
 		{
-			cblk->chars.chars[i + left] = newp[i];
-			if (newp[i] == '\n') nlines++;
+			cblk->chars.chars[i + left] = new[i];
+			if (new[i] == '\n') nlines++;
 		}
 
 		/* unlock that block, and update its nchars and nlines */
@@ -1045,8 +981,8 @@ register int	i;
 			     i < o_blkfill && newlen > 0;
 			     i++, j++, newlen--)
 			{
-				cblk->chars.chars[i] = *newp;
-				if (*newp++ == '\n')
+				cblk->chars.chars[i] = *new;
+				if (*new++ == '\n')
 				{
 					nlines++;
 					totlines++;
@@ -1077,8 +1013,8 @@ register int	i;
 			nlines = 0;
 			for (i = 0; i < o_blkfill && newlen > 0; i++, newlen--)
 			{
-				newblk->chars.chars[i] = *newp;
-				if (*newp++ == '\n')
+				newblk->chars.chars[i] = *new;
+				if (*new++ == '\n')
 				{
 					nlines++;
 				}
@@ -1130,10 +1066,6 @@ long lowdelete(dst, dsttop, dstbottom)
 
 	safeinspect();
 
-#if LINECACHE
-	clobbercache(dst);
-#endif
-
 	/* lock the bufinfo block, and fetch it */
 	(void)seslock(dst, True, SES_BUFINFO);
 	biblk = sesblk(dst);
@@ -1279,17 +1211,17 @@ long lowdelete(dst, dsttop, dstbottom)
 /* Replace one chunk of text with another, and return the change in the
  * number of lines.
  */
-long lowreplace(dst, dsttop, dstbottom, newp, newlen)
+long lowreplace(dst, dsttop, dstbottom, new, newlen)
 	_BLKNO_	dst;		/* BUFINFO of the lowbuf */
 	long	dsttop;		/* offset where replacement begins */
 	long	dstbottom;	/* offset of end of old text */
-	CHAR	*newp;		/* new text */
+	CHAR	*new;		/* new text */
 	long	newlen;		/* length of new text */
 {
 	long	totlines = 0;	/* total change in line count */
 
 	totlines += lowdelete(dst, dsttop, dstbottom);
-	totlines += lowinsert(dst, dsttop, newp, newlen);
+	totlines += lowinsert(dst, dsttop, new, newlen);
 	return totlines;
 }
 
@@ -1312,11 +1244,7 @@ long lowpaste(dst, dsttop, src, srctop, srcbottom)
 	safeinspect();
 
 	/* destination can't be in the middle of the source */
-	assert(src != dst || dsttop <= srctop || dsttop >= srcbottom);
-
-#if LINECACHE
-	clobbercache(dst);
-#endif
+	assert(src != dst || dsttop <= srctop || dsttop > srcbottom);
 
 	/* Small problem: If the destination is in the same buffer as the source,
 	 * and happens to be located before the source in the same block, then
