@@ -3,7 +3,6 @@
 
 
 /* Create a new buffer in the session file */
-BEGIN_EXTERNC
 extern void	lowinit P_((void (*bufproc)(_BLKNO_ bufinfo, long nchars, long nlines, long changes, long prevloc, CHAR *name)));
 extern BLKNO	lowalloc P_((char *name));
 extern BLKNO	lowdup P_((_BLKNO_ originfo));
@@ -11,8 +10,7 @@ extern void	lowfree P_((_BLKNO_ bufinfo));
 extern long	lowline P_((_BLKNO_ bufinfo, long lineno));
 extern BLKNO	lowoffset P_((_BLKNO_ bufinfo, long offset, COUNT *left, COUNT *right, LBLKNO *lptr, long *linenum));
 extern long	lowdelete P_((_BLKNO_ dst, long dsttop, long dstbottom));
-extern long	lowinsert P_((_BLKNO_ dst, long dsttop, CHAR *newp, long newlen));
-extern long	lowreplace P_((_BLKNO_ dst, long dsttop, long dstbottom, CHAR *newp, long newlen));
+extern long	lowinsert P_((_BLKNO_ dst, long dsttop, CHAR *new, long newlen));
+extern long	lowreplace P_((_BLKNO_ dst, long dsttop, long dstbottom, CHAR *new, long newlen));
 extern long	lowpaste P_((_BLKNO_ dst, long dsttop, _BLKNO_ src, long srctop, long srcbottom));
 extern void	lowflush P_((_BLKNO_ bufinfo));
-END_EXTERNC
