@@ -1,7 +1,7 @@
 /* map.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_map[] = "$Id: map.c,v 2.26 1996/09/21 00:01:46 steve Exp $";
+char id_map[] = "$Id: map.c,v 2.25 1996/05/27 07:51:26 steve Exp $";
 
 #include "elvis.h"
 
@@ -20,7 +20,6 @@ typedef struct _map
 	short		rawlen;	/* length of the "rawin" characters */
 	short		cooklen;/* length of the "cooked" characters */
 	MAPFLAGS	flags;	/* various flags */
-	BOOLEAN		invoked;/* has the map been used lately? */
 } MAP;
 
 static MAP	*maps;	/* the map table */
@@ -445,10 +444,6 @@ MAPSTATE mapdo(keys, nkeys)
 		if (qty == 0)
 		{
 			assert(resolved == 0);
-			for (scan = maps; scan; scan = scan->next)
-			{
-				scan->invoked = False;
-			}
 			return MAP_CLEAR;
 		}
 
@@ -510,24 +505,6 @@ MAPSTATE mapdo(keys, nkeys)
 		/* if any complete map, then apply it */
 		if (match)
 		{
-			/* detect animation macros, so we can update the screen
-			 * while they run.  Note that we bypass this for key
-			 * maps, since an autorepeated arrow key shouldn't make
-			 * us update the screen for each line scrolled.
-			 */
-			if (!match->label && o_optimize)
-			{
-				if (!match->invoked)
-				{
-					match->invoked = True;
-				}
-				else
-				{
-					drawimage(windefault);
-					guiflush();
-				}
-			}
-
 			/* maybe show the the map queue before */
 			if ((match->flags & MAP_BREAK) && o_maptrace == 'r')
 				o_maptrace = 's';
