@@ -19,9 +19,7 @@ typedef enum
  * into verbose message via the "elvis messages" buffer, and then calls the
  * GUI's msg() function with the expanded text.
  */
-BEGIN_EXTERNC
 extern void msg P_((MSGIMP imp, char *terse, ...));
 
 /* This function flushes queued message to the screen */
 extern void msgflush P_((void));
-END_EXTERNC
