@@ -1,7 +1,7 @@
 /* move.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_move[] = "$Id: move.c,v 2.39 1996/09/10 16:51:38 steve Exp $";
+char id_move[] = "$Id: move.c,v 2.35 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
@@ -303,7 +303,7 @@ RESULT m_absolute(win, vinf)
 			 * I'm slightly careful here to avoid overflowing
 			 * the long int which stores the offset.
 			 */
-			if (o_bufchars(buf) > 1000000L)
+			if (o_bufchars(buf) > 1000000)
 			{
 				marksetoffset(win->state->cursor,
 					(o_bufchars(buf) / 100) * vinf->count);
@@ -705,11 +705,11 @@ RESULT	m_scroll(win, vinf)
 	  case ELVCTRL('Y'):
 		win->di->topline = markoffset((*win->md->move)(win, marktmp(tmp, markbuffer(win->cursor), win->di->topline), -vinf->count, 0, True));
 		win->di->bottomline = markoffset((*win->md->move)(win, marktmp(tmp, markbuffer(win->cursor), win->di->bottomline), -vinf->count, 0, True));
-		marksetoffset(win->cursor, markoffset(dispmove(win, 0, win->wantcol)));
+		marksetoffset(win->cursor, markoffset(dispmove(win, 0, 0)));
 		if (markoffset(win->cursor) >= win->di->bottomline)
 		{
 			marksetoffset(win->cursor, win->di->bottomline);
-			marksetoffset(win->cursor, markoffset(dispmove(win, -1, win->wantcol)));
+			marksetoffset(win->cursor, markoffset(dispmove(win, -1, 0)));
 		}
 		break;
 
@@ -719,7 +719,6 @@ RESULT	m_scroll(win, vinf)
 		if (markoffset(win->cursor) < win->di->topline)
 		{
 			marksetoffset(win->cursor, win->di->topline);
-			marksetoffset(win->cursor, markoffset(dispmove(win, 0, win->wantcol)));
 		}
 		break;
 
@@ -1292,16 +1291,11 @@ RESULT m_fsentence(win, vinf)
 	VIINFO	*vinf;	/* information about the command */
 {
 	BOOLEAN	ending;	/* have we seen at least one sentence ender? */
-	BOOLEAN	didpara;/* between paragraph and first sentence in paragraph */
 	int	spaces;	/* number of spaces seen so far */
 	CHAR	*cp;	/* used for scanning through text */
 	CHAR	*end;	/* characters that end a sentence */
 	CHAR	*quote;	/* quote/parenthesis character that may appear at end */
-	CHAR	oper;	/* operator command, or '\0' */
 	long	newline;/* offset of first newline in trailing whitespace */
-	long	para;
-	long	offset;
-	long	count;
 
 	DEFAULT(1);
 
@@ -1309,108 +1303,22 @@ RESULT m_fsentence(win, vinf)
 	end = o_sentenceend ? o_sentenceend : toCHAR(".?!");
 	quote = o_sentencequote ? o_sentencequote : toCHAR("\")]");
 
-	count = vinf->count;
-	oper = vinf->oper;
-
-	/* detect whether we're at the start of a paragraph */
-	offset = markoffset(win->state->cursor);
-	scanalloc(&cp, win->state->cursor);
-	if (*cp != '\n')
-		scanprev(&cp);
-	else
-		while (cp && *cp == '\n')
-		{
-			scanprev(&cp);
-		}
-	if (cp)
-	{
-		marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
-		vinf->count = 1;
-		vinf->command = '}';
-		m_fsection(win, vinf);
-		para = markoffset(win->state->cursor);
-	}
-	else
-	{
-		para = 0;
-	}
-	marksetoffset(win->state->cursor, offset);
-	if (para == offset)
-		count++;
-	else if (para < offset)
-	{
-		/* find the next paragraph */
-		marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
-		vinf->count = 1;
-		vinf->command = '}';
-		m_fsection(win, vinf);
-		para = markoffset(win->state->cursor);
-	}
-
 	/* for each count... */
 	newline = -1;
-	scanseek(&cp, win->state->cursor);
-	for (; cp && count > 0; count--)
+	scanalloc(&cp, win->state->cursor);
+	for (; cp && vinf->count > 0; vinf->count--)
 	{
 		/* for each character in the sentence... */
-		for (ending = didpara = False, spaces = 0;
+		for (ending = False, spaces = 0;
 		     cp && (!ending || spaces < o_sentencegap || isspace(*cp));
-		     scannext(&cp), offset++)
+		     scannext(&cp))
 		{
-			/* if paragraph, then... */
-			if (offset == para)
-			{
-				/* if still more sentences to skip... */
-				if (count > 1)
-				{
-					/* count this as a sentence, and
-					 * arrange for next line to also be a
-					 * sentence.
-					 */
-					count--;
-					newline = -1;
-					ending = True;
-					if (*cp == '\n')
-					{
-						didpara = False;
-						spaces = o_sentencegap;
-					}
-					else
-					{
-						didpara = True;
-						spaces = 0;
-					}
-
-					/* oh, and we need to find the next
-					 * paragraph, too.
-					 */
-					marksetoffset(win->state->cursor, offset);
-					vinf->count = 1;
-					vinf->command = '}';/*{*/
-					if (m_fsection(win, vinf) == RESULT_COMPLETE)
-						para = markoffset(win->state->cursor);
-					else
-						para = -1;
-
-					continue;
-				}
-				else
-				{
-					/* we're here! */
-					break;
-				}
-			}
-
-			/* check the character */
 			if (*cp == '\n')
 			{
 				spaces = o_sentencegap;
-				didpara = False;
 				if (newline < 0)
 					newline = markoffset(scanmark(&cp));
 			}
-			else if (didpara)
-				/* skip characters in a ".P" line */;
 			else if (isspace(*cp))
 				spaces++;
 			else if (CHARchr(end, *cp))
@@ -1429,7 +1337,7 @@ RESULT m_fsentence(win, vinf)
 		 * trailing whitespace, then move the cursor to that newline;
 		 * else move the cursor to the start of the following sentence.
 		 */
-		if (oper && newline >= 0)
+		if (vinf->oper && newline >= 0)
 			marksetoffset(win->state->cursor, newline);
 		else
 			marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
@@ -1444,15 +1352,10 @@ RESULT m_bsentence(win, vinf)
 {
 	BOOLEAN	first;	/* True until we pass some mid-sentence stuff */
 	BOOLEAN	ending;	/* have we seen at least one sentence ender? */
-	BOOLEAN	anynext;/* any text seen on following line */
-	BOOLEAN	anythis;/* any text seen on this line */
 	int	spaces;	/* number of spaces seen so far */
 	CHAR	*cp;	/* used for scanning through text */
 	CHAR	*end;	/* characters that end a sentence */
 	CHAR	*quote;	/* quote/parenthesis character that may appear at end */
-	long	count;	/* sentences to move over */
-	long	para;	/* top of current paragraph */
-	long	offset;
 	RESULT	result;
 
 	DEFAULT(1);
@@ -1461,10 +1364,6 @@ RESULT m_bsentence(win, vinf)
 	end = o_sentenceend ? o_sentenceend : toCHAR(".?!");
 	quote = o_sentencequote ? o_sentencequote : toCHAR("\")]");
 
-	/* misc initialization */
-	anynext = anythis = False;
-	offset = markoffset(win->state->cursor);
-
 	/* NOTE: The "first" variable is used to handle the situation where
 	 * we start at the beginning of one sentence.  From here, we want
 	 * to go back one extra sentence-end; otherwise <(> would just move
@@ -1472,126 +1371,46 @@ RESULT m_bsentence(win, vinf)
 	 */
 	first = True;
 
-	/* Start scanning at the cursor location */
+	/* for each count... */
 	scanalloc(&cp, win->state->cursor);
-	count = vinf->count;
-
-	/* Find the start of this paragraph.  That counts as a "sentence" */
-	vinf->command = '{';
-	vinf->count = 1;
-	para = (m_bsection(win, vinf) == RESULT_COMPLETE) ? markoffset(win->state->cursor) : -1;
-
-	/* For each count... */
-	for (; cp && count > 0; count--)
+	for (; cp && vinf->count > 0; vinf->count--)
 	{
 		/* for each character in the sentence... */
-		for (ending = True, anythis = anynext = False, spaces = 0,
-			scanprev(&cp), offset = markoffset(scanmark(&cp));
-		     cp && offset != para &&
-			(!ending || spaces<o_sentencegap || !CHARchr(end,*cp));
-		     scanprev(&cp), offset--)
+		for (ending = True, spaces = 0, scanprev(&cp);
+		     cp && (!ending || spaces < o_sentencegap || !CHARchr(end, *cp));
+		     scanprev(&cp))
 		{
 			if (*cp == '\n')
-			{
-				spaces = o_sentencegap;
-				ending = True;
-				anynext = anythis;
-				anythis = False;
-			}
+				spaces = o_sentencegap, ending = True;
 			else if (isspace(*cp))
-			{
-				spaces++,
-				ending = True;
-			}
+				spaces++, ending = True;
 			else if (!CHARchr(quote, *cp))
-			{
-				first = ending = False;
-				anythis = True;
-				spaces = 0;
-			}
-			else
-			{
-				anythis = True;
-			}
-		}
-
-		/* If this sentence is actually a paragraph start, and we
-		 * still have more sentences to move over, then find the
-		 * next higher paragraph.
-		 */
-		if (offset == para && count > 1)
-		{
-			vinf->count = 1;
-			para = (m_bsection(win, vinf) == RESULT_COMPLETE) ? markoffset(win->state->cursor) : -1;
+				first = ending = False, spaces = 0;
 		}
 
 		/* If this sentence ender was encountered before any
 		 * mid-sentence text (i.e., if we started at the front of
 		 * a sentence) then we should go back one extra sentence-end.
 		 */
-		if (first && offset != para)
+		if (first)
 		{
-			count++;
+			vinf->count++;
 		}
 	}
 
-	/* If we hit a paragraph top which occurs on a blank line, then we
-	 * need to do a little extra processing because we exited the loop
-	 * a little too soon.
-	 */
-	if (offset == para && cp && *cp == '\n')
-	{
-		anynext = anythis;
-	}
-
 	/* did we find it? */
 	if (cp)
 	{
 		/* move the cursor to the start of the next sentence */
-		if (offset > para)
+		do
 		{
-			/* found a sentence end -- move the cursor to the to
-			 * start of the following sentence.
-			 */
-			do
-			{
-				scannext(&cp);
-				assert(cp);
-			} while (isspace(*cp) || CHARchr(quote, *cp));
-			marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
-		}
-		else if (anynext)
-		{
-			/* bumped into a paragraph start after scanning some
-			 * sentence text -- move the cursor to the first text
-			 * on the next non-blank line.
-			 */
-			while (cp && *cp != '\n')
-			{
-				scannext(&cp);
-			}
-			while (cp && isspace(*cp))
-			{
-				scannext(&cp);
-			}
-			if (&cp)
-				marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
-			else
-			{
-				marksetoffset(win->state->cursor, o_bufchars(markbuffer(win->state->cursor)) - 2);
-				if (markoffset(win->state->cursor) < 0)
-					marksetoffset(win->state->cursor, 0);
-			}
-		}
-		else
-		{
-			/* found a paragraph start -- leave the cursor there */
-			offset = para;
-			marksetoffset(win->state->cursor, para);
-		}
+			scannext(&cp);
+			assert(cp);
+		} while (isspace(*cp) || CHARchr(quote, *cp));
+		marksetoffset(win->state->cursor, markoffset(scanmark(&cp)));
 		result = RESULT_COMPLETE;
 	}
-	else if (count == 0)
+	else if (vinf->count == 0)
 	{
 		/* move the cursor to the first character of the buffer */
 		marksetoffset(win->state->cursor, 0);
