@@ -16,7 +16,5 @@
 #endif
 
 #ifdef NEED_STRDUP
-BEGIN_EXTERNC
 extern char *strdup P_((const char *str));
-END_EXTERNC
 #endif
