@@ -1,7 +1,7 @@
 /* options.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_options[] = "$Id: options.c,v 2.31 1996/08/09 20:13:45 steve Exp $";
+char id_options[] = "$Id: options.c,v 2.28 1996/05/30 17:39:04 steve Exp $";
 
 /* This file contains functions which manipulate options.
  *
@@ -620,13 +620,6 @@ BOOLEAN optputstr(name, value)
 				continue;
 			}
 
-			/* if the option is locked, then fail */
-			if (dom->val[i].flags & OPT_LOCK)
-			{
-				msg(MSG_ERROR, "[S]$1 is locked", name);
-				return False;
-			}
-
 			/* convert it */
 			ret = True;
 			if (dom->desc[i].isvalid) /* non-boolean? */
@@ -647,13 +640,6 @@ BOOLEAN optputstr(name, value)
 
 			/* set the "set" flag */
 			dom->val[i].flags |= OPT_SET;
-
-			/* if the "redraw" flag is set, then force redraw */
-			if ((dom->val[i].flags & OPT_REDRAW) != 0 && windefault)
-			{
-				windefault->di->logic = DRAW_CHANGED;
-			}
-
 			return ret;
 		}
 	}
@@ -923,7 +909,7 @@ void optsave(custom)
 	OPTDOMAIN *dom;
 	int	  i, j;
 	CHAR	  *str;
-	char	  *tmp;
+	char	  tmp[80];
 
 	/* for each domain of options... */
 	for (dom = head; dom; dom = dom->next)
@@ -945,11 +931,10 @@ void optsave(custom)
 				/* then add it to the custom buffer */
 				if (dom->desc[i].asstring)
 				{
-					str = (*dom->desc[i].asstring)(&dom->desc[i], &dom->val[i]);
-					tmp = safealloc(7 + strlen(dom->desc[i].longname) + 2 * CHARlen(str), sizeof(char));
 					strcpy(tmp, "set ");
 					strcat(tmp, dom->desc[i].longname);
 					strcat(tmp, "=");
+					str = (*dom->desc[i].asstring)(&dom->desc[i], &dom->val[i]);
 					for (j = strlen(tmp); *str; )
 					{
 						if (*str == ' ' || *str == '\t' || *str == '|' || *str == '\\')
@@ -963,13 +948,11 @@ void optsave(custom)
 				}
 				else
 				{
-					tmp = safealloc(8 + strlen(dom->desc[i].longname), sizeof(char));
 					sprintf(tmp, "set %s%s\n",
 						dom->val[i].value.boolean ? "" : "no",
 						dom->desc[i].longname);
 				}
 				bufreplace(marktmp(m, custom, o_bufchars(custom)), &m, toCHAR(tmp), (long)strlen(tmp));
-				safefree((void *)tmp);
 			}
 		}
 	}
