@@ -1,6 +1,6 @@
 /* oswin32/tcaphelp.c */
 
-char id_tcaphelp[] = "$Id: tcaphelp.c,v 2.18 1996/09/18 20:37:17 steve Exp $";
+char id_tcaphelp[] = "$Id: tcaphelp.c,v 2.12 1996/05/10 09:49:22 steve Exp $";
 
 #include "elvis.h"
 #if defined(GUI_TERMCAP) || defined(GUI_OPEN)
@@ -31,17 +31,13 @@ char id_tcaphelp[] = "$Id: tcaphelp.c,v 2.18 1996/09/18 20:37:17 steve Exp $";
 /* This variable is defined in guitcap.c */
 extern long ttycaught;
 
-static void catchsig(int signo);
-
 /* This variable is used to indicate that the BIOS interface is being used */
-static BOOLEAN	useconsole = True;
+static BOOLEAN	useconsole;
 static HANDLE	inConsole, outConsole, myConsole, console;
 static CONSOLE_SCREEN_BUFFER_INFO consinfo;
 static DWORD	inMode, outMode;
 static int	prevWidth, prevHeight;
 static BOOLEAN	resized;
-static WORD	origattr;
-static WORD	attr;	/* attribute byte for writing subsequent text */
 
 /* The codepage option stores the code page */
 static OPTVAL win32val[1];
@@ -64,25 +60,15 @@ static int setcp(OPTDESC *opt, OPTVAL *val, CHAR *newval)
 }
 static OPTDESC win32desc[] =
 {
-	{"codepage", "cpg",	optnstring,	setcp}
+	{"codepage", "cp",	optnstring,	setcp}
 };
 
 
-/* This function catches signals, especially SIGINT */
-static void catchsig(signo)
-	int	signo;
-{
-	ttycaught |= (1 << signo);
-}
-
-
 /* remember the original terminal settings */
 void ttyinit()
 {
 	SECURITY_ATTRIBUTES	sec;
 	COORD			size;
-	COORD			home;
-	DWORD			dummy;
 
 	/* get handles of the console */
 	inConsole = GetStdHandle(STD_INPUT_HANDLE);
@@ -116,15 +102,6 @@ void ttyinit()
 	prevWidth = size.X = consinfo.srWindow.Right - consinfo.srWindow.Left + 1;
 	prevHeight = size.Y = consinfo.srWindow.Bottom - consinfo.srWindow.Top + 1;
 	SetConsoleScreenBufferSize(myConsole, size);
-
-	/* make the default colors of the new console buffer be the same as the
-	 * old console buffer.
-	 */
-	attr = origattr = consinfo.wAttributes;
-	SetConsoleTextAttribute(myConsole, attr);
-	home.X = home.Y = 0;
-	FillConsoleOutputAttribute(myConsole, attr, size.X * size.Y, home, &dummy);
-	FillConsoleOutputCharacter(myConsole, ' ', size.X * size.Y, home, &dummy);
 }
 
 
@@ -153,9 +130,6 @@ void ttyraw(erasekey)
 		/* switch to binary mode */
 		_setmode(1, _O_BINARY);
 	}
-
-	/* trap ^C signals */
-	signal(SIGINT, catchsig);
 }
 
 
@@ -167,6 +141,10 @@ void ttynormal()
 		/* switch back to the original mode */
 		SetConsoleMode(inConsole, inMode);
 		SetConsoleMode(outConsole, outMode);
+#if 0
+		SetConsoleTextAttribute(outConsole,
+			FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
+#endif
 	}
 	else
 	{
@@ -188,6 +166,15 @@ static void switchcsbi(HANDLE which)
 	console = which;
 }
 
+/* This function catches signals, especially SIGINT */
+static void catchsig(int signo);
+static void catchsig(signo)
+	int	signo;
+{
+	ttycaught |= (1 << signo);
+}
+
+
 
 /* Read from keyboard, with timeout.  Since Win32 apparently can't read with
  * timeout, this function simulates a timeout immediately if timeouts are
@@ -414,8 +401,8 @@ void ttywrite(buf, len)
  static int	   arg[9];	/* arguments to escape sequence */
  static int	   argno = -1;	/* -1 normally, else index of escape sequence argument */
  static COORD	   coord;	/* coordinates of cursor position */
- static BOOLEAN	   bgset = False;/* has the background color been set? */
- static BOOLEAN	   boldset = False;/* has the foreground brightness been set? */
+ static WORD	   attr = 7;	/* attribute byte for writing subsequent text */
+ static BOOLEAN	   rvid;	/* in reverse-video mode? */
 	SMALL_RECT rect;	/* source for inserting/deleting/scrolling */
 	COORD	   dest;	/* destination for inserting/deletingi/scrolling */
 	CHAR_INFO  ci;		/* fill info */
@@ -425,7 +412,7 @@ void ttywrite(buf, len)
 	long	   dummy;
 
 	/* if not using a WIN32 console, then just write the characters to stdout */
-	if (!useconsole || gui != &guitermcap)
+	if (!useconsole)
 	{
 		_write(1, buf, len);
 		return;
@@ -715,30 +702,23 @@ void ttywrite(buf, len)
 					switch (arg[j])
 					{
 					  case 0:
-						attr = origattr;
-						bgset = False;
-						boldset = False;
+						attr &= ~FOREGROUND_INTENSITY;
+						if (rvid)
+						{
+							rvid = False;
+							attr ^= FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE
+								| BACKGROUND_RED|BACKGROUND_GREEN|BACKGROUND_BLUE;
+						}
 						break;
 
 					  case 1:
-						if (attr == origattr
-						 && (attr & FOREGROUND_INTENSITY) != 0)
-							attr |= FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE;
-						else
-							attr |= FOREGROUND_INTENSITY;
-						boldset = True;
-						break;
-
-					  case 4:
-						if (!bgset)
-							attr ^= BACKGROUND_RED;
+						attr |= FOREGROUND_INTENSITY;
 						break;
 
 					  case 7:
 						attr ^= FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE
 							| BACKGROUND_RED|BACKGROUND_GREEN|BACKGROUND_BLUE;
-						if (!boldset)
-							attr &= ~FOREGROUND_INTENSITY;
+						rvid = (BOOLEAN)!rvid;
 						break;
 
 					  case 30: 
@@ -750,8 +730,6 @@ void ttywrite(buf, len)
 					  case 36:
 					  case 37:
 						attr &= ~(FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
-						if (!boldset)
-							attr &= ~FOREGROUND_INTENSITY;
 						if ((arg[j] - 30) & 1)
 							attr |= FOREGROUND_RED;
 						if ((arg[j] - 30) & 2)
@@ -775,7 +753,6 @@ void ttywrite(buf, len)
 							attr |= BACKGROUND_GREEN;
 						if ((arg[j] - 40) & 4)
 							attr |= BACKGROUND_BLUE;
-						bgset = True;
 						break;
 					}
 				}
@@ -857,8 +834,7 @@ BOOLEAN ttysize(linesptr, colsptr)
  * operation.  Here we check to see if SIGINT has been caught recently.
  * Returns True to abort an operation, or False to continue it.
  */
-BOOLEAN ttypoll(reset)
-	BOOLEAN reset;
+BOOLEAN ttypoll()
 {
 	return (BOOLEAN)((ttycaught & (1 << SIGINT)) != 0);
 }
