@@ -14,13 +14,11 @@ typedef struct regexp {
 	CHAR	program[1];	/* Unwarranted chumminess with compiler. */
 } regexp;
 
-BEGIN_EXTERNC
 extern regexp	*regcomp P_((CHAR *retext, MARK cursor));
 extern int	regexec P_((regexp *re, MARK from, BOOLEAN bol));
-extern CHAR	*regtilde P_((CHAR *newp));
-extern CHAR	*regsub P_((regexp *re, CHAR *newp, BOOLEAN doit));
+extern CHAR	*regtilde P_((CHAR *new));
+extern CHAR	*regsub P_((regexp *re, CHAR *new, BOOLEAN doit));
 extern void	regerror P_((char *errmsg));
-END_EXTERNC
 
 #ifndef REG
 # define REG /* as nothing */
