@@ -1,7 +1,7 @@
 /* vi.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_vi[] = "$Id: vi.c,v 2.52 1996/09/21 00:27:55 steve Exp $";
+char id_vi[] = "$Id: vi.c,v 2.47 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
@@ -10,7 +10,6 @@ static RESULT parse(_CHAR_ key, void *info);
 static ELVCURSOR shape(WINDOW win);
 #endif
 
-#define FIX_EMPTY	/* when command would fail on empty buffer: define=fix buffer */
 
 /* This variable stores the command which <.> is supposed to repeat.
  * If the command is supposed to switch to input mode, then those
@@ -20,7 +19,6 @@ static ELVCURSOR shape(WINDOW win);
  */
 VIINFO	dotcmd;
 BOOLEAN	dotviz;	/* was the dotcmd a visible operator? */
-long	dotlines;/* number of lines for visible operator, or 0L if char/rect */
 
 /* These are the "when" conditions in the array below */
 #define WHEN_SEL	0x0001	/* legal while text is selected */
@@ -60,7 +58,7 @@ vikeys[] =
 /* ^D  scroll dn 1/2page */	{ m_scroll,	WHEN_SEL_ONCE_OPEN,		TWEAK_FIXCOL,		"^D"	},
 /* ^E  scroll up	 */	{ m_scroll,	WHEN_SEL_ONCE,			TWEAK_FIXCOL,		"^E"	},
 /* ^F  page forward	 */	{ m_scroll,	WHEN_SEL_ONCE,			TWEAK_FRONT,		"^F"	},
-/* ^G  show file status	 */	{ m_absolute,	WHEN_SEL_ONCE_OPEN|WHEN_EMPTY,	TWEAK_NONE,		"^G"	},
+/* ^G  show file status	 */	{ m_absolute,	WHEN_SEL_ONCE|WHEN_EMPTY,	TWEAK_NONE,		"^G"	},
 /* ^H  move left, like h */	{ m_left,	WHEN_ANY,			TWEAK_NONE,		"^H"	},
 /* ^I  move to next tag  */	{ m_tag,	WHEN_ONCE_OPEN,			TWEAK_NONE,		"^I"	},
 /* ^J  move down	 */	{ m_updown,	WHEN_ANY,			TWEAK_IGNCOL_INCL_LINE,	"^J"	},
@@ -73,20 +71,20 @@ vikeys[] =
 /* ^Q  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE,		"^Q"	},
 /* ^R  redo change	 */	{ v_undo,	WHEN_ONCE_OPEN|WHEN_EMPTY,	TWEAK_NONE,		"^R"	},
 /* ^S  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE,		"^S"	},
-/* ^T  pop tagstack	 */	{ v_tag,	WHEN_OPEN|WHEN_EMPTY,		TWEAK_NONE,		"^T"	},
-/* ^U  scroll up 1/2page */	{ m_scroll,	WHEN_SEL_ONCE_OPEN,		TWEAK_FIXCOL,		"^U"	},
+/* ^T  pop tagstack	 */	{ v_tag,	WHEN_NEVER|WHEN_EMPTY,		TWEAK_NONE,		"^T"	},
+/* ^U  scroll up 1/2page */	{ m_scroll,	WHEN_SEL_ONCE,			TWEAK_FIXCOL,		"^U"	},
 /* ^V  start visible	 */	{ v_visible,	WHEN_SEL_ONCE,			TWEAK_LINE,		"^V"	},
 /* ^W  window operations */	{ v_window,	WHEN_SEL_ONCE_OPEN_HIST|WHEN_EMPTY,TWEAK_NONE,		"^W"	},
-/* ^X  move to phys col	 */	{ m_column,	WHEN_SEL_ONCE_OPEN_HIST|WHEN_MOVE,TWEAK_IGNCOL_MARK,	"^X"	},
+/* ^X  move to phys col	 */	{ m_column,	WHEN_SEL_ONCE_HIST_MOVE,	TWEAK_IGNCOL_MARK,	"^X"	},
 /* ^Y  scroll down	 */	{ m_scroll,	WHEN_SEL_ONCE,			TWEAK_FIXCOL,		"^Y"	},
 /* ^Z  not defined	 */	{ v_notex,	WHEN_ANY,			TWEAK_NONE,		"^Z"	},
 /* ESC end visible mark	 */	{ v_visible,	WHEN_SEL,			TWEAK_NONE,		"^obra"	},
 /* ^\  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE,		"^bksl"	},
-/* ^]  keyword is tag	 */	{ v_tag,	WHEN_ONCE_OPEN,			TWEAK_NONE,		"^cbra"	},
-/* ^^  previous file	 */	{ v_notex,	WHEN_ONCE_OPEN|WHEN_EMPTY,		TWEAK_FRONT,		"^^"	},
+/* ^]  keyword is tag	 */	{ v_tag,	WHEN_ONCE,			TWEAK_NONE,		"^cbra"	},
+/* ^^  previous file	 */	{ v_notex,	WHEN_ONCE|WHEN_EMPTY,		TWEAK_FRONT,		"^^"	},
 /* ^_  move to row	 */	{ NULL,		WHEN_SEL_ONCE,			TWEAK_FIXCOL_INCL,	"^_"	},
 /* SPC move right,like l */	{ m_right,	WHEN_ANY,			TWEAK_NONE,		"l"	},
-/*  !  run thru filter	 */	{ NULL,		WHEN_NEVER,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,	"bang"	},
+/*  !  run thru filter	 */	{ NULL,		WHEN_MORE,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,	"bang"	},
 /*  "  select cut buffer */	{ NULL,		0,/* This is handled by the command parser */0,		"quote"	},
 /*  #  increment number	 */	{ v_number,	WHEN_ONCE_OPEN_HIST,		TWEAK_DOT_UNDO,		"hash"	},
 /*  $  move to rear	 */	{ m_column,	WHEN_ANY,			TWEAK_IGNCOL_INCL,	"dollar"},
@@ -95,7 +93,7 @@ vikeys[] =
 /*  '  move to a mark	 */	{ m_mark,	WHEN_SEL_ONCE_OPEN_MOVE,	TWEAK_FRONT_INCL_MARK_LINE, "apost"},
 /*  (  mv back sentence	 */	{ m_bsentence,	WHEN_ANY,			TWEAK_NONE,		"open"	},
 /*  )  mv fwd sentence	 */	{ m_fsentence,	WHEN_ANY,			TWEAK_NONE,		"close"	},
-/*  *  errlist		 */	{ v_notex,	WHEN_OPEN|WHEN_EMPTY,		TWEAK_FRONT,			},
+/*  *  errlist		 */	{ v_notex,	WHEN_NEVER|WHEN_EMPTY,		TWEAK_FRONT,			},
 /*  +  mv front next ln  */	{ m_updown,	WHEN_ANY,			TWEAK_FRONT_INCL_LINE,		},
 /*  ,  reverse [fFtT] cmd*/	{ m_csearch,	WHEN_ANY,			TWEAK_INCL,		"comma"	},
 /*  -  mv front prev ln	 */	{ m_updown,	WHEN_ANY,			TWEAK_FRONT_INCL_LINE		},
@@ -113,22 +111,22 @@ vikeys[] =
 /*  9  part of count	 */	{ NULL		/* This is handled by the command parser */			},
 /*  :  run single EX cmd */	{ v_ex,		WHEN_SEL_ONCE_OPEN_HIST|WHEN_EMPTY, TWEAK_NONE,		"colon"	},
 /*  ;  repeat [fFtT] cmd */	{ m_csearch,	WHEN_ANY,			TWEAK_INCL,		"semi"	},
-/*  <  shift text left	 */	{ NULL,		WHEN_OPEN,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,"lt"},
-/*  =  preset filter	 */	{ NULL,		WHEN_OPEN,			TWEAK_DOT_OPER_UNDO		},
-/*  >  shift text right	 */	{ NULL,		WHEN_OPEN,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,"gt"},
+/*  <  shift text left	 */	{ NULL,		WHEN_NEVER,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,"lt"},
+/*  =  preset filter	 */	{ NULL,		WHEN_NEVER,			TWEAK_DOT_OPER_UNDO		},
+/*  >  shift text right	 */	{ NULL,		WHEN_NEVER,			TWEAK_FRONT_LINE_DOT_OPER_UNDO,"gt"},
 /*  ?  backward search	 */	{ m_search,	WHEN_ANY|WHEN_MORE,		TWEAK_MARK,		"quest"	},
 /*  @  execute a cutbuf  */	{ v_at,		WHEN_SEL_ONCE_OPEN_HIST|WHEN_EMPTY,TWEAK_NONE,		"at"	},
 /*  A  append at EOL	 */	{ v_input,	WHEN_ONCE_OPEN_HIST|WHEN_EMPTY|WHEN_MORE, TWEAK_DOT_UNDO	},
 /*  B  move back Word	 */	{ m_bigword,	WHEN_ANY,			TWEAK_NONE			},
-/*  C  change to EOL	 */	{ v_notop,	WHEN_OPEN,			TWEAK_DOT_UNDO			},
-/*  D  delete to EOL	 */	{ v_notop,	WHEN_OPEN,			TWEAK_DOT_UNDO			},
+/*  C  change to EOL	 */	{ v_notop,	WHEN_NEVER,			TWEAK_DOT_UNDO			},
+/*  D  delete to EOL	 */	{ v_notop,	WHEN_NEVER,			TWEAK_DOT_UNDO			},
 /*  E  move end of Word	 */	{ m_bigword,	WHEN_ANY,			TWEAK_INCL			},
 /*  F  move bk to char	 */	{ m_csearch,	WHEN_ANY,			TWEAK_INCL			},
 /*  G  move to line #	 */	{ m_absolute,	WHEN_ANY,			TWEAK_FRONT_INCL_MARK_LINE	},
 /*  H  move to row	 */	{ m_scrnrel,	WHEN_SEL_ONCE_MOVE,		TWEAK_FRONT_INCL_LINE		},
 /*  I  insert at front	 */	{ v_input,	WHEN_ONCE_OPEN_HIST|WHEN_EMPTY|WHEN_MORE, TWEAK_DOT_UNDO	},
-/*  J  join lines	 */	{ v_notex,	WHEN_OPEN,			TWEAK_DOT_UNDO			},
-/*  K  look up keyword	 */	{ v_notex,	WHEN_OPEN,			TWEAK_NONE			},
+/*  J  join lines	 */	{ v_notex,	WHEN_NEVER,			TWEAK_DOT_UNDO			},
+/*  K  look up keyword	 */	{ v_notex,	WHEN_NEVER,			TWEAK_NONE			},
 /*  L  move to last row	 */	{ m_scrnrel,	WHEN_SEL_ONCE_MOVE,		TWEAK_FRONT_INCL_LINE		},
 /*  M  move to mid row	 */	{ m_scrnrel,	WHEN_SEL_ONCE_MOVE,		TWEAK_FRONT_INCL_LINE		},
 /*  N  reverse prev srch */	{ m_search,	WHEN_ANY,			TWEAK_MARK			},
@@ -136,13 +134,13 @@ vikeys[] =
 /*  P  paste before	 */	{ v_paste,	WHEN_ONCE_OPEN_HIST|WHEN_EMPTY,	TWEAK_DOT_UNDO			},
 /*  Q  quit to EX mode	 */	{ v_ex,		WHEN_OPEN|WHEN_EMPTY,		TWEAK_NONE			},
 /*  R  overtype		 */	{ v_input,	WHEN_ONCE_OPEN_HIST|WHEN_MORE,	TWEAK_DOT_UNDO			},
-/*  S  change line	 */	{ v_notop,	WHEN_OPEN,			TWEAK_DOT_UNDO			},
+/*  S  change line	 */	{ v_notop,	WHEN_NEVER,			TWEAK_DOT_UNDO			},
 /*  T  move bk to char	 */	{ m_csearch,	WHEN_ANY,			TWEAK_INCL			},
 /*  U  undo whole line	 */	{ v_undo,	WHEN_OPEN_HIST|WHEN_EMPTY,	TWEAK_FRONT_UNDO		},
 /*  V  start visible	 */	{ v_visible,	WHEN_SEL_ONCE,			TWEAK_LINE			},
 /*  W  move forward Word */	{ m_bigword,	WHEN_ANY,			TWEAK_NONE			},
 /*  X  delete to left	 */	{ v_delchar,	WHEN_ONCE_OPEN_HIST,		TWEAK_DOT_UNDO			},
-/*  Y  yank text	 */	{ v_notop,	WHEN_OPEN,			TWEAK_NONE			},
+/*  Y  yank text	 */	{ v_notop,	WHEN_NEVER,			TWEAK_NONE			},
 /*  Z  save file & exit	 */	{ v_quit,	WHEN_ONCE_OPEN|WHEN_EMPTY,	TWEAK_NONE			},
 /*  [  move back section */	{ m_bsection,	WHEN_ANY,			TWEAK_LINE_MARK,	"obra"	},
 /*  \  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE,		"bksl"	},
@@ -152,8 +150,8 @@ vikeys[] =
 /*  `  move to mark	 */	{ m_mark,	WHEN_SEL_ONCE_OPEN_MOVE,	TWEAK_MARK,		"grave"	},
 /*  a  append at cursor	 */	{ v_input,	WHEN_OPEN_HIST|WHEN_EMPTY|WHEN_MORE, TWEAK_DOT_UNDO		},
 /*  b  move back word	 */	{ m_word,	WHEN_ANY,			TWEAK_NONE			},
-/*  c  change text	 */	{ NULL,		WHEN_OPEN,			TWEAK_DOT_OPER_UNDO		},
-/*  d  delete op	 */	{ NULL,		WHEN_OPEN,			TWEAK_DOT_OPER_UNDO		},
+/*  c  change text	 */	{ NULL,		WHEN_NEVER,			TWEAK_DOT_OPER_UNDO		},
+/*  d  delete op	 */	{ NULL,		WHEN_NEVER,			TWEAK_DOT_OPER_UNDO		},
 /*  e  move end word	 */	{ m_word,	WHEN_ANY,			TWEAK_INCL			},
 /*  f  move fwd for char */	{ m_csearch,	WHEN_ANY,			TWEAK_INCL			},
 /*  g  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE			},
@@ -168,13 +166,13 @@ vikeys[] =
 /*  p  paste after	 */	{ v_paste,	WHEN_ONCE_OPEN_HIST,		TWEAK_DOT_UNDO			},
 /*  q  not defined	 */	{ NULL,		WHEN_NEVER,			TWEAK_NONE			},
 /*  r  replace chars	 */	{ v_delchar,	WHEN_ONCE_OPEN_HIST,		TWEAK_DOT_UNDO			},
-/*  s  subst N chars	 */	{ v_notop,	WHEN_OPEN,			TWEAK_DOT_UNDO			},
+/*  s  subst N chars	 */	{ v_notop,	WHEN_NEVER,			TWEAK_DOT_UNDO			},
 /*  t  move fwd to char	 */	{ m_csearch,	WHEN_ANY,			TWEAK_INCL			},
 /*  u  undo		 */	{ v_undo,	WHEN_ONCE_OPEN|WHEN_EMPTY,	TWEAK_NONE			},
 /*  v  start visible	 */	{ v_visible,	WHEN_SEL_ONCE,			TWEAK_NONE			},
 /*  w  move fwd word	 */	{ m_word,	WHEN_ANY,			TWEAK_NONE			},
 /*  x  delete character	 */	{ v_delchar,	WHEN_ONCE_OPEN_HIST,		TWEAK_DOT_UNDO			},
-/*  y  yank text	 */	{ NULL,		WHEN_OPEN,			TWEAK_OPER			},
+/*  y  yank text	 */	{ NULL,		WHEN_NEVER,			TWEAK_OPER			},
 /*  z  adjust scrn row	 */	{ m_z,		WHEN_SEL_ONCE,			TWEAK_FIXCOL			},
 /*  {  back paragraph	 */	{ m_bsection,	WHEN_ANY,			TWEAK_NONE,		"ocur"	},
 /*  |  move to column	 */	{ m_column,	WHEN_ANY,			TWEAK_IGNCOL_MARK,	"bar"	},
@@ -218,6 +216,13 @@ RESULT	viperform(win, vinf)
 	unsigned short flags;
 	long	tmp;
 
+	/* see if this function call is actually triggered by a previous RESULT_MORE */
+	if (state->flags & ELVIS_MORE)
+	{
+		/* state = state->pop; */
+		assert((vikeys[vinf->oper].when | vikeys[vinf->command].when) & WHEN_MORE);
+	}
+
 	/* If command is ^M and this display mode has a tagnext() function,
 	 * then pretend this is a ^] command (which calls tagatcursor() ).
 	 * This makes hypertext easier to use.
@@ -241,20 +246,11 @@ RESULT	viperform(win, vinf)
 		*vinf = dotcmd;
 
 		/* If it was a visible operator the first time, then it must
-		 * be visible now too.  Exception: if the earlier command was
-		 * applied to whole lines, then reapply it now to the same
-		 * quantity of whole lines.
+		 * be visible now too.
 		 */
 		if (dotviz && !win->seltop)
 		{
-			if (dotlines)
-			{
-				vinf->count = dotlines;
-			}
-			else
-			{
-				return RESULT_ERROR;
-			}
+			return RESULT_ERROR;
 		}
 	}
 
@@ -265,27 +261,13 @@ RESULT	viperform(win, vinf)
 	 || ((state->flags & ELVIS_BOTTOM) && !(flags & WHEN_OPEN))
 	 || (state->acton && !(flags & WHEN_HIST))
 	 || (vinf->oper && !(flags & WHEN_MOVE))
-#ifndef FIX_EMPTY
 	 || (o_bufchars(markbuffer(state->cursor)) == 0 && !(flags & WHEN_EMPTY))
-#endif
 	 || vikeys[vinf->command].func == NULL)
 	{
 		viinitcmd(vinf);
 		return RESULT_ERROR;
 	}
 
-#ifdef FIX_EMPTY
-	/* if the buffer is empty, and this command doesn't work on an empty
-	 * buffer, then stuff a newline into the buffer.
-	 */
-	if (o_bufchars(markbuffer(state->cursor)) == 0 && !(flags & WHEN_EMPTY))
-	{
-		/* bufwilldo(state->cursor); */
-		bufreplace(marktmp(from, markbuffer(state->cursor), 0L), &from, toCHAR("\n"), 1L);
-		o_modified(markbuffer(state->cursor)) = False;
-	}
-#endif
-
 	/* If we're using an operator, then combine the operator's flags
 	 * with the movement command's flags.
 	 */
@@ -335,7 +317,6 @@ RESULT	viperform(win, vinf)
 			dotcmd = *vinf;
 			dotcmd.tweak |= TWEAK_DOTTING;
 			dotviz = False;
-			dotlines = 0L;
 		}
 
 		/* call the function */
@@ -381,15 +362,6 @@ RESULT	viperform(win, vinf)
 			dotcmd = *vinf;
 			dotcmd.tweak |= TWEAK_DOTTING;
 			dotviz = True;
-			if (win->seltype == 'l' || win->seltype == 'L')
-			{
-				dotlines = markline(win->selbottom) - markline(win->seltop) + 1;
-			}
-			else
-			{
-				dotlines = 0L;
-			}
-
 		}
 
 		from = *win->seltop;
@@ -460,7 +432,7 @@ RESULT	viperform(win, vinf)
 		}
 
 		/* do the operator */
-		result = oper(win, vinf, &from, &to);
+		result = operator(win, vinf, &from, &to);
 
 		/* if we were doing visible marking before, end it now */
 		if (win->seltop)
