@@ -1,7 +1,7 @@
 /* vicmd.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_vicmd[] = "$Id: vicmd.c,v 2.44 1996/09/18 19:16:11 steve Exp $";
+char id_vicmd[] = "$Id: vicmd.c,v 2.42 1996/05/30 17:39:04 steve Exp $";
 
 #include "elvis.h"
 
@@ -307,19 +307,9 @@ RESULT v_delchar(win, vinf)
 
 	DEFAULT(1);
 
-	/* Find the endpoints of this line.  Note that we need to be careful
-	 * about the end of the line, because we don't want to allow the
-	 * newline character to be deleted.
-	 */
+	/* find the endpoints of this line */
 	front = markoffset(dispmove(win, 0, 0));
-	if (win->state->acton)
-	{
-		end = markoffset((*dmnormal.move)(win, win->state->cursor, 0, INFINITY, True));
-	}
-	else
-	{
-		end = markoffset((*win->md->move)(win, win->state->cursor, 0, INFINITY, True));
-	}
+	end = markoffset(dispmove(win, 0, INFINITY));
 
 	/* choose a starting offset */
 	if (vinf->command == 'X')
@@ -501,7 +491,10 @@ RESULT v_window(win, vinf)
 		return RESULT_COMPLETE;
 
 	  case 'S':
-		o_wrap(win) = (BOOLEAN)!o_wrap(win);
+		if (o_sidescroll(win) == 0)
+			o_sidescroll(win) = 8;
+		else
+			o_sidescroll(win) = 0;
 		win->di->logic = DRAW_CHANGED;
 		return RESULT_COMPLETE;
 
